﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Windows.Forms;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Design;

namespace CustomControls
{
    [ToolboxBitmap(typeof(System.Windows.Forms.ListView))]
    public partial class ExtendedListView : ListView
    {
        private const int WM_SCROLL = 0x115;
        private Int32 m_ColumnIndexToSortOn;
        private ImageList imageList;
        private IContainer components;
        private ExtendedListViewItemCollection m_Items;
        private ExtendedListViewItemComparator m_FourStateListViewItemComparator;
        private bool m_IsComparatorAssigned;
        private Color m_AlternatingBackColor;
        private SelectedSmallImageList m_SelectedSmallImageList;
        private VirtualModeCache m_VirtualModeCache;

        //sort indicator
        //www.csharpfriends.com/Forums/ShowPost.aspx.PostID=78715
        private const Int32 HDI_FORMAT = 0x4;
        private const Int32 HDF_SORTUP = 0x400;
        private const Int32 HDF_SORTDOWN = 0x200;
        private const Int32 LVM_GETHEADER = 0x101f;
        private const Int32 HDM_GETITEM = 0x120b;
        private const Int32 HDM_SETITEM = 0x120c;

        [System.Runtime.InteropServices.DllImport("user32.dll")]
        private static extern IntPtr SendMessage(IntPtr hWnd, uint Msg, IntPtr wParam, IntPtr lParam);

        [System.Runtime.InteropServices.DllImport("user32.dll", EntryPoint = "SendMessage")]
        private static extern IntPtr SendMessageLVCOLUMN(IntPtr hWnd, Int32 Msg, IntPtr wParam, ref LVCOLUMN lPLVCOLUMN);

        [System.Runtime.InteropServices.StructLayout(System.Runtime.InteropServices.LayoutKind.Sequential)]
        private struct LVCOLUMN
        {
            public Int32 mask;
            public Int32 cx;
            [System.Runtime.InteropServices.MarshalAs(System.Runtime.InteropServices.UnmanagedType.LPTStr)]
            public string pszText;
            public IntPtr hbm;
            public Int32 cchTextMax;
            public Int32 fmt;
            public Int32 iSubItem;
            public Int32 iImage;
            public Int32 iOrder;
        }
        //

        public delegate void SortVirtualExtendedListviewHandler(object sender, SortVirtualExtendedListviewEventArgs args);
        public event SortVirtualExtendedListviewHandler SortVirtualExtendedListview;

        public enum SelectedSmallImageList
        {
            None = 0,
            SmallImageList = 1
        }

        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(ExtendedListView));
            this.imageList = new System.Windows.Forms.ImageList(this.components);
            this.SuspendLayout();
            // 
            // imageList
            // 
            this.imageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList.ImageStream")));
            this.imageList.TransparentColor = System.Drawing.Color.Transparent;
            this.imageList.Images.SetKeyName(0, "unchecked.ico");
            this.imageList.Images.SetKeyName(1, "checked.ico");
            this.imageList.Images.SetKeyName(2, "undetermined.ico");
            this.imageList.Images.SetKeyName(3, "none.ico");
            this.ResumeLayout(false);

        }

        public ExtendedListView()
        {
            InitializeComponent();
            this.SetStyle(ControlStyles.DoubleBuffer, true);
            this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
            this.ColumnIndexToSortOn = 0;
            this.m_Items = new ExtendedListViewItemCollection(this);
            this.m_FourStateListViewItemComparator = new ExtendedListViewItemComparator();
            this.VirtualModeCache = new VirtualModeCache();
            this.VirtualModeCache.ItemAdded += new VirtualModeCache.VirtualModeCacheItemAddedHandler(this.OnVirtualModeCache_ItemAdded);
            this.View = View.Details;
            this.SelectSmallImageList = SelectedSmallImageList.SmallImageList;
            this.AlternatingBackColor = this.BackColor;
            this.RetrieveVirtualItem += new RetrieveVirtualItemEventHandler(this.OnExtendedListView_RetrieveVirtualItem);
            this.SearchForVirtualItem += new SearchForVirtualItemEventHandler(this.OnExtendedListView_SearchForVirtualItem);
            this.ColumnClick += new ColumnClickEventHandler(this.OnExtendedListView_ColumnClick);
            this.m_Items.ItemAdded += new ExtendedListViewItemCollection.ItemAddedHandler(this.OnItemAdded);
            this.m_Items.ItemRemoved += new ExtendedListViewItemCollection.ItemRemovedHandler(this.OnItemRemoved);
      
        }

        [Editor(typeof(ExtendedListViewItemCollectionEditor), typeof(UITypeEditor))]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        [Browsable(true), Category("Behavior"), Description("The items in the ListView.")]
        public new ExtendedListViewItemCollection Items
        {
            get
            {
                return this.m_Items;
            }
        }

        [Browsable(false)]
        public new IList<Int32> CheckedIndices
        {
            get
            {
                return this.GetCheckedIndices();
            }
        }

        [Browsable(false)]
        public new IList<FourStateListViewItem> CheckedItems
        {
            get
            {
                return this.GetCheckedItems();
            }
        }

        [Browsable(true), DefaultValue(typeof(Color), "Window"), Category("Appearance"), Description("Alternating line color")]
        public Color AlternatingBackColor
        {
            get
            {
                return this.m_AlternatingBackColor;
            }
            set
            {
                this.m_AlternatingBackColor = value;
                this.SetLinesBackColor();
            }
        }

        public Int32 GetIndexForColumnName(string name)
        {
            Int32 result = -1;
            for (Int32 index = 0; index < this.Columns.Count; index++)
            {
                if (this.Columns[index].Text.ToLower() == name.ToLower())
                {
                    result = index;
                    index = this.Columns.Count;
                }
            }
            return result;
        }

        [Browsable(true), DefaultValue(typeof(SelectedSmallImageList), "SmallImageList"), Category("Misc"), Description("The ImageList control used by the ListView for images in all view except for the large icon view.")]
        public SelectedSmallImageList SelectSmallImageList
        {
            get
            {
                return this.m_SelectedSmallImageList;
            }
            set
            {
                this.m_SelectedSmallImageList = value;
                this.SetSmallImageList();
            }
        }

        //hide from design view
        [Browsable(false)]
        public new ImageList SmallImageList
        {
            get
            {
                return base.SmallImageList;
            }
            set
            {
                base.SmallImageList = value;
            }
        }

        public new void Sort()
        {
            if (!this.DesignMode)
            {
                if (this.ColumnIndexToSortOn != this.m_FourStateListViewItemComparator.ColumnIndex)
                {
                    //first click on column --> set default sort settings
                    this.m_FourStateListViewItemComparator.Order = SortOrder.Ascending;
                }

                //
                this.m_FourStateListViewItemComparator.ColumnIndex = this.ColumnIndexToSortOn;

                //update column icons
                for (int index = 0; index < this.Columns.Count; index++)
                {
                    if (index == this.ColumnIndexToSortOn)
                    {
                        switch (this.m_FourStateListViewItemComparator.Order)
                        {
                            case SortOrder.Ascending:
                                this.SetSortIcon(index, SortOrder.Ascending);
                                break;
                            case SortOrder.Descending:
                                this.SetSortIcon(index, SortOrder.Descending);
                                break;
                        }
                    }
                    else
                    {
                        //this.Columns[index].ImageKey = null;
                        this.SetSortIcon(index, SortOrder.None);
                    }
                }

                //use internal sorter only if not in virtual mode
                if (!this.VirtualMode)
                {
                    //assign comparator
                    if (!this.m_IsComparatorAssigned)
                    {
                        //first request to sort
                        //assign needed comparator for sorting
                        this.ListViewItemSorter = this.m_FourStateListViewItemComparator;
                        this.m_IsComparatorAssigned = true;
                    }

                    //sort using comparator
                    base.Sort();
                }
                else
                {
                    //send event for external sorter
                    this.OnSortVirtualExtendedListview(new SortVirtualExtendedListviewEventArgs(this.m_FourStateListViewItemComparator.Order, this.ColumnIndexToSortOn));
                }
            }
        }

        [Browsable(true), DefaultValue(typeof(Int32), "0"), Category("Misc"), Description("Index of the column to sort on.")]
        public Int32 ColumnIndexToSortOn
        {
            get
            {
                return this.m_ColumnIndexToSortOn;
            }
            set
            {
                this.m_ColumnIndexToSortOn = value;
            }
        }

        //hide from design view
        [Browsable(false)]
        public VirtualModeCache VirtualModeCache
        {
            get
            {
                return this.m_VirtualModeCache;
            }
            set
            {
                this.m_VirtualModeCache = value;
            }
        }

        //sort indicator
        private void SetSortIcon(int ColumnIndex, System.Windows.Forms.SortOrder Order)
        {
            IntPtr ColumnHeader = SendMessage(this.Handle, LVM_GETHEADER, IntPtr.Zero, IntPtr.Zero);
        IntPtr ColumnPtr = new IntPtr(ColumnIndex);
            LVCOLUMN lvColumn = new LVCOLUMN();
            lvColumn.mask = HDI_FORMAT;
            SendMessageLVCOLUMN(ColumnHeader, HDM_GETITEM, ColumnPtr, ref lvColumn);

            switch (Order)
            {
                case System.Windows.Forms.SortOrder.Ascending:
                    lvColumn.fmt &= ~HDF_SORTDOWN;
                    lvColumn.fmt |= HDF_SORTUP;
                    break;
                case System.Windows.Forms.SortOrder.Descending:
                    lvColumn.fmt &= ~HDF_SORTUP;
                    lvColumn.fmt |= HDF_SORTDOWN;
                    break;
                case SortOrder.None:
                    lvColumn.fmt &= ~(HDF_SORTDOWN | HDF_SORTUP);
                    break;
            }
            SendMessageLVCOLUMN(ColumnHeader, HDM_SETITEM, ColumnPtr, ref lvColumn);
        }

        private void SetSmallImageList()
        {
            if (this.SelectSmallImageList == SelectedSmallImageList.SmallImageList)
            {
                this.SmallImageList = this.imageList;
            }
            else
            {
                this.SmallImageList = null;
                //to remove space added by imagelist it's nessecary to change the alignment to something else
                //and back to it's original value to update the listview state
                this.Alignment = ListViewAlignment.SnapToGrid;
                this.Alignment = ListViewAlignment.Left;
            }
        }

        private IList<Int32> GetCheckedIndices()
        {
            IList<Int32> result = new List<Int32>();
            for (Int32 index = 0; index < this.Items.Count; index++)
            {
                FourStateListViewItem item = (FourStateListViewItem)this.Items[index];
                if (item.Checked == FourStateListViewItem.CheckStates.Check)
                {
                    result.Add(index);
                }
            }
            return new List<Int32>(result);
        }

        private IList<FourStateListViewItem> GetCheckedItems()
        {
            IList<FourStateListViewItem> result = new List<FourStateListViewItem>();
            IEnumerator enumerator = this.Items.GetEnumerator();
            while (enumerator.MoveNext())
            {
                FourStateListViewItem item = (FourStateListViewItem)enumerator.Current;
                if (item.Checked == FourStateListViewItem.CheckStates.Check)
                {
                    result.Add(item);
                }
            }
            return new List<FourStateListViewItem>(result);
        }

        private void OnExtendedListView_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            this.ColumnIndexToSortOn = e.Column;
            if (this.ColumnIndexToSortOn == this.m_FourStateListViewItemComparator.ColumnIndex)
            {
                if (this.m_FourStateListViewItemComparator.Order == SortOrder.Ascending)
                {
                    this.m_FourStateListViewItemComparator.Order = SortOrder.Descending;
                }
                else
                {
                    this.m_FourStateListViewItemComparator.Order = SortOrder.Ascending;
                }
            }
            this.Sort();
        }

        //protected override void OnColumnClick(ColumnClickEventArgs e)
        //{
        //    base.OnColumnClick(e);
        //    this.ColumnIndexToSortOn = e.Column;
        //    if (this.ColumnIndexToSortOn == this.m_FourStateListViewItemComparator.ColumnIndex)
        //    {
        //        if (this.m_FourStateListViewItemComparator.Order == SortOrder.Ascending)
        //        {
        //            this.m_FourStateListViewItemComparator.Order = SortOrder.Descending;
        //        }
        //        else
        //        {
        //            this.m_FourStateListViewItemComparator.Order = SortOrder.Ascending;
        //        }
        //    }
        //    this.Sort();
        //}

        protected override void WndProc(ref Message m)
        {
            base.WndProc(ref m);
            if (m.Msg.Equals(WM_SCROLL))
            {
                this.OnMouseScroll(new ScrollEventArgs((ScrollEventType)m.WParam.ToInt32(), 0));
            }
        }

        protected virtual void OnSortVirtualExtendedListview(SortVirtualExtendedListviewEventArgs e)
        {
            if (this.SortVirtualExtendedListview != null)
                this.SortVirtualExtendedListview(this, e);
        }

        private void OnExtendedListView_RetrieveVirtualItem(object sender, RetrieveVirtualItemEventArgs e)
        {
            if (this.VirtualModeCache != null && this.VirtualModeCache.Count > 0 && this.VirtualModeCache.Count > e.ItemIndex && e.ItemIndex > -1)
            {
                e.Item = this.VirtualModeCache[e.ItemIndex];
            }
            else
            {
                FourStateListViewItem listViewItem = FourStateListViewItem.InitializeSubItems(this.Columns.Count);
                e.Item = listViewItem;
            }
        }

        private void OnExtendedListView_SearchForVirtualItem(object sender, SearchForVirtualItemEventArgs e)
        {
            bool found = false;
            int index = -1;
            IEnumerator<FourStateListViewItem> cacheEnumerator = this.VirtualModeCache.GetEnumerator();
            while (cacheEnumerator.MoveNext() && !found)
            {
                if (e.IncludeSubItemsInSearch)
                {
                    IEnumerator subItemEnumerator = cacheEnumerator.Current.SubItems.GetEnumerator();
                    while (subItemEnumerator.MoveNext() && !found)
                    {
                        if (((ListViewItem.ListViewSubItem)subItemEnumerator.Current).Text.ToLower().StartsWith(e.Text.ToLower()))
                        {
                            found = true;
                        }
                    }
                }
                else
                {
                    if (cacheEnumerator.Current.Text.ToLower().StartsWith(e.Text.ToLower()))
                    {
                        found = true;
                    }
                }
                index += 1;
            }

            e.Index = -1;
            if (found)
            {
                e.Index = index;
            }
        }

        private void OnVirtualModeCache_ItemAdded(object sender, EventArgs args)
        {
            this.VirtualListSize = this.VirtualModeCache.Count;
        }

        private void OnMouseScroll(ScrollEventArgs e)
        {
            if (e.Type == ScrollEventType.EndScroll)
            {
                this.Invalidate();
            }
        }

        private void OnItemAdded(object sender, ItemAddedEventArgs e)
        {
            this.SetLinesBackColor();
        }

        private void OnItemRemoved(object sender, ItemRemovedEventArgs e)
        {
            this.SetLinesBackColor();
        }

        private void SetLinesBackColor()
        {
            IEnumerator enumerator = this.Items.GetEnumerator();
            while (enumerator.MoveNext())
            {
                if ((enumerator.Current) is FourStateListViewItem)
                {
                    this.SetLineBackColor((FourStateListViewItem)enumerator.Current);
                }
                else if ((enumerator.Current) is ListViewItem)
                {
                    this.SetLineBackColor((ListViewItem)enumerator.Current);
                }
            }
        }

        private void SetLineBackColor(FourStateListViewItem fourStatelistViewItem)
        {
            Color lineColor;
            if (fourStatelistViewItem.Index % 2 == 0)
            {
                //even
                lineColor = BackColor;
            }
            else
            {
                //odd
                lineColor = AlternatingBackColor;
            }
            fourStatelistViewItem.BackColor = lineColor;
        }

        private void SetLineBackColor(ListViewItem listViewItem)
        {
            Color lineColor;
            if (listViewItem.Index % 2 == 0)
            {
                //even
                lineColor = BackColor;
            }
            else
            {
                //odd
                lineColor = AlternatingBackColor;
            }
            listViewItem.BackColor = lineColor;
        }

    }

}
