﻿using System;
using System.Windows.Forms;

namespace CustomControls
{
    public class ExtendedListViewItemCollection : ListView.ListViewItemCollection
    {
        public delegate void ItemAddedHandler(object sender, ItemAddedEventArgs args);
        public event ItemAddedHandler ItemAdded;

        public delegate void ItemRemovedHandler(object sender, ItemRemovedEventArgs args);
        public event ItemRemovedHandler ItemRemoved;

        public ExtendedListViewItemCollection(ListView owner) : base(owner)
        {
        }

        public FourStateListViewItem Add(FourStateListViewItem value)
        {
            FourStateListViewItem item = (FourStateListViewItem)base.Add((ListViewItem)value);
            this.OnItemAdded(new ItemAddedEventArgs(item));
            return item;
        }

        public new FourStateListViewItem Add(string key, string text, int imageIndex)
        {
            FourStateListViewItem item = (FourStateListViewItem)base.Add(key, text, imageIndex);
            this.OnItemAdded(new ItemAddedEventArgs(item));
            return item;
        }

        public new FourStateListViewItem Add(string key, string text, string imageKey)
        {
            FourStateListViewItem item = (FourStateListViewItem)base.Add(key, text, imageKey);
            this.OnItemAdded(new ItemAddedEventArgs(item));
            return item;
        }

        public new FourStateListViewItem Add(string text)
        {
            FourStateListViewItem item = (FourStateListViewItem) base.Add(text);
            this.OnItemAdded(new ItemAddedEventArgs(item));
            return item;
        }

        public new FourStateListViewItem Add(string text, int imageIndex)
        {
            FourStateListViewItem item = (FourStateListViewItem)base.Add(text, imageIndex);
            this.OnItemAdded(new ItemAddedEventArgs(item));
            return item;
        }

        public new FourStateListViewItem Add(string text, string imageKey)
        {
            FourStateListViewItem item = (FourStateListViewItem)base.Add(text, imageKey);
            this.OnItemAdded(new ItemAddedEventArgs(item));
            return item;
        }

        public void Remove(FourStateListViewItem item)
        {
            Int32 index = base.IndexOf(item);
            base.Remove(item);
            this.OnItemRemoved(new ItemRemovedEventArgs(item, index));
        }

        public override void RemoveAt(int index)
        {
            FourStateListViewItem item = (FourStateListViewItem)base[index];
            base.RemoveAt(index);
            this.OnItemRemoved(new ItemRemovedEventArgs(item, index));
        }

        protected virtual void OnItemAdded(ItemAddedEventArgs e)
        {
            if (this.ItemAdded != null)
                this.ItemAdded(this, e);
        }

        protected virtual void OnItemRemoved(ItemRemovedEventArgs e)
        {
            if (this.ItemRemoved != null)
                this.ItemRemoved(this, e);
        }
    }

    #region "events"
    public class ItemAddedEventArgs : EventArgs
    {
        private FourStateListViewItem m_Item;
        public ItemAddedEventArgs(FourStateListViewItem item)
        {
            this.Item = item;
        }

        public FourStateListViewItem Item
        {
            get
            {
                return this.m_Item;
            }
            private set
            {
                this.m_Item = value;
            }
        }
    }

    public class ItemRemovedEventArgs : EventArgs
    {
        private FourStateListViewItem m_Item;
        private Int32 m_Index;
        public ItemRemovedEventArgs(FourStateListViewItem item, Int32 index)
        {
            this.Item = item;
            this.Index = index;
        }

        public FourStateListViewItem Item
        {
            get
            {
                return this.m_Item;
            }
            private set
            {
                this.m_Item = value;
            }
        }

        public Int32 Index
        {
            get
            {
                return this.m_Index;
            }

            private set
            {
                this.m_Index = value;
            }
        }
    }
    #endregion
}
