﻿using System;
using System.Windows.Forms;
using System.ComponentModel;
using System.Drawing;

namespace CustomControls
{
    [DesignTimeVisible(false), TypeConverter("CustomControls.FourStateListviewItemConverter")]
    public class FourStateListViewItem : ListViewItem
    {
        public enum CheckStates
        {
            None = -1,
            UnCheck = 0,
            Check = 1,
            Undetermined = 2
        }

        private CheckStates m_CheckState;

        public FourStateListViewItem()
            : base()
        {
            this.ProcessParameters(string.Empty, -1);
            
        }

        public FourStateListViewItem(string text)
            : base(text)
        {
            this.ProcessParameters(text, -1);
        }

        public FourStateListViewItem(string text, Int32 imageIndex)
            : base(text, imageIndex)
        {
            this.ProcessParameters(text, imageIndex);
        }

        public FourStateListViewItem(string[] subitems)
            : base(subitems)
        {
            this.ProcessParameters(subitems, -1);
        }

        public FourStateListViewItem(string[] subitems, Int32 imageIndex)
            : base(subitems, imageIndex)
        {
            this.ProcessParameters(subitems, imageIndex);
        }

        public FourStateListViewItem(string[] subitems, Int32 imageIndex, Color foreColor, Color backColor, Font font)
            : base(subitems, imageIndex, foreColor, backColor, font)
        {
            this.ProcessParameters(subitems,imageIndex);
        }

        [Browsable(true), DefaultValue(typeof(CheckStates), "UnCheck"), Category("Appearance")]
        public new CheckStates Checked
        {
            get
            {
                return this.m_CheckState;
            }
            set
            {
                this.m_CheckState = value;
                this.UpdateStateImageIndex();
            }
        }

        [Browsable(true), DefaultValue("-1"), Category("Appearance")]
        public new Int32 ImageIndex
        {
            get
            {
                return base.ImageIndex;
            }
            set
            {
                base.ImageIndex = value;
                this.UpdateChecked(value);
            }
        }

        public static FourStateListViewItem InitializeSubItems(int columnCount)
        {
            FourStateListViewItem fourStateListViewItem = new FourStateListViewItem();

            for (int index = 0; index < columnCount - 1; index++)
            {
                fourStateListViewItem.SubItems.Add(string.Empty);
            }
            return fourStateListViewItem;
        }

        private void ProcessParameters(string text, Int32 imageIndex)
        {
            this.Text = text;
            this.UpdateChecked(imageIndex);
            this.UpdateStateImageIndex();
        }

        private void ProcessParameters(string[] subitems, Int32 imageIndex)
        {
            //do not process text and subitems here
            //class FourStateListviewItemConverter takes care of this
            this.UpdateChecked(imageIndex);
            this.UpdateStateImageIndex();
        }

        private void UpdateChecked(Int32 imageIndex)
        {
            switch (imageIndex)
            {
                case -1:
                    this.m_CheckState = CheckStates.None;
                    break;
                case 0:
                    this.m_CheckState = CheckStates.UnCheck;
                    break;
                case 1:
                    this.m_CheckState = CheckStates.Check;
                    break;
                case 2:
                    this.m_CheckState = CheckStates.Undetermined;
                    break;
            }
        }

        private void UpdateStateImageIndex()
        {
            switch (Checked)
            {
                case FourStateListViewItem.CheckStates.None:
                    this.ImageIndex = -1;
                    break;
                case FourStateListViewItem.CheckStates.UnCheck:
                    this.ImageIndex = 0;
                    break;
                case FourStateListViewItem.CheckStates.Check:
                    this.ImageIndex = 1;
                    break;
                case FourStateListViewItem.CheckStates.Undetermined:
                    this.ImageIndex = 2;
                    break;
            }
        }

    }

}
