﻿using System;
using System.ComponentModel;
using System.ComponentModel.Design.Serialization;
using System.Drawing;
using System.Windows.Forms;

namespace CustomControls
{
    class FourStateListviewItemConverter : TypeConverter
    {
        public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
        {
            if (destinationType == typeof(InstanceDescriptor))
            {
                return true;
            }
            return base.CanConvertTo(context, destinationType);
        }

        public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
        {
            if (destinationType == typeof(InstanceDescriptor) && value is FourStateListViewItem)
            {
                FourStateListViewItem fourStateListViewItem = (FourStateListViewItem)value;

                Type[] types = null;
                object[] properties = null;

                if (fourStateListViewItem.SubItems.Count > 0)
                {
                    //subitems
                    string[] subItems = new string[fourStateListViewItem.SubItems.Count];
                    for (int index = 0; index < fourStateListViewItem.SubItems.Count; index++)
                    {
                        subItems[index] = fourStateListViewItem.SubItems[index].Text;
                    }

                    if (this.IsDefaultBackColor(fourStateListViewItem.BackColor) && this.IsDefaultForeColor(fourStateListViewItem.ForeColor) && this.IsDefaultFont(fourStateListViewItem.Font))
                    {
                        //instantiate arrays for subitems and imageindex
                        types = new Type[2];
                        properties = new object[2];
                        //assign values
                        types[0] = typeof(string[]);
                        types[1] = typeof(Int32);
                        properties[0] = subItems;
                        properties[1] = fourStateListViewItem.ImageIndex;
                    }
                    else
                    {
                        //instantiate arrays for subitems, imageindex, foreColor, backColor and font
                        types = new Type[5];
                        properties = new object[5];
                        //assign values
                        types[0] = typeof(string[]);
                        types[1] = typeof(Int32);
                        types[2] = typeof(Color);
                        types[3] = typeof(Color);
                        types[4] = typeof(Font);
                        properties[0] = subItems;
                        properties[1] = fourStateListViewItem.ImageIndex;
                        properties[2] = fourStateListViewItem.ForeColor;
                        properties[3] = fourStateListViewItem.BackColor;
                        properties[4] = fourStateListViewItem.Font;
                    }
                }
                else
                {
                    //no parameters specified
                    types = new Type[] { };
                }
                System.Reflection.ConstructorInfo ci = typeof(FourStateListViewItem).GetConstructor(types);
                if (ci != null)
                {
                    return new InstanceDescriptor(ci, properties, false);
                }
            }
            return base.ConvertTo(context, culture, value, destinationType);
        }

        private bool IsDefaultFont(Font font)
        {
            bool b = font.Name == "Microsoft Sans Serif";
            b &= font.Size == 8.25F;
            b &= font.Style == FontStyle.Regular;
            return b;
        }

        private bool IsDefaultBackColor(Color backColor)
        {
            return backColor.Equals(System.Drawing.SystemColors.Window);
        }

        private bool IsDefaultForeColor(Color foreColor)
        {
            return foreColor.Equals(System.Drawing.SystemColors.WindowText);
        }
    }
}
