﻿using System;
using System.Windows.Forms;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing;
using System.Drawing.Design;

namespace CustomControls
{
    //[ToolboxBitmapAttribute(typeof(CustomControls.ExtendedTabControl), "tabcontrol.bmp")]
    [ToolboxBitmap(typeof(System.Windows.Forms.TabControl))]
    [Designer(typeof(ExtendedTabControlDesigner), typeof(IDesigner))]
    public class ExtendedTabControl : TabControl, IExtendedTabControl
    {
        private Color m_SelectedTabForeColor;
        private Color m_SelectedTabBackColor;

        public ExtendedTabControl()
        {
            this.DrawItem += new DrawItemEventHandler(this.OnExtendedTabControl_DrawItem);

            this.Margin = new Padding(0);
            this.Padding = new Point(0);
            this.DrawMode = TabDrawMode.OwnerDrawFixed;
            this.SelectedTabForeColor = Color.Black;
            this.SelectedTabBackColor = Color.Silver;
        }

        [Editor(typeof(ExtendedTabControlTabPagesCollectionEditor), typeof(UITypeEditor))]
        public new TabPageCollection TabPages
        {
            get
            {
                return base.TabPages;
            }
        }

        [Browsable(true), DefaultValue(typeof(Color), "Black"), Category("Misc"), Description("Selected tab foreground color")]
        public Color SelectedTabForeColor
        {
            get
            {
                return this.m_SelectedTabForeColor;
            }
            set
            {
                this.m_SelectedTabForeColor = value;
                this.Refresh();
            }
        }

        [Browsable(true), DefaultValue(typeof(Color), "Silver"), Category("Misc"), Description("Selected tab background color")]
        public Color SelectedTabBackColor
        {
            get
            {
                return this.m_SelectedTabBackColor;
            }
            set
            {
                this.m_SelectedTabBackColor = value;
                this.Refresh();
            }
        }

        private void OnExtendedTabControl_DrawItem(Object sender, System.Windows.Forms.DrawItemEventArgs e)
        {
            Graphics g = e.Graphics;
            Brush textBrush = null;

            // Get the item from the collection.
            ITabPage tabPage = (ITabPage)this.TabPages[e.Index];
            
            // Get the real bounds for the tab rectangle.
            Rectangle tabBounds = this.GetTabRect(e.Index);

            //draw tab's
            Type tabpageType = this.TabPages[e.Index].GetType();
            if (tabpageType == typeof(ExtendedTabPage) && !((ExtendedTabPage)tabPage).Selectable)
            {
                //show as disabled
                textBrush = new SolidBrush(SystemColors.GrayText);
                g.FillRectangle(Brushes.Transparent, e.Bounds);
            }
            else
            {
                if (e.State == DrawItemState.Selected)
                {
                    //Draw a different background color, and don't paint a focus rectangle.
                    textBrush = new SolidBrush(this.SelectedTabForeColor);
                    g.FillRectangle(new SolidBrush(this.SelectedTabBackColor), e.Bounds);
                }
                else
                {
                    //default values
                    textBrush = new SolidBrush(e.ForeColor);
                    g.FillRectangle(Brushes.Transparent, e.Bounds);
                }
            }

            // Draw string. Center the text.
            StringFormat stringFormat = new StringFormat();
            stringFormat.Alignment = StringAlignment.Center;
            stringFormat.LineAlignment = StringAlignment.Center;
            if (this.Alignment == TabAlignment.Left || this.Alignment == TabAlignment.Right)
            {
                stringFormat.FormatFlags = StringFormatFlags.DirectionVertical;
            }
            g.DrawString(tabPage.Text, this.Font, textBrush, tabBounds, new StringFormat(stringFormat));

            //tab icon
            if ((tabPage.ImageKey != null && tabPage.ImageKey != string.Empty) || tabPage.ImageIndex > -1)
            {
                Image image = null;
                if (tabPage.ImageKey != null && tabPage.ImageKey != string.Empty)
                {
                    image = this.ImageList.Images[tabPage.ImageKey];
                }
                else
                {
                    image = this.ImageList.Images[tabPage.ImageIndex];
                }
                //unselected tabs
                Int32 x = e.Bounds.Location.X - 2;
                Int32 y = e.Bounds.Location.Y - 1;

                if (e.State == DrawItemState.Selected)
                {
                    //selected tab
                    x = e.Bounds.Location.X + 2;
                    y = e.Bounds.Location.Y + 3;
                }
                if (this.Alignment == TabAlignment.Left || this.Alignment == TabAlignment.Right)
                {
                    //unselected tabs
                    x += 4;
                    if (e.State == DrawItemState.Selected)
                    {
                        //selected tab
                        x -= 2;
                    }
                }
                if (image != null)
                {
                    g.DrawImage(image, new Point(x, y));
                }
            }
            g.Dispose();
        }

        #region IExtendedTabControl Members
        public new void MouseClick(Point location)
        {
            for (Int32 index = 0; index < this.TabPages.Count; index++)
            {
                if (this.GetTabRect(index).Contains(location))
                {
                    TabPage tabPage = this.TabPages[index];
                    this.SelectedTab = tabPage;
                    index = this.TabPages.Count;
                }
            }
        }
        #endregion
    }
}
