﻿using System;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.ComponentModel.Design;
using System.Drawing;

namespace CustomControls
{
    public class ExtendedTabControlDesigner : ParentControlDesigner
    {
        private const int WM_LBUTTONDOWN = 0x201;
        private const string ADD_HEADER_TABPAGE = "Add Header TabPage";
        private const string ADD_EXTENDED_TABPAGE = "Add Extended TabPage";
        private const string REMOVE_SELECTED_TABPAGE = "Remove Selected TabPage";
        private const string HEADER_TABPAGE = "headerTabPage";
        private const string EXTENDED_TABPAGE = "extendedTabPage";
        private ExtendedTabControl m_ExtendedTabControl;
        private IComponentChangeService m_ChangeService;

        public override void Initialize(IComponent component)
        {
            base.Initialize(component);

            this.m_ExtendedTabControl = component as ExtendedTabControl;
            this.m_ChangeService = (IComponentChangeService)GetService(typeof(IComponentChangeService));
            Verbs.Add(new DesignerVerb(ADD_HEADER_TABPAGE, new EventHandler(OnHeaderTabPageAdded)));
            Verbs.Add(new DesignerVerb(ADD_EXTENDED_TABPAGE, new EventHandler(OnExtendedTabPageAdded)));
            Verbs.Add(new DesignerVerb(REMOVE_SELECTED_TABPAGE, new EventHandler(OnHeaderTabPageRemoved)));
        }

        public override void InitializeNewComponent(System.Collections.IDictionary defaultValues)
        {
            base.InitializeNewComponent(defaultValues);
            addPages();
        }

        private void OnHeaderTabPageAdded(object sender, EventArgs e)
        {
            IDesignerHost designerHost = this.GetService(typeof(IDesignerHost)) as IDesignerHost;
            HeaderTabPage headerTabPage = (HeaderTabPage)designerHost.CreateComponent(typeof(HeaderTabPage));
            this.AddTabPage(designerHost, ADD_HEADER_TABPAGE, HEADER_TABPAGE, headerTabPage);
        }

        private void OnExtendedTabPageAdded(object sender, EventArgs e)
        {
            IDesignerHost designerHost = this.GetService(typeof(IDesignerHost)) as IDesignerHost;
            ExtendedTabPage extendedTabPage = (ExtendedTabPage)designerHost.CreateComponent(typeof(ExtendedTabPage));
            this.AddTabPage(designerHost, ADD_EXTENDED_TABPAGE, EXTENDED_TABPAGE, extendedTabPage);
        }

        private void OnHeaderTabPageRemoved(object sender, EventArgs e)
        {
            if (this.m_ExtendedTabControl.TabPages.Count > 0)
            {
                IDesignerHost designerHost = this.GetService(typeof(IDesignerHost)) as IDesignerHost;
                DesignerTransaction transaction = designerHost.CreateTransaction("Remove Selected TabPage");
                TabPage tabPage = this.m_ExtendedTabControl.SelectedTab;
                //remove from tabpage collection
                this.m_ChangeService.OnComponentChanging(this.m_ExtendedTabControl, null);
                this.m_ExtendedTabControl.TabPages.Remove(tabPage);
                this.m_ChangeService.OnComponentChanged(this.m_ExtendedTabControl, null, null, null);
                //remove code from designer
                designerHost.DestroyComponent(tabPage);
                transaction.Commit();
            }
        }

        private void AddTabPage(IDesignerHost designerHost, string transactionName, string headerName, ITabPage tabpage)
        {
            DesignerTransaction transaction = designerHost.CreateTransaction(transactionName);
            //add code to designer
            bool found = false;
            Int32 counter = 0;
            while (!found)
            {
                counter += 1;
                found = !this.m_ExtendedTabControl.TabPages.ContainsKey(this.GetKey(headerName, counter.ToString()));
            }
            tabpage.Text = this.GetKey(headerName, counter.ToString());
            //add to tabpage collection
            this.m_ChangeService.OnComponentChanging(this.m_ExtendedTabControl, null);
            this.m_ExtendedTabControl.TabPages.Add((TabPage) tabpage);
            this.m_ChangeService.OnComponentChanged(this.m_ExtendedTabControl, null, null, null);
            transaction.Commit();
        }

        protected virtual void addPages()
        {
            IDesignerHost designerHost = this.GetService(typeof(IDesignerHost)) as IDesignerHost;
            if (designerHost != null)
            {
                ExtendedTabPage page1 = designerHost.CreateComponent(typeof(ExtendedTabPage)) as ExtendedTabPage;
                ExtendedTabPage page2 = designerHost.CreateComponent(typeof(ExtendedTabPage)) as ExtendedTabPage;

                page1.Text = "extendedTabPage1";
                page2.Text = "extendedTabPage2";

                this.m_ExtendedTabControl.TabPages.Add(page1);
                this.m_ExtendedTabControl.TabPages.Add(page2);

                this.m_ExtendedTabControl.Invalidate();
            }
        }

        protected override void PreFilterProperties(IDictionary properties)
        {
            base.PreFilterProperties(properties);
            properties.Remove("DrawMode");
        }

        protected override void WndProc(ref Message m)
        {
            if (m.Msg == WM_LBUTTONDOWN)
            {
                Point point = this.m_ExtendedTabControl.PointToClient(Cursor.Position);
                for (Int32 index = 0; index < this.m_ExtendedTabControl.TabPages.Count; index++)
                {
                    if (this.m_ExtendedTabControl.GetTabRect(index).Contains(point))
                    {
                        ITabPage tabpage = (ITabPage) this.m_ExtendedTabControl.TabPages[index];
                        this.m_ChangeService.OnComponentChanging(this.m_ExtendedTabControl, null);
                        this.m_ExtendedTabControl.SelectedTab = (TabPage) tabpage;
                        this.m_ChangeService.OnComponentChanged(this.m_ExtendedTabControl, null, null, null);
                        index = this.m_ExtendedTabControl.TabPages.Count;
                    }
                }
            }
            base.WndProc(ref m);
        }

        private string GetKey(string name, string counter)
        {
            return string.Format("{0}{1}", name, counter);
        }
    }

}
