﻿using System;
using System.Windows.Forms;
using System.Drawing;
using System.ComponentModel;
using System.ComponentModel.Design;

namespace CustomControls
{
    [Designer(typeof(HeaderTabPageControlDesigner), typeof(IDesigner))]
    public class HeaderTabPage : TabPage, ITabPage
    {
        private const int WM_MOUSEMOVE = 0x200;
        private ImageList imageListHeader;
        private Label labelHeader;
        private System.ComponentModel.IContainer components;
        private Rectangle m_RectanglePushPin;
        private Rectangle m_RectangleClose;
        private Rectangle m_RectangleControlBox;
        private Rectangle m_HighLightRectangle;
        private int m_Xoffset;
        private Boolean m_Showclose;
        private Brush m_HighLightRectangleBrush;
        private Color m_HeaderSelectedButtonHighLightColor;
        private Color m_HeaderBackgroundColor;
        private Panel m_PanelItems;

        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(HeaderTabPage));
            this.imageListHeader = new System.Windows.Forms.ImageList(this.components);
            this.SuspendLayout();
            // 
            // imageListHeader
            // 
            this.imageListHeader.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageListHeader.ImageStream")));
            this.imageListHeader.TransparentColor = System.Drawing.Color.Transparent;
            this.imageListHeader.Images.SetKeyName(0, "pushpin_close_32x16.png");
            this.imageListHeader.Images.SetKeyName(1, "pushpin_32x16.png");
            this.ResumeLayout(false);

        }

        public HeaderTabPage()
        {
            this.InitializeComponent();

            //labelHeader
            this.labelHeader = new Label();
            this.labelHeader.Dock = System.Windows.Forms.DockStyle.Top;
            this.labelHeader.ImageAlign = System.Drawing.ContentAlignment.MiddleRight;
            this.labelHeader.ImageKey = "pushpin_32x16.png";
            this.labelHeader.ImageList = this.imageListHeader;
            this.labelHeader.Location = new System.Drawing.Point(0, 0);
            this.labelHeader.Name = "labelHeader";
            this.labelHeader.Size = new System.Drawing.Size(100, 20);
            this.labelHeader.TabIndex = 0;
            this.labelHeader.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;

            //panelItems
            this.m_PanelItems = new Panel();
            this.m_PanelItems.Dock = DockStyle.Fill;
            this.m_PanelItems.BorderStyle = BorderStyle.None ;

            //events
            this.TextChanged += new EventHandler(this.OnHeaderTabPage_TextChanged);
            this.labelHeader.MouseMove += new MouseEventHandler(this.OnLabelHeader_MouseMove);
            this.labelHeader.MouseLeave += new EventHandler(this.OnLabelHeader_MouseLeave);
            this.labelHeader.MouseClick += new MouseEventHandler(this.OnLabelHeader_MouseClick);
            this.labelHeader.Paint += new PaintEventHandler(this.OnLabelHeader_Paint);
            this.labelHeader.Resize +=new EventHandler(this.OnLabelHeader_Resize);

            //default values
            this.UseVisualStyleBackColor = true;
            this.BorderStyle = BorderStyle.FixedSingle;
            this.ShowClose = false;
            this.HeaderBackgroundColor = Color.LightBlue;
            this.HeaderSelectedButtonHighLightColor = Color.Red;

            //add controls
            this.Controls.Add(this.labelHeader);
            this.Controls.Add(this.m_PanelItems);

            //bring panelHeader to front
            this.m_PanelItems.BringToFront();

            //hide selected butten highlight rectangle
            this.HideHighLightRectangle();

            //calculate size of controlbox and sub items
            this.SetControlPicture();
        }

        private void SetControlPicture()
        {
            //labelHeader
            if (!this.ShowClose)
            {
                this.labelHeader.ImageKey = "pushpin_32x16.png";
                this.m_Xoffset = this.labelHeader.Width - 16 - 4;
                int yOffset = (this.labelHeader.Height - 16) / 2;
                this.m_RectangleControlBox = new Rectangle(this.m_Xoffset, yOffset, 16, 16);
                this.m_RectanglePushPin = new Rectangle(this.m_Xoffset, yOffset, 16, 16);
            }
            else
            {
                this.labelHeader.ImageKey = "pushpin_close_32x16.png";
                this.m_Xoffset = this.labelHeader.Width - 32 - 5;
                int yOffset = (this.labelHeader.Height - 16) / 2;
                this.m_RectangleControlBox = new Rectangle(this.m_Xoffset, yOffset, 32, 16);
                this.m_RectanglePushPin = new Rectangle(this.m_Xoffset, yOffset, 16, 16);
                this.m_RectangleClose = new Rectangle(this.m_Xoffset + 16, yOffset, 16, 16);
            }
        }

        //mouse move over label
        private void OnLabelHeader_MouseMove(Object sender, MouseEventArgs e)
        {
            if (this.m_RectangleControlBox.Contains(e.Location))
            {
                if (this.m_RectangleClose.Contains(e.Location))
                {
                    this.m_HighLightRectangle = this.m_RectangleClose;
                }
                else
                {
                    this.m_HighLightRectangle = this.m_RectanglePushPin;
                }
            }
            else
            {
                //hide highlight rectangle
                this.HideHighLightRectangle();
            }
            //repaint
            this.labelHeader.Invalidate();
        }

        private void OnLabelHeader_MouseLeave(Object sender, EventArgs e)
        {
            //hide highlight rectangle
            this.HideHighLightRectangle();

            //repaint
            this.labelHeader.Invalidate();
        }

        private void OnLabelHeader_MouseClick(Object sender, MouseEventArgs e)
        {
            //pushpin
            if (this.m_RectanglePushPin.Contains(e.Location))
            {
                PushPinClickEvent pushPinClickEvent = new PushPinClickEvent(this);
                DispatcherPushPinClickEvent.sendPushPinClickEvent(pushPinClickEvent);
                //hide selected butten highlight rectangle
                this.HideHighLightRectangle();
            }
            //close
            else if (this.m_RectangleClose.Contains(e.Location))
            {
                this.Dispose();
            }
        }

        //paint
        private void OnLabelHeader_Paint(Object sender, PaintEventArgs e)
        {
            if (this.m_HighLightRectangle.Height > 0 && this.m_HighLightRectangle.Width > 0)
            {
                e.Graphics.FillRectangle(this.HighLightRectangleBrush, this.m_HighLightRectangle);
            }
        }

        //tabpage resize
        private void OnLabelHeader_Resize(Object sender, EventArgs e)
        {
            //hide selected butten highlight rectangle
            this.HideHighLightRectangle();
            this.SetControlPicture();
        }

        //tabpage text changed
        private void OnHeaderTabPage_TextChanged(Object sender, EventArgs e)
        {
            this.labelHeader.Text = this.Text;
        }

        //hide highlightRectangle
        public void HideHighLightRectangle()
        {
            this.m_HighLightRectangle = new Rectangle(0, 0, 0, 0);
        }

        //show close button
        [Browsable(true), DefaultValue(false), Category("Misc"), Description("Show close button")]
        public Boolean ShowClose
        {
            get
            {
                return this.m_Showclose;
            }
            set
            {
                this.m_Showclose = value;
                this.SetControlPicture();
            }
        }

        [Browsable(true), DefaultValue(typeof(Color), "LightBlue"), Category("Misc"), Description("Header background color")]
        public Color HeaderBackgroundColor
        {
            get
            {
                return this.m_HeaderBackgroundColor;
            }
            set
            {
                this.m_HeaderBackgroundColor = value;
                this.labelHeader.BackColor = this.HeaderBackgroundColor;
            }
        }

        [Browsable(true), DefaultValue(typeof(Color), "Red"), Category("Misc"), Description("Selected header button highlight color")]
        public Color HeaderSelectedButtonHighLightColor
        {
            get
            {
                return this.m_HeaderSelectedButtonHighLightColor;
            }
            set
            {
                this.m_HeaderSelectedButtonHighLightColor = value;
                this.SetHighLightRectangleBrush();
            }
        }

        //expose panel internal to the ExtendedTabPageControlDesigner
        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        public Panel Items
        {
            get
            {
                return this.m_PanelItems;
            }
        }

        private Brush HighLightRectangleBrush
        {
            get
            {
                return this.m_HighLightRectangleBrush;
            }
            set
            {
                this.m_HighLightRectangleBrush = value;
            }
        }

        private void SetHighLightRectangleBrush()
        {
            this.HighLightRectangleBrush = new SolidBrush(Color.FromArgb(96, this.HeaderSelectedButtonHighLightColor.R, this.HeaderSelectedButtonHighLightColor.G, this.HeaderSelectedButtonHighLightColor.B));
        }

    }
}
