﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business.DataHolders

Public Class ProjectProposalRevisionSelection
    Inherits GenericSelection(Of ProjectProposalRevision)
    Private m_CurrentRevisionStatusLine As RevisionStatusLine

    Public Enum ChangedRevisionStatusType As Integer
        Minor = 0
        MinorSub = 1
    End Enum

    Public Sub New()
        MyBase.New()
    End Sub

    ''' <summary>
    ''' Add a project proposal revision to the selection
    ''' </summary>
    ''' <param name="selectedObject">Project proposal revision object to add to the selection</param>
    ''' <remarks></remarks>
    Public Overrides Sub Add(ByVal selectedObject As DataHolders.ProjectProposalRevision)
        MyBase.Add(selectedObject)
        SetCurrentRevisionStatus()
    End Sub

    ''' <summary>
    ''' Update description
    ''' </summary>
    ''' <param name="description">Description of the project</param>
    ''' <remarks></remarks>
    Public Sub UpdateDescription(ByVal description As String)
        MyBase.Selection.ProjectDescription = description
    End Sub

    ''' <summary>
    ''' Update auditor description
    ''' </summary>
    ''' <param name="auditorDescription">Auditor remarks</param>
    ''' <remarks>This field contains the explication why the project is accepted or rejected</remarks>
    Public Sub UpdateAuditorDescription(ByVal auditorDescription As String)
        MyBase.Selection.RevisionInformation.Description = auditorDescription
        MyBase.Selection.RevisionInformation.RevisedBy = MyBase.Selection.Auditor.ToString()
    End Sub

    ''' <summary>
    ''' Update tye revision status
    ''' </summary>
    ''' <param name="revisionStatus">Revision status object</param>
    ''' <remarks></remarks>
    Public Sub UpdateRevisionStatus(ByVal revisionStatus As RevisionStatus)
        MyBase.Selection.RevisionInformation.RevisionStatus = revisionStatus
    End Sub

    ''' <summary>
    ''' Update date modified
    ''' </summary>
    ''' <param name="dateModified">The current datetime</param>
    ''' <remarks>Indicates when this project is last updated</remarks>
    Public Sub UpdateRevisionLastModified(ByVal dateModified As DateTime)
        MyBase.Selection.RevisionInformation.DateModified = dateModified
    End Sub

    ''' <summary>
    ''' Update the creator
    ''' </summary>
    ''' <param name="creator">Owner of the project</param>
    ''' <remarks>Only this person can edit the project data</remarks>
    Public Sub UpdateCreator(ByVal creator As Creator)
        MyBase.Selection.Creator = creator
    End Sub

    ''' <summary>
    ''' Update the auditor
    ''' </summary>
    ''' <param name="auditor">The reviewer of this project</param>
    ''' <remarks>The role of this person is to decide whether the project is accepted or rejected</remarks>
    Public Sub UpdateAuditor(ByVal auditor As Auditor)
        MyBase.Selection.Auditor = auditor
    End Sub

    ''' <summary>
    ''' Update sponser
    ''' </summary>
    ''' <param name="sponsorId">Employee id</param>
    ''' <param name="sponsorName">Employee name</param>
    ''' <remarks></remarks>
    Public Sub UpdateSponsor(ByVal sponsorId As String, ByVal sponsorName As String)
        MyBase.Selection.SponsorId = sponsorId
        MyBase.Selection.SponsorName = sponsorName
    End Sub

    ''' <summary>
    ''' Update pusher
    ''' </summary>
    ''' <param name="pusherId">Employee id</param>
    ''' <param name="pusherName">Employee name</param>
    ''' <remarks></remarks>
    Public Sub UpdatePusher(ByVal pusherId As String, ByVal pusherName As String)
        MyBase.Selection.PusherId = pusherId
        MyBase.Selection.PusherName = pusherName
    End Sub

    ''' <summary>
    ''' Update coordinator
    ''' </summary>
    ''' <param name="coordinatorId">Employee id</param>
    ''' <param name="coordinatorName">Employee name</param>
    ''' <remarks></remarks>
    Public Sub UpdateCoordinator(ByVal coordinatorId As String, ByVal coordinatorName As String)
        MyBase.Selection.CoordinatorId = coordinatorId
        MyBase.Selection.CoordinatorName = coordinatorName
    End Sub

    ''' <summary>
    ''' Update tyhe budget
    ''' </summary>
    ''' <param name="budget">Budget</param>
    ''' <remarks></remarks>
    Public Sub UpdateBudget(ByVal budget As String)
        MyBase.Selection.Budget = budget
    End Sub

    ''' <summary>
    ''' Update why
    ''' </summary>
    ''' <param name="why">Why</param>
    ''' <remarks></remarks>
    Public Sub UpdateWhy(ByVal why As String)
        MyBase.Selection.Why = why
    End Sub

    ''' <summary>
    ''' Update include
    ''' </summary>
    ''' <param name="include">Include</param>
    ''' <remarks></remarks>
    Public Sub UpdateInclude(ByVal include As String)
        MyBase.Selection.Include = include
    End Sub

    ''' <summary>
    ''' Update exclude
    ''' </summary>
    ''' <param name="exclude">Exclude</param>
    ''' <remarks></remarks>
    Public Sub UpdateExclude(ByVal exclude As String)
        MyBase.Selection.Exclude = exclude
    End Sub

    ''' <summary>
    ''' Update as is
    ''' </summary>
    ''' <param name="asIs">Asis</param>
    ''' <remarks></remarks>
    Public Sub UpdateAsIs(ByVal asIs As String)
        MyBase.Selection.AsIs = asIs
    End Sub

    ''' <summary>
    ''' Update to be
    ''' </summary>
    ''' <param name="toBe">To be</param>
    ''' <remarks></remarks>
    Public Sub UpdateToBe(ByVal toBe As String)
        MyBase.Selection.ToBe = toBe
    End Sub

    ''' <summary>
    ''' Update objectifs
    ''' </summary>
    ''' <param name="objectifs">Objectifs</param>
    ''' <remarks></remarks>
    Public Sub UpdateObjectifs(ByVal objectifs As String)
        MyBase.Selection.Objectifs = objectifs
    End Sub

    ''' <summary>
    ''' Update cost
    ''' </summary>
    ''' <param name="cost">Cost</param>
    ''' <remarks></remarks>
    Public Sub UpdateCost(ByVal cost As String)
        MyBase.Selection.Cost = cost
    End Sub

    ''' <summary>
    ''' Update benefits
    ''' </summary>
    ''' <param name="benefits">Benefits</param>
    ''' <remarks></remarks>
    Public Sub UpdateBenefits(ByVal benefits As String)
        MyBase.Selection.Benefits = benefits
    End Sub

    ''' <summary>
    ''' Update alignation
    ''' </summary>
    ''' <param name="alignation">Alignation</param>
    ''' <remarks></remarks>
    Public Sub UpdateAlignation(ByVal alignation As String)
        MyBase.Selection.Alignation = alignation
    End Sub

    ''' <summary>
    ''' Update preconditoins
    ''' </summary>
    ''' <param name="preconditions">Preconditoins</param>
    ''' <remarks></remarks>
    Public Sub UpdatePreconditions(ByVal preconditions As String)
        MyBase.Selection.Preconditions = preconditions
    End Sub

    ''' <summary>
    ''' 'Update constrains
    ''' </summary>
    ''' <param name="constraints">Constrains</param>
    ''' <remarks></remarks>
    Public Sub UpdateConstraints(ByVal constraints As String)
        MyBase.Selection.Constraints = constraints
    End Sub

    ''' <summary>
    ''' Update dependencies
    ''' </summary>
    ''' <param name="dependencies">Dependencies</param>
    ''' <remarks></remarks>
    Public Sub UpdateDependencies(ByVal dependencies As String)
        MyBase.Selection.Dependencies = dependencies
    End Sub

    ''' <summary>
    ''' Update risk
    ''' </summary>
    ''' <param name="risk">Risk</param>
    ''' <remarks></remarks>
    Public Sub UpdateRisk(ByVal risk As String)
        MyBase.Selection.Risk = risk
    End Sub

    ''' <summary>
    ''' Update support
    ''' </summary>
    ''' <param name="support">Support</param>
    ''' <remarks></remarks>
    Public Sub UpdateSupport(ByVal support As String)
        MyBase.Selection.Support = support
    End Sub

    ''' <summary>
    ''' Update who changed the revision status
    ''' </summary>
    ''' <param name="name">Name of the person who changed the revision status</param>
    ''' <remarks></remarks>
    Public Sub UpdateRevisedBy(ByVal name As String)
        Try
            MyBase.Selection.RevisionInformation.RevisedBy = name
        Catch ex As Exception
            Throw New Exception("Unable to update revised by.")
        End Try
    End Sub

    ''' <summary>
    ''' Update the project proposal name
    ''' </summary>
    ''' <param name="projectProposalName">Name of the project</param>
    ''' <remarks></remarks>
    Public Sub UpdateProjectProposalName(ByVal projectProposalName As ProjectProposalName)
        MyBase.Selection.Project = projectProposalName
    End Sub

    ''' <summary>
    ''' Update the subcategory
    ''' </summary>
    ''' <param name="subCategory">Subcategory</param>
    ''' <remarks></remarks>
    Public Sub UpdateSubCategory(ByVal subCategory As SubCategory)
        MyBase.Selection.SubCategory = subCategory
    End Sub

    ''' <summary>
    ''' Add or update a workpackage
    ''' </summary>
    ''' <param name="workPackage">Workpackage object to add or update</param>
    ''' <remarks></remarks>
    Public Sub AddOrUpdateWorkPackage(ByVal workPackage As WorkPackage)
        Dim workPackages As IList(Of WorkPackage) = New List(Of WorkPackage)(MyBase.Selection.WorkPackages)
        Dim index As Integer = GetIndexOfWorkPackage(workPackages, workPackage.Name)
        If (index > -1) Then
            workPackages(index) = workPackage
        Else
            workPackages.Add(workPackage)
        End If
        MyBase.Selection.WorkPackages = New List(Of WorkPackage)(workPackages)
    End Sub

    ''' <summary>
    ''' Remove a workpackage by name
    ''' </summary>
    ''' <param name="name">Workpackage name</param>
    ''' <remarks></remarks>
    Public Sub RemoveWorkPackage(ByVal name As String)
        Dim workPackages As IList(Of WorkPackage) = New List(Of WorkPackage)(MyBase.Selection.WorkPackages)
        Dim index As Integer = GetIndexOfWorkPackage(workPackages, name)
        workPackages.RemoveAt(index)
        MyBase.Selection.WorkPackages = New List(Of WorkPackage)(workPackages)
    End Sub

    ''' <summary>
    ''' Add or update a milestone
    ''' </summary>
    ''' <param name="workPackageName">Workpackage name</param>
    ''' <param name="milestone">Milestone object to add or update</param>
    ''' <remarks></remarks>
    Public Sub AddOrUpdateMilestone(ByVal workPackageName As String, ByVal milestone As Milestone)
        Dim milestones As IList(Of Milestone) = InternalGetMilestonesForWorkPackage(workPackageName)
        If (milestone.Id > -1) Then
            Dim found As Boolean
            Dim index As Integer = -1
            Dim milestoneEnumerator As IEnumerator(Of Milestone) = milestones.GetEnumerator()
            While (milestoneEnumerator.MoveNext() AndAlso Not found)
                index += 1
                If (milestoneEnumerator.Current.Name.Equals(milestone.Name)) Then
                    found = True
                End If
            End While
            If (found) Then
                milestones(index) = milestone
            End If
        Else
            milestones.Add(milestone)
        End If
    End Sub

    ''' <summary>
    ''' Remove a milstone from a workpackage
    ''' </summary>
    ''' <param name="workPackageName">Workpackage name</param>
    ''' <param name="milestoneName">Milestone name</param>
    ''' <remarks></remarks>
    Public Sub RemoveMilestone(ByVal workPackageName As String, ByVal milestoneName As String)
        Dim found As Boolean
        Dim milestones As IList(Of Milestone) = InternalGetMilestonesForWorkPackage(workPackageName)
        Dim milestoneEnumerator As IEnumerator(Of Milestone) = milestones.GetEnumerator()
        While (milestoneEnumerator.MoveNext() AndAlso Not found)
            If (milestoneEnumerator.Current.Name.Equals(milestoneName)) Then
                found = True
                milestones.Remove(milestoneEnumerator.Current)
            End If
        End While
    End Sub

    ''' <summary>
    ''' Add or update a team member
    ''' </summary>
    ''' <param name="teamMember">Teammember object to add or update</param>
    ''' <remarks></remarks>
    Public Sub AddOrUpdateTeamMember(ByVal teamMember As TeamMember)
        Dim teamMembers As IList(Of TeamMember) = New List(Of TeamMember)(MyBase.Selection.TeamMembers)
        Dim index As Integer = GetIndexOfTeamMember(teamMembers, teamMember.EmployeeId)
        If (index > -1) Then
            teamMembers(index) = teamMember
        Else
            teamMembers.Add(teamMember)
        End If
        MyBase.Selection.TeamMembers = New List(Of TeamMember)(teamMembers)
    End Sub

    ''' <summary>
    ''' Remove teammember by employee id
    ''' </summary>
    ''' <param name="employeeId">Employee id</param>
    ''' <remarks></remarks>
    Public Sub RemoveTeamMember(ByVal employeeId As String)
        Dim teamMembers As IList(Of TeamMember) = New List(Of TeamMember)(MyBase.Selection.TeamMembers)
        Dim index As Integer = GetIndexOfTeamMember(teamMembers, employeeId)
        teamMembers.RemoveAt(index)
        MyBase.Selection.TeamMembers = New List(Of TeamMember)(teamMembers)
    End Sub

    ''' <summary>
    ''' Get selection
    ''' </summary>
    ''' <returns>Returns the project proposal revision object</returns>
    ''' <remarks></remarks>
    Public Overrides Function GetSelection() As DataHolders.ProjectProposalRevision
        Return MyBase.Selection.DeepClone()
    End Function

    ''' <summary>
    ''' Is the revision status changed
    ''' </summary>
    ''' <returns>Returns true or false</returns>
    ''' <remarks></remarks>
    Public Function IsRevisionStatusChanged() As Boolean
        Return CurrentRevisionStatus.RevisionStatus.Name.Equals(MyBase.Selection.RevisionInformation.RevisionStatus.Name)
    End Function

    ''' <summary>
    ''' Get the new revision status
    ''' </summary>
    ''' <returns>Returns the new revision status</returns>
    ''' <remarks></remarks>
    Public Function GetChangedRevisionStatusType() As ChangedRevisionStatusType
        Select Case MyBase.Selection.RevisionInformation.RevisionStatus.Name
            Case TaskRevisionStatus.DRAFT, TaskRevisionStatus.REVIEW, TaskRevisionStatus.REJECTED
                GetChangedRevisionStatusType = ChangedRevisionStatusType.MinorSub
            Case TaskRevisionStatus.RELEASED
                GetChangedRevisionStatusType = ChangedRevisionStatusType.Minor
        End Select
    End Function

    ''' <summary>
    ''' Get workpackage by row identifier
    ''' </summary>
    ''' <param name="workPackageName">WorkPackage name</param>
    ''' <returns>Returns the requested workpackage</returns>
    ''' <remarks></remarks>
    Public Function GetWorkPackageByName(ByVal workPackageName As String) As WorkPackage
        Dim result As WorkPackage = Nothing
        Dim index As Integer = GetIndexOfWorkPackage(MyBase.Selection.WorkPackages, workPackageName)
        If (index > -1) Then
            result = MyBase.Selection.WorkPackages(index)
        End If
        Return result
    End Function

    ''' <summary>
    ''' Get all milestones for the specified workpackage
    ''' </summary>
    ''' <param name="workPackageName">WorkPackage name</param>
    ''' <returns>Returns a IList of milestones</returns>
    ''' <remarks></remarks>
    Public Function GetMilestonesForWorkPackage(ByVal workPackageName As String) As IList(Of Milestone)
        Return New List(Of Milestone)(InternalGetMilestonesForWorkPackage(workPackageName))
    End Function

    ''' <summary>
    ''' Get the requested milestone 
    ''' </summary>
    ''' <param name="workPackageName">Row identifier</param>
    ''' <param name="milestoneName">Row identifier</param>
    ''' <returns>Returns the requested milestone</returns>
    ''' <remarks></remarks>
    Public Function GetMilestoneByName(ByVal workPackageName As String, ByVal milestoneName As String) As Milestone
        Dim result As Milestone = Nothing
        Dim found As Boolean
        Dim milestones As IList(Of Milestone) = GetMilestonesForWorkPackage(workPackageName)
        Dim milestoneEnumerator As IEnumerator(Of Milestone) = milestones.GetEnumerator()
        While (milestoneEnumerator.MoveNext() AndAlso Not found)
            If (milestoneEnumerator.Current.Name.Equals(milestoneName)) Then
                found = True
                result = milestoneEnumerator.Current
            End If
        End While
        Return result
    End Function

    ''' <summary>
    ''' Get all teammembers
    ''' </summary>
    ''' <returns>Returns a IList of teammmebers</returns>
    ''' <remarks></remarks>
    Public Function GetAllTeamMembers() As IList(Of TeamMember)
        Return MyBase.Selection.TeamMembers
    End Function

    ''' <summary>
    ''' Get teammember by employee id
    ''' </summary>
    ''' <param name="employeeId">Employee id</param>
    ''' <returns>Returns the requested teammember</returns>
    ''' <remarks></remarks>
    Public Function GetTeamMemberByEmployeeId(ByVal employeeId As String) As TeamMember
        Dim result As TeamMember = Nothing
        Dim found As Boolean
        Dim teamMembers As IList(Of TeamMember) = MyBase.Selection.TeamMembers
        Dim teamMemberEnumerator As IEnumerator(Of TeamMember) = teamMembers.GetEnumerator()
        While (teamMemberEnumerator.MoveNext() AndAlso Not found)
            If (teamMemberEnumerator.Current.EmployeeId.Equals(employeeId)) Then
                found = True
                result = teamMemberEnumerator.Current
            End If
        End While
        Return result
    End Function

    Private Sub SetCurrentRevisionStatus()
        If (CurrentRevisionStatus Is Nothing) Then
            CurrentRevisionStatus = MyBase.Selection.RevisionInformation.DeepClone()
        End If
    End Sub

    Private Function GetIndexOfTeamMember(ByVal list As IList(Of TeamMember), ByVal employeeId As String) As Integer
        Dim result As Integer = -1
        Dim index As Integer = -1
        Dim enumerator As IEnumerator(Of TeamMember) = list.GetEnumerator()
        While (enumerator.MoveNext() AndAlso result.Equals(-1))
            index += 1
            If (enumerator.Current.EmployeeId.Equals(employeeId)) Then
                result = index
            End If
        End While
        Return result
    End Function

    Private Function GetIndexOfWorkPackage(ByVal list As IList(Of WorkPackage), ByVal name As String) As Integer
        Dim result As Integer = -1
        Dim index As Integer = -1
        Dim enumerator As IEnumerator(Of WorkPackage) = list.GetEnumerator()
        While (enumerator.MoveNext() AndAlso result.Equals(-1))
            index += 1
            If (enumerator.Current.Name.Equals(name)) Then
                result = index
            End If
        End While
        Return result
    End Function

    Private Function GetIndexOf(Of T)(ByVal list As IList(Of T), ByVal id As Integer) As Integer
        Dim result As Integer = -1
        Dim index As Integer = -1
        If (id > -1) Then
            Dim enumerator As IEnumerator(Of T) = list.GetEnumerator()
            While (enumerator.MoveNext() AndAlso result.Equals(-1))
                index += 1
                If (DirectCast(enumerator.Current, IIdentifier).Id.Equals(id)) Then
                    result = index
                End If
            End While
        End If
        Return result
    End Function

    Private Function InternalGetMilestonesForWorkPackage(ByVal workPackageName As String) As IList(Of Milestone)
        Dim result As IList(Of Milestone) = New List(Of Milestone)()
        Dim index As Integer = GetIndexOfWorkPackage(MyBase.Selection.WorkPackages, workPackageName)
        If (index > -1) Then
            result = MyBase.Selection.WorkPackages(index).Milestones
        End If
        Return result
    End Function

    Private Property CurrentRevisionStatus() As RevisionStatusLine
        Get
            Return m_CurrentRevisionStatusLine
        End Get
        Set(ByVal value As RevisionStatusLine)
            m_CurrentRevisionStatusLine = value
        End Set
    End Property
End Class
