﻿Option Strict On
Option Explicit On
Option Infer Off

Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Net.Mail
Imports System.Configuration

Public Class TaskMail
    Private Shared m_TaskMail As TaskMail = New TaskMail()
    Private m_OutgoingMailserver As String

    Private Sub New()
        OutgoingMailserver = ConfigurationManager.AppSettings("outgoingMailServer")
    End Sub

    Public Shared ReadOnly Property Instance() As TaskMail
        Get
            Return m_TaskMail
        End Get
    End Property

    ''' <summary>
    ''' Send mail
    ''' </summary>
    ''' <param name="projectProposalRevision">Current project revision</param>
    ''' <remarks></remarks>
    Public Sub SendMail(ByVal projectProposalRevision As ProjectProposalRevision)
        'send mail
        If Not (projectProposalRevision.RevisionInformation.RevisionStatus.Name.Equals(TaskRevisionStatus.DRAFT)) Then
            Dim mail As Mail = New Mail()
            Dim mailFrom As String = projectProposalRevision.Auditor.EmailAddress
            Dim mailTo As String = projectProposalRevision.Creator.EmailAddress
            Dim mailText As String = String.Format("{0}{1}", projectProposalRevision.Project.Name, ConfigurationManager.AppSettings("toEditor"))
            If (projectProposalRevision.RevisionInformation.RevisionStatus.Name.Equals(TaskRevisionStatus.REVIEW)) Then
                mailFrom = projectProposalRevision.Creator.EmailAddress
                mailTo = projectProposalRevision.Auditor.EmailAddress
                mailText = String.Format("{0}{1}", projectProposalRevision.Project.Name, ConfigurationManager.AppSettings("toAuditor"))
            End If
            mail.From = mailFrom
            mail.To = mailTo
            mail.Subject = String.Format("{0}{1}", projectProposalRevision.Project.Name, ConfigurationManager.AppSettings("subject"))
            mail.Text = mailText
            TaskMail.Instance.InternalSendMail(mail)
        End If
    End Sub

    Private Sub InternalSendMail(ByVal mail As Mail)
        Try
            Dim message As MailMessage = New MailMessage()
            message.From = New MailAddress(mail.From)
            message.To.Add(New MailAddress(mail.To))
            message.Subject = mail.Subject
            message.Body = mail.Text
            Dim smtp As SmtpClient = New SmtpClient(OutgoingMailserver)
            smtp.Send(message)
        Catch ex As Exception
            Throw New Exception("Unable to send mail.", ex.InnerException)
        End Try
    End Sub

    Private Property OutgoingMailserver() As String
        Get
            Return m_OutgoingMailserver
        End Get
        Set(ByVal value As String)
            m_OutgoingMailserver = value
        End Set
    End Property
End Class
