﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business.DataHolders

Public Class TaskProjectProposalRevision
    Private Shared m_TaskProjectProposalRevision As TaskProjectProposalRevision = New TaskProjectProposalRevision()
    Private m_Selection As ProjectProposalRevisionSelection

    Private Sub New()
        InternalSelection = New ProjectProposalRevisionSelection()
    End Sub

    Public Shared ReadOnly Property Instance() As TaskProjectProposalRevision
        Get
            Return m_TaskProjectProposalRevision
        End Get
    End Property

    ''' <summary>
    ''' Undo changes
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub Cancel()
        InternalSelection.Undo()
    End Sub

    ''' <summary>
    ''' Add this project to the selection
    ''' </summary>
    ''' <param name="projectProposalRevision">Project object</param>
    ''' <remarks></remarks>
    Public Sub Add(ByVal projectProposalRevision As ProjectProposalRevision)
        Try
            InternalSelection.Add(projectProposalRevision)
        Catch ex As Exception
            Throw New Exception("Unable to update projectProposal revision.")
        End Try
    End Sub

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="auditorDescription"></param>
    ''' <remarks></remarks>
    Public Sub UpdateAuditorDescription(ByVal auditorDescription As String)
        Try
            InternalSelection.UpdateAuditorDescription(auditorDescription)
        Catch ex As Exception
            Throw New Exception("Unable to update auditor revision review description.")
        End Try
    End Sub

    ''' <summary>
    ''' Update tye revision status
    ''' </summary>
    ''' <param name="revisionStatus">Revision status object</param>
    ''' <remarks></remarks>
    Public Sub UpdateRevisionStatus(ByVal revisionStatus As RevisionStatus)
        Try
            InternalSelection.UpdateRevisionStatus(revisionStatus)
        Catch ex As Exception
            Throw New Exception("Unable to update revision status.")
        End Try
    End Sub

    ''' <summary>
    ''' Update date modified
    ''' </summary>
    ''' <param name="dateModified">he current datetime</param>
    ''' <remarks>Indicates when this project is last updated</remarks>
    Public Sub UpdateRevisionLastModified(ByVal dateModified As DateTime)
        Try
            InternalSelection.UpdateRevisionLastModified(dateModified)
        Catch ex As Exception
            Throw New Exception("Unable to update revision last modified.")
        End Try
    End Sub

    ''' <summary>
    ''' Update the project name
    ''' </summary>
    ''' <param name="name">Name of project</param>
    ''' <remarks></remarks>
    Public Sub UpdateProjectName(ByVal name As String)
        Try
            Dim projectName As ProjectProposalName = InternalSelection.GetSelection().Project
            projectName.Name = name
            InternalSelection.UpdateProjectProposalName(projectName)
        Catch ex As Exception
            Throw New Exception("Unable to update project name.")
        End Try
    End Sub

    ''' <summary>
    ''' Update description
    ''' </summary>
    ''' <param name="description">Description of the project</param>
    ''' <remarks></remarks>
    Public Sub UpdateDescription(ByVal description As String)
        Try
            InternalSelection.UpdateDescription(description)
        Catch ex As Exception
            Throw New Exception("Unable to update description.")
        End Try
    End Sub

    ''' <summary>
    ''' Update the subcategory
    ''' </summary>
    ''' <param name="subCategory">Subcategory</param>
    ''' <remarks></remarks>
    Public Sub UpdateSubCategory(ByVal subCategory As SubCategory)
        Try
            subCategory.Category = TaskProjectProposalRevision.Instance.InternalSelection.GetSelection().SubCategory.Category
            InternalSelection.UpdateSubCategory(subCategory)
        Catch ex As Exception
            Throw New Exception("Unable to update subcategory.")
        End Try
    End Sub

    ''' <summary>
    ''' Update the category
    ''' </summary>
    ''' <param name="category">Category</param>
    ''' <remarks></remarks>
    Public Sub UpdateCategory(ByVal category As Category)
        Try
            Dim subCategory As SubCategory = TaskProjectProposalRevision.Instance.InternalSelection.GetSelection().SubCategory
            If (subCategory Is Nothing) Then
                subCategory = New SubCategory()
            End If
            subCategory.Category = category
            InternalSelection.UpdateSubCategory(subCategory)
        Catch ex As Exception
            Throw New Exception("Unable to update category.")
        End Try
    End Sub

    ''' <summary>
    ''' Update the creator
    ''' </summary>
    ''' <param name="employeeId">Owner of the project</param>
    ''' <remarks>Only this person can edit the project data</remarks>
    Public Sub UpdateCreator(ByVal employeeId As String)
        Try
            Dim employee As Employee = TaskEmployee.Instance.GetEmployeeForEmployeeID(employeeId)
            Dim creator As Creator = New Creator()
            creator.EmailAddress = employee.EmailAddress
            creator.EmployeeId = employee.EmployeeId
            creator.Name = employee.Name
            creator.SirName = employee.SirName
            InternalSelection.UpdateCreator(creator)
        Catch ex As Exception
            Throw New Exception("Unable to update creator.")
        End Try
    End Sub

    ''' <summary>
    ''' Update the auditor
    ''' </summary>
    ''' <param name="employeeId">The reviewer of this project</param>
    ''' <remarks>The role of this person is to decide whether the project is accepted or rejected</remarks>
    Public Sub UpdateAuditor(ByVal employeeId As String)
        Try
            Dim employee As Employee = TaskEmployee.Instance.GetEmployeeForEmployeeID(employeeId)
            Dim auditor As Auditor = New Auditor()
            auditor.EmailAddress = employee.EmailAddress
            auditor.EmployeeId = employee.EmployeeId
            auditor.Name = employee.Name
            auditor.SirName = employee.SirName
            auditor.Title = employee.Title
            InternalSelection.UpdateAuditor(auditor)
        Catch ex As Exception
            Throw New Exception("Unable to update auditor.")
        End Try
    End Sub

    ''' <summary>
    ''' Update sponser
    ''' </summary>
    ''' <param name="sponsorId">Employee id</param>
    ''' <param name="sponsorName">Employee name</param>
    ''' <remarks></remarks>
    Public Sub UpdateSponsor(ByVal sponsorId As String, ByVal sponsorName As String)
        Try
            InternalSelection.UpdateSponsor(sponsorId, sponsorName)
        Catch ex As Exception
            Throw New Exception("Unable to update sponsor id and/or sponsor name.")
        End Try
    End Sub

    ''' <summary>
    ''' Update pusher
    ''' </summary>
    ''' <param name="pusherId">Employee id</param>
    ''' <param name="pusherName">Employee name</param>
    ''' <remarks></remarks>
    Public Sub UpdatePusher(ByVal pusherId As String, ByVal pusherName As String)
        Try
            InternalSelection.UpdatePusher(pusherId, pusherName)
        Catch ex As Exception
            Throw New Exception("Unable to update pusher id and/or pusher name.")
        End Try
    End Sub

    ''' <summary>
    ''' Update coordinator
    ''' </summary>
    ''' <param name="coordinatorId">Employee id</param>
    ''' <param name="coordinatorName">Employee name</param>
    ''' <remarks></remarks>
    Public Sub UpdateCoordinator(ByVal coordinatorId As String, ByVal coordinatorName As String)
        Try
            InternalSelection.UpdateCoordinator(coordinatorId, coordinatorName)
        Catch ex As Exception
            Throw New Exception("Unable to update coordinator id and/or coordinator name.")
        End Try
    End Sub

    ''' <summary>
    ''' Update tyhe budget
    ''' </summary>
    ''' <param name="budget">Budget</param>
    ''' <remarks></remarks>
    Public Sub UpdateBudget(ByVal budget As String)
        Try
            InternalSelection.UpdateBudget(budget)
        Catch ex As Exception
            Throw New Exception("Unable to update budget.")
        End Try
    End Sub

    ''' <summary>
    ''' Update why
    ''' </summary>
    ''' <param name="why">Why</param>
    ''' <remarks></remarks>
    Public Sub UpdateWhy(ByVal why As String)
        Try
            InternalSelection.UpdateWhy(why)
        Catch ex As Exception
            Throw New Exception("Unable to update why.")
        End Try
    End Sub

    ''' <summary>
    ''' Update include
    ''' </summary>
    ''' <param name="include">Include</param>
    ''' <remarks></remarks>
    Public Sub UpdateInclude(ByVal include As String)
        Try
            InternalSelection.UpdateInclude(include)
        Catch ex As Exception
            Throw New Exception("Unable to update include.")
        End Try
    End Sub

    ''' <summary>
    ''' Update exclude
    ''' </summary>
    ''' <param name="exclude">Exclude</param>
    ''' <remarks></remarks>
    Public Sub UpdateExclude(ByVal exclude As String)
        Try
            InternalSelection.UpdateExclude(exclude)
        Catch ex As Exception
            Throw New Exception("Unable to update exclude.")
        End Try
    End Sub

    ''' <summary>
    ''' Update as is
    ''' </summary>
    ''' <param name="asIs">Asis</param>
    ''' <remarks></remarks>
    Public Sub UpdateAsIs(ByVal asIs As String)
        Try
            InternalSelection.UpdateAsIs(asIs)
        Catch ex As Exception
            Throw New Exception("Unable to update asIs.")
        End Try
    End Sub

    ''' <summary>
    ''' Update to be
    ''' </summary>
    ''' <param name="toBe">To be</param>
    ''' <remarks></remarks>
    Public Sub UpdateToBe(ByVal toBe As String)
        Try
            InternalSelection.UpdateToBe(toBe)
        Catch ex As Exception
            Throw New Exception("Unable to update toBe.")
        End Try
    End Sub

    ''' <summary>
    ''' Update objectifs
    ''' </summary>
    ''' <param name="objectifs">Objectifs</param>
    ''' <remarks></remarks>
    Public Sub UpdateObjectifs(ByVal objectifs As String)
        Try
            InternalSelection.UpdateObjectifs(objectifs)
        Catch ex As Exception
            Throw New Exception("Unable to update objectifs.")
        End Try
    End Sub

    ''' <summary>
    ''' Update cost
    ''' </summary>
    ''' <param name="cost">Cost</param>
    ''' <remarks></remarks>
    Public Sub UpdateCost(ByVal cost As String)
        Try
            InternalSelection.UpdateCost(cost)
        Catch ex As Exception
            Throw New Exception("Unable to update cost.")
        End Try
    End Sub

    ''' <summary>
    ''' Update benefits
    ''' </summary>
    ''' <param name="benefits">Benefits</param>
    ''' <remarks></remarks>
    Public Sub UpdateBenefits(ByVal benefits As String)
        Try
            InternalSelection.UpdateBenefits(benefits)
        Catch ex As Exception
            Throw New Exception("Unable to update benefits.")
        End Try
    End Sub

    ''' <summary>
    ''' Update alignation
    ''' </summary>
    ''' <param name="alignation">Alignation</param>
    ''' <remarks></remarks>
    Public Sub UpdateAlignation(ByVal alignation As String)
        Try
            InternalSelection.UpdateAlignation(alignation)
        Catch ex As Exception
            Throw New Exception("Unable to update alignation.")
        End Try
    End Sub

    ''' <summary>
    ''' Update preconditoins
    ''' </summary>
    ''' <param name="preconditions">Preconditoins</param>
    ''' <remarks></remarks>
    Public Sub UpdatePreconditions(ByVal preconditions As String)
        Try
            InternalSelection.UpdatePreconditions(preconditions)
        Catch ex As Exception
            Throw New Exception("Unable to update preconditions.")
        End Try
    End Sub

    ''' <summary>
    ''' 'Update constrains
    ''' </summary>
    ''' <param name="constraints">Constrains</param>
    ''' <remarks></remarks>
    Public Sub UpdateConstraints(ByVal constraints As String)
        Try
            InternalSelection.UpdateConstraints(constraints)
        Catch ex As Exception
            Throw New Exception("Unable to update constraints.")
        End Try
    End Sub

    ''' <summary>
    ''' Update dependencies
    ''' </summary>
    ''' <param name="dependencies">Dependencies</param>
    ''' <remarks></remarks>
    Public Sub UpdateDependencies(ByVal dependencies As String)
        Try
            InternalSelection.UpdateDependencies(dependencies)
        Catch ex As Exception
            Throw New Exception("Unable to update dependencies.")
        End Try
    End Sub

    ''' <summary>
    ''' Update risk
    ''' </summary>
    ''' <param name="risk">Risk</param>
    ''' <remarks></remarks>
    Public Sub UpdateRisk(ByVal risk As String)
        Try
            InternalSelection.UpdateRisk(risk)
        Catch ex As Exception
            Throw New Exception("Unable to update risk.")
        End Try
    End Sub

    ''' <summary>
    ''' Update support
    ''' </summary>
    ''' <param name="support">Support</param>
    ''' <remarks></remarks>
    Public Sub UpdateSupport(ByVal support As String)
        Try
            InternalSelection.UpdateSupport(support)
        Catch ex As Exception
            Throw New Exception("Unable to update support.")
        End Try
    End Sub

    ''' <summary>
    ''' Update who changed the revision status
    ''' </summary>
    ''' <param name="name">Name of the person who changed the revision status</param>
    ''' <remarks></remarks>
    Public Sub UpdateRevisedBy(ByVal name As String)
        Try
            InternalSelection.UpdateRevisedBy(name)
        Catch ex As Exception
            Throw New Exception("Unable to update revised by.")
        End Try
    End Sub

    ''' <summary>
    ''' Add or update a workpackage
    ''' </summary>
    ''' <param name="workPackage">Workpackage object to add or update</param>
    ''' <remarks></remarks>
    Public Sub AddOrUpdateWorkPackage(ByVal workPackage As WorkPackage)
        Try
            InternalSelection.AddOrUpdateWorkPackage(workPackage)
        Catch ex As Exception
            Throw New Exception("Unable to add/update workpackage.")
        End Try
    End Sub

    ''' <summary>
    ''' Remove a workpackage by name
    ''' </summary>
    ''' <param name="name">Workpackage name</param>
    ''' <remarks></remarks>
    Public Sub RemoveWorkPackage(ByVal name As String)
        Try
            InternalSelection.RemoveWorkPackage(name)
        Catch ex As Exception
            Throw New Exception("Unable to remove workpackage.")
        End Try
    End Sub

    ''' <summary>
    ''' Add or update a milestone
    ''' </summary>
    ''' <param name="workPackageName">Workpackage name</param>
    ''' <param name="milestone">Milestone object to add or update</param>
    ''' <remarks></remarks>
    Public Sub AddOrUpdateMilestone(ByVal workPackageName As String, ByVal milestone As Milestone)
        Try
            InternalSelection.AddOrUpdateMilestone(workPackageName, milestone)
        Catch ex As Exception
            Throw New Exception("Unable to add/update workpackage.")
        End Try
    End Sub

    ''' <summary>
    ''' Remove a milstone from a workpackage
    ''' </summary>
    ''' <param name="workPackageName">Workpackage name</param>
    ''' <param name="milestoneName">Milestone name</param>
    ''' <remarks></remarks>
    Public Sub RemoveMilestone(ByVal workPackageName As String, ByVal milestoneName As String)
        Try
            InternalSelection.RemoveMilestone(workPackageName, milestoneName)
        Catch ex As Exception
            Throw New Exception("Unable to remove workpackage.")
        End Try
    End Sub

    ''' <summary>
    ''' Add or update a team member
    ''' </summary>
    ''' <param name="teamMember">Teammember object to add or update</param>
    ''' <remarks></remarks>
    Public Sub AddOrUpdateTeamMember(ByVal teamMember As TeamMember)
        Try
            InternalSelection.AddOrUpdateTeamMember(teamMember)
        Catch ex As Exception
            Throw New Exception("Unable to add/update teammember.")
        End Try
    End Sub

    ''' <summary>
    ''' Remove teammember by employee id
    ''' </summary>
    ''' <param name="employeeId">Employee id</param>
    ''' <remarks></remarks>
    Public Sub RemoveTeamMember(ByVal employeeId As String)
        Try
            InternalSelection.RemoveTeamMember(employeeId)
        Catch ex As Exception
            Throw New Exception("Unable to remove teammember.")
        End Try
    End Sub

    ''' <summary>
    ''' Clear the selection
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub ClearSelection()
        Try
            InternalSelection.Clear()
        Catch ex As Exception
            Throw New Exception("Unable to clear selection.")
        End Try
    End Sub

    ''' <summary>
    ''' Save the project revision
    ''' </summary>
    ''' <param name="currentProjectProposalRevision">Current project revision</param>
    ''' <param name="newProjectProposalRevision">New project revision</param>
    ''' <remarks></remarks>
    Public Sub Save(ByVal currentProjectProposalRevision As ProjectProposalRevision, ByVal newProjectProposalRevision As ProjectProposalRevision)
        Dim session As ISessionDao = Nothing
        Try
            session = Facade.Instance.RepositoryFactory.CreateSession()
            Facade.Instance.RepositoryFactory.ProjectProposalRevision.Save(session, currentProjectProposalRevision)
            Dim savedProject As ProjectProposalRevision = Facade.Instance.RepositoryFactory.ProjectProposalRevision.Save(session, newProjectProposalRevision)
            InternalSelection.Clear()
            InternalSelection.Add(savedProject)
            session.CommitAndClose()
        Catch ex As RepositoryException
            If (session IsNot Nothing) Then
                session.CancelAndClose()
            End If
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Sub

    ''' <summary>
    ''' Save the project revision
    ''' </summary>
    ''' <param name="currentProjectProposalRevision">Current project revision</param>
    ''' <remarks></remarks>
    Public Sub Save(ByVal currentProjectProposalRevision As ProjectProposalRevision)
        Dim session As ISessionDao = Nothing
        Try
            session = Facade.Instance.RepositoryFactory.CreateSession()
            Dim savedProject As ProjectProposalRevision = Facade.Instance.RepositoryFactory.ProjectProposalRevision.Save(session, currentProjectProposalRevision)
            InternalSelection.Clear()
            InternalSelection.Add(savedProject)
            session.CommitAndClose()
        Catch ex As RepositoryException
            If (session IsNot Nothing) Then
                session.CancelAndClose()
            End If
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Sub

    ''' <summary>
    ''' Get selection
    ''' </summary>
    ''' <returns>Returns the project proposal revision object</returns>
    ''' <remarks></remarks>
    Public Function GetSelection() As ProjectProposalRevision
        Return InternalSelection.GetSelection()
    End Function

    ''' <summary>
    ''' Get all project revisions for project name id
    ''' </summary>
    ''' <param name="projectProposalNameId"></param>
    ''' <param name="properties"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetAllRevisionsForProjectPrososalNameId(ByVal projectProposalNameId As Integer, ByVal properties As IList(Of String)) As IList(Of ProjectProposalRevision)
        Dim session As ISessionDao = Nothing
        Try
            session = Facade.Instance.RepositoryFactory.CreateSession()
            GetAllRevisionsForProjectPrososalNameId = Facade.Instance.RepositoryFactory.ProjectProposalRevision.GetAllRevisionsByProjectPrososalNameId(session, projectProposalNameId, properties)
            session.CancelAndClose()
        Catch ex As RepositoryException
            If (session IsNot Nothing) Then
                session.CancelAndClose()
            End If
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Function

    ''' <summary>
    ''' Get the project revison for id
    ''' </summary>
    ''' <param name="id">Project revision id</param>
    ''' <returns>Returns the requested project revision</returns>
    ''' <remarks></remarks>
    Public Function GetProjectProposalRevisionById(ByVal id As Integer) As ProjectProposalRevision
        Dim session As ISessionDao = Nothing
        Try
            session = Facade.Instance.RepositoryFactory.CreateSession()
            GetProjectProposalRevisionById = Facade.Instance.RepositoryFactory.ProjectProposalRevision.GetProjectProposalRevisionById(session, id)(0)
            session.CancelAndClose()
        Catch ex As RepositoryException
            If (session IsNot Nothing) Then
                session.CancelAndClose()
            End If
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Function

    ''' <summary>
    ''' Get the project creator
    ''' </summary>
    ''' <returns>Returns the person who created the project</returns>
    ''' <remarks></remarks>
    Public Function GetCreator() As Creator
        Try
            Return InternalSelection.GetSelection().Creator
        Catch ex As Exception
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Function

    ''' <summary>
    ''' Is the project editable
    ''' </summary>
    ''' <returns>Returns true or false</returns>
    ''' <remarks></remarks>
    Public Function IsEditable() As Boolean
        Return TaskInformation.Instance.IsCreator(InternalSelection.GetSelection().Creator) AndAlso TaskProjectProposalRevision.Instance.GetSelection().RevisionInformation.RevisionStatus.Name.Equals(TaskRevisionStatus.DRAFT) AndAlso Not TaskRevisionStatus.Instance.IsReadOnly(InternalSelection.GetSelection())
    End Function

    ''' <summary>
    ''' Is the project status reviewing
    ''' </summary>
    ''' <returns>Returns true or false</returns>
    ''' <remarks></remarks>
    Public Function IsReviewed() As Boolean
        Return TaskRevisionStatus.Instance.IsReviewing(InternalSelection.GetSelection())
    End Function

    ''' <summary>
    ''' Is the project status changed
    ''' </summary>
    ''' <returns>Returns true or false</returns>
    ''' <remarks></remarks>
    Public Function IsRevisionStatusChanged() As Boolean
        Return InternalSelection.IsRevisionStatusChanged()
    End Function

    ''' <summary>
    ''' Are the required fields filled in
    ''' </summary>
    ''' <returns>Returns true or false</returns>
    ''' <remarks></remarks>
    Public Function AreRequiredFieldsFilledIn() As Boolean
        Dim result As Boolean = InternalSelection.GetSelection().Project.Name IsNot Nothing
        result = result AndAlso InternalSelection.GetSelection().ProjectDescription IsNot Nothing
        result = result AndAlso InternalSelection.GetSelection().Creator.Name IsNot Nothing
        result = result AndAlso InternalSelection.GetSelection().Auditor.Name IsNot Nothing
        If (InternalSelection.GetSelection().SubCategory IsNot Nothing) Then
            result = result AndAlso InternalSelection.GetSelection().SubCategory.Category.Description IsNot Nothing
            result = result AndAlso InternalSelection.GetSelection().SubCategory.Description IsNot Nothing
        End If
        Return result
    End Function

    ''' <summary>
    ''' Get the names of the required field
    ''' </summary>
    ''' <returns>Returns an IList of string</returns>
    ''' <remarks></remarks>
    Public Function GetRequiredFieldNames() As IList(Of String)
        Dim result As IList(Of String) = New List(Of String)()
        result.Add("Project name")
        result.Add("Description")
        result.Add("Creator")
        result.Add("Auditor")
        If (InternalSelection.GetSelection().SubCategory IsNot Nothing) Then
            result.Add("Category")
            result.Add("SubCategory")
        End If
        Return result
    End Function

    ''' <summary>
    ''' Get workpackage by row identifier
    ''' </summary>
    ''' <param name="name">Workpackage name</param>
    ''' <returns>Returns the requested workpackage</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property GetWorkPackageByName(ByVal name As String) As WorkPackage
        Get
            Return InternalSelection.GetWorkPackageByName(name)
        End Get
    End Property

    ''' <summary>
    ''' Get all milestones for the specified workpackage
    ''' </summary>
    ''' <param name="workPackageName">WorkPackage name</param>
    ''' <returns>Returns a IList of milestones</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property GetMilestonesForWorkPackage(ByVal workPackageName As String) As IList(Of Milestone)
        Get
            Return InternalSelection.GetMilestonesForWorkPackage(workPackageName)
        End Get
    End Property

    ''' <summary>
    ''' Get the requested milestone 
    ''' </summary>
    ''' <param name="workPackageName">WorkPackage name</param>
    ''' <param name="milestoneName">Milestone name</param>
    ''' <returns>Returns the requested milestone</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property GetMilestoneByName(ByVal workpackageName As String, ByVal milestoneName As String) As Milestone
        Get
            Return InternalSelection.GetMilestoneByName(workpackageName, milestoneName)
        End Get
    End Property

    ''' <summary>
    ''' Get teammember by employee id
    ''' </summary>
    ''' <param name="employeeId">Employee id</param>
    ''' <returns>Returns the requested teammember</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property GetTeamMemberByEmployeeId(ByVal employeeId As String) As TeamMember
        Get
            Return InternalSelection.GetTeamMemberByEmployeeId(employeeId)
        End Get
    End Property

    ''' <summary>
    ''' Get all teammembers
    ''' </summary>
    ''' <returns>Returns a IList of teammmebers</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property GetAllTeamMembers() As IList(Of TeamMember)
        Get
            Return InternalSelection.GetAllTeamMembers()
        End Get
    End Property

    ''' <summary>
    ''' Get the current project revsion
    ''' </summary>
    ''' <returns>Returns the opened project revision</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property GetCurrentProjectProposalRevision() As ProjectProposalRevision
        Get
            Return InternalSelection.GetSelection()
        End Get
    End Property

    Private Property InternalSelection() As ProjectProposalRevisionSelection
        Get
            Return m_Selection
        End Get
        Set(ByVal value As ProjectProposalRevisionSelection)
            m_Selection = value
        End Set
    End Property
End Class
