﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business
Imports Securex.FIT.Fitclient.Business.DataHolders

Public Class TaskRevision
    Private Shared m_TaskRevision As TaskRevision = New TaskRevision()

    Private Sub New()
    End Sub

    Public Shared ReadOnly Property Instance() As TaskRevision
        Get
            Return m_TaskRevision
        End Get
    End Property

    ''' <summary>
    ''' Create and save a new project revision
    ''' </summary>
    ''' <param name="projectProposalNameId">Id of the project name </param>
    ''' <remarks></remarks>
    Public Sub CreateAndSaveNewRevision(ByVal projectProposalNameId As Integer)
        Dim newProjectRevision As ProjectProposalRevision = Nothing
        Try
            Dim revisions As IList(Of ProjectProposalRevision) = TaskProjectProposalRevision.Instance.GetAllRevisionsForProjectPrososalNameId(projectProposalNameId, Nothing)
            If (revisions IsNot Nothing AndAlso revisions.Count > 0) Then
                'get current revision
                Dim found As Boolean
                Dim currentProjectRevision As ProjectProposalRevision = Nothing
                Dim enumerator As IEnumerator(Of ProjectProposalRevision) = revisions.GetEnumerator()
                While (enumerator.MoveNext() AndAlso Not found)
                    If (enumerator.Current.IsCurrentRevision) Then
                        currentProjectRevision = enumerator.Current
                        found = True
                    End If
                End While
                Select Case currentProjectRevision.RevisionInformation.RevisionStatus.Name
                    Case TaskRevisionStatus.REJECTED
                        newProjectRevision = CreateNewMinorSubRevision(currentProjectRevision)
                    Case TaskRevisionStatus.RELEASED
                        newProjectRevision = CreateNewMajorRevision(currentProjectRevision)
                    Case Else
                        Throw New Exception()
                End Select
                newProjectRevision.RevisionInformation.RevisionStatus = TaskRevisionStatus.Instance.GetRevisionStatusByName(TaskRevisionStatus.DRAFT)
                newProjectRevision.RevisionInformation.Description = Nothing
                Dim employee As Employee = TaskEmployee.Instance.GetEmployeeForEmployeeID(TaskInformation.Instance.UserId)
                Dim creator As Creator = New Creator()
                creator.Name = employee.Name
                creator.SirName = employee.SirName
                creator.EmployeeId = employee.EmployeeId
                creator.EmailAddress = employee.EmailAddress
                newProjectRevision.Creator = creator
                'current
                currentProjectRevision.IsCurrentRevision = False
                TaskProjectProposalRevision.Instance.Save(currentProjectRevision, newProjectRevision)
                TaskProjectProposalRevision.Instance.ClearSelection()
            End If
        Catch ex As Exception
            Throw New Exception("There isn't a revision that meets the requirements to create a new one.")
        End Try
    End Sub

    ''' <summary>
    ''' Create and save a new project revision
    ''' </summary>
    ''' <param name="currentProjectRevision">Current project revision</param>
    ''' <remarks></remarks>
    Public Sub CreateAndSaveNewRevision(ByVal currentProjectRevision As ProjectProposalRevision)
        Dim newRevision As ProjectProposalRevision = Nothing
        Try
            Select Case currentProjectRevision.RevisionInformation.RevisionStatus.Name
                Case TaskRevisionStatus.REVIEW
                    newRevision = CreateNewMinorSubRevision(currentProjectRevision)
                    currentProjectRevision.RevisionInformation.RevisionStatus = TaskRevisionStatus.Instance.GetRevisionStatusByName(TaskRevisionStatus.DRAFT)
                Case TaskRevisionStatus.REJECTED
                    newRevision = CreateNewMinorSubRevision(currentProjectRevision)
                    currentProjectRevision.RevisionInformation.RevisionStatus = TaskRevisionStatus.Instance.GetRevisionStatusByName(TaskRevisionStatus.REVIEW)
                    currentProjectRevision.RevisionInformation.Description = Nothing
                Case TaskRevisionStatus.RELEASED
                    newRevision = CreateNewMinorRevision(currentProjectRevision)
                    currentProjectRevision.RevisionInformation.RevisionStatus = TaskRevisionStatus.Instance.GetRevisionStatusByName(TaskRevisionStatus.REVIEW)
                    currentProjectRevision.RevisionInformation.Description = Nothing
                Case TaskRevisionStatus.DRAFT
                    newRevision = currentProjectRevision
            End Select
            If (currentProjectRevision.Id = newRevision.Id) Then
                TaskProjectProposalRevision.Instance.Save(currentProjectRevision)
            Else
                'current
                currentProjectRevision.IsCurrentRevision = False
                TaskProjectProposalRevision.Instance.Save(currentProjectRevision, newRevision)
            End If
        Catch ex As Exception
            Throw New Exception("There isn't a revision that meets the requirements to create a new one.")
        End Try
    End Sub

    Private Function CreateNewMinorRevision(ByVal currentProjectRevision As ProjectProposalRevision) As ProjectProposalRevision
        Try
            Dim major As Integer = currentProjectRevision.RevisionInformation.MajorRevision
            Dim minor As Integer = currentProjectRevision.RevisionInformation.MinorRevision + 1
            Dim minorSub As Integer = currentProjectRevision.RevisionInformation.MinorSubRevision
            Return CreateNewRevision(currentProjectRevision, major, minor, minorSub)
        Catch ex As Exception
            Throw New Exception("There isn't a revision that meets the requirements to create a new one.")
        End Try
    End Function

    Private Function CreateNewMinorSubRevision(ByVal currentProjectRevision As ProjectProposalRevision) As ProjectProposalRevision
        Try
            Dim major As Integer = currentProjectRevision.RevisionInformation.MajorRevision
            Dim minor As Integer = currentProjectRevision.RevisionInformation.MinorRevision
            Dim minorSub As Integer = currentProjectRevision.RevisionInformation.MinorSubRevision + 1
            Return CreateNewRevision(currentProjectRevision, major, minor, minorSub)
        Catch ex As Exception
            Throw New Exception("There isn't a revision that meets the requirements to create a new one.")
        End Try
    End Function

    Private Function CreateNewMajorRevision(ByVal currentProjectRevision As ProjectProposalRevision) As ProjectProposalRevision
        Try
            Dim major As Integer = currentProjectRevision.RevisionInformation.MajorRevision + 1
            Return CreateNewRevision(currentProjectRevision, major, 0, 0)
        Catch ex As Exception
            Throw New Exception("There isn't a revision that meets the requirements to create a new one.")
        End Try
    End Function

    Private Function CreateNewRevision(ByVal currentProjectRevision As ProjectProposalRevision, ByVal major As Integer, ByVal minor As Integer, ByVal minorSub As Integer) As ProjectProposalRevision
        'create new revision
        Dim newProjectRevision As ProjectProposalRevision = currentProjectRevision.DeepClone()
        newProjectRevision.Id = -1
        'revision information
        newProjectRevision.RevisionInformation.Id = -1
        newProjectRevision.RevisionInformation.MajorRevision = major
        newProjectRevision.RevisionInformation.MinorRevision = minor
        newProjectRevision.RevisionInformation.MinorSubRevision = minorSub
        Dim workPackageEnumerator As IEnumerator(Of WorkPackage) = newProjectRevision.WorkPackages.GetEnumerator()
        'workpackages and milestones
        While (workPackageEnumerator.MoveNext())
            workPackageEnumerator.Current.Id = -1
            Dim mileStoneEnumerator As IEnumerator(Of Milestone) = workPackageEnumerator.Current.Milestones.GetEnumerator()
            While (mileStoneEnumerator.MoveNext())
                mileStoneEnumerator.Current.Id = -1
            End While
        End While
        'teammembers
        Dim teamMemberEnumerator As IEnumerator(Of TeamMember) = newProjectRevision.TeamMembers.GetEnumerator()
        While (teamMemberEnumerator.MoveNext())
            teamMemberEnumerator.Current.Id = -1
        End While
        'clear revised by
        newProjectRevision.RevisionInformation.RevisedBy = Nothing
        'creator
        newProjectRevision.Creator.Id = -1
        'auditor
        newProjectRevision.Auditor.Id = -1
        'reference to previous revision
        newProjectRevision.PreviousRevisionId = currentProjectRevision.Id
        'set iscurrentrevision
        newProjectRevision.IsCurrentRevision = True
        Return newProjectRevision
    End Function
End Class
