﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Presenters

Public Class FormCategory
    Implements IViewFormCategory
    Private m_Presenter As PresenterFormCategory
    Private m_FormCategoryOverview As FormCategoryOverview
    Private m_FormCategoryEditor As FormCategoryEditor
    Private m_FormSubCategoryEditor As FormSubCategoryEditor
    Private m_ChildViews As IDictionary(Of String, IChildView)
    Private Const SCREEN_NAME As String = "Category"

    Public Sub New()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        'initialize dictionary
        ChildViews = New Dictionary(Of String, IChildView)

        'initialize forms and add to list
        ChildViews.Add(PresenterFormCategory.CATEGORY_OVERVIEW, New FormCategoryOverview())
        ChildViews.Add(PresenterFormCategory.CATEGORY_EDITOR, CategoryEditor)
        ChildViews.Add(PresenterFormCategory.SUB_CATEGORY_EDITOR, SubCategoryEditor)

        'initialize presenter
        Presenter = New PresenterFormCategory(Me, ChildViews)
    End Sub

    Private Property Presenter() As PresenterFormCategory
        Get
            Return m_Presenter
        End Get
        Set(ByVal value As PresenterFormCategory)
            m_Presenter = value
        End Set
    End Property

    Private Property ChildViews() As IDictionary(Of String, IChildView)
        Get
            Return m_ChildViews
        End Get
        Set(ByVal value As IDictionary(Of String, IChildView))
            m_ChildViews = value
        End Set
    End Property

    Private Property CategoryEditor() As FormCategoryEditor
        Get
            Return m_FormCategoryEditor
        End Get
        Set(ByVal value As FormCategoryEditor)
            m_FormCategoryEditor = value
        End Set
    End Property

    Private Property SubCategoryEditor() As FormSubCategoryEditor
        Get
            Return m_FormSubCategoryEditor
        End Get
        Set(ByVal value As FormSubCategoryEditor)
            m_FormSubCategoryEditor = value
        End Set
    End Property

#Region "IViewFormCategory"
    Public Function GetChildPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetChildPanel
        Return PanelChild
    End Function

    Public Function GetMainPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetMainPanel
        Return PanelMain
    End Function

    Public Function GetText() As String Implements Presenters.IChildView.GetText
        Return SCREEN_NAME
    End Function

    Public Function GetToolstrip() As System.Windows.Forms.ToolStrip Implements Presenters.IChildView.GetToolstrip
        Return ToolStripMain
    End Function

    Public Sub ShowChild() Implements Presenters.IChildView.ShowChild
        Show()
    End Sub

    Public Sub UpdateTopLevel(ByVal value As Boolean) Implements Presenters.IChildView.UpdateTopLevel
        SetTopLevel(value)
    End Sub
#Region "IViewFormCategory"
    Public Sub CloseCategoryEditor() Implements Presenters.IViewFormCategory.CloseCategoryEditor
        CategoryEditor = Nothing
    End Sub

    Public Sub CreateCategoryEditor(ByVal category As Business.DataHolders.Category) Implements Presenters.IViewFormCategory.CreateCategoryEditor
        If (category IsNot Nothing) Then
            CategoryEditor = New FormCategoryEditor(category)
        Else
            CategoryEditor = New FormCategoryEditor()
        End If
    End Sub

    Public Function GetCategoryEditor() As Presenters.IViewFormCategoryEditor Implements Presenters.IViewFormCategory.GetCategoryEditor
        Return CategoryEditor
    End Function

    Public Sub CloseSubCategoryEditor() Implements Presenters.IViewFormCategory.CloseSubCategoryEditor
        SubCategoryEditor = Nothing
    End Sub

    Public Sub CreateSubCategoryEditor(ByVal subCategory As Business.DataHolders.SubCategory) Implements Presenters.IViewFormCategory.CreateSubCategoryEditor
        If (subCategory IsNot Nothing) Then
            SubCategoryEditor = New FormSubCategoryEditor(subCategory)
        Else
            SubCategoryEditor = New FormSubCategoryEditor()
        End If
    End Sub

    Public Function GetSubCategoryEditor() As Presenters.IViewFormSubCategoryEditor Implements Presenters.IViewFormCategory.GetSubCategoryEditor
        Return SubCategoryEditor
    End Function
#End Region

#End Region
End Class