﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports Securex.FIT.Fitclient.Presenters

Public Class FormMilestoneEditor
    Implements IViewFormMilestoneEditor
    Private m_Presenter As PresenterFormMilestoneEditor
    Private Const SCREEN_NAME As String = "Milestone Editor"

    Public Sub New(ByVal workPackageName As String)
        Initialize(workPackageName)
    End Sub

    Public Sub New(ByVal workPackageName As String, ByVal milestone As Milestone)
        Initialize(workPackageName)
        Presenter.SetSelection(milestone)
        UpdateView(milestone)
    End Sub

    Private Sub Initialize(ByVal workPackageName As String)
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        Presenter = New PresenterFormMilestoneEditor(Me, workPackageName)

        'disable ToolStripButtonApply
        ToolStripButtonApplyEnabled(False)
    End Sub

    Private Sub TextBoxName_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TextBoxName.TextChanged
        Presenter.UpdateName(TextBoxName.Text)
        ToolStripButtonApplyEnabled(RequiredFieldsAreFilledIn())
    End Sub

    Private Sub TextBoxDescription_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TextBoxDescription.TextChanged
        Presenter.UpdateDescription(TextBoxDescription.Text)
        ToolStripButtonApplyEnabled(RequiredFieldsAreFilledIn())
    End Sub

    Private Sub DateTimePickerBeginDate_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DateTimePickerBeginDate.ValueChanged
        Presenter.UpdateBeginDate(DateTimePickerBeginDate.Value)
        ToolStripButtonApplyEnabled(RequiredFieldsAreFilledIn())
    End Sub

    Private Sub DateTimePickerEndDate_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DateTimePickerEndDate.ValueChanged
        Presenter.UpdateEndDate(DateTimePickerEndDate.Value)
        ToolStripButtonApplyEnabled(RequiredFieldsAreFilledIn())
    End Sub

    Private Sub ToolStripButtonApply_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonApply.Click
        Presenter.UpdateMilestone()
        Presenter.Apply()
    End Sub

    Private Sub ToolStripButtonCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonCancel.Click
        Presenter.Cancel()
    End Sub

    Private Sub ToolStripButtonApplyEnabled(ByVal enabled As Boolean)
        ToolStripButtonApply.Enabled = enabled
    End Sub

    Private Function RequiredFieldsAreFilledIn() As Boolean
        Return TextBoxName.Text.Trim() IsNot Nothing AndAlso TextBoxDescription.Text.Trim() IsNot Nothing
    End Function

    Private Property Presenter() As PresenterFormMilestoneEditor
        Get
            Return m_Presenter
        End Get
        Set(ByVal value As PresenterFormMilestoneEditor)
            m_Presenter = value
        End Set
    End Property

#Region "IViewFormMilestoneEditor"
    Public Function GetChildPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetChildPanel
        Return Nothing
    End Function

    Public Function GetText() As String Implements Presenters.IChildView.GetText
        Return SCREEN_NAME
    End Function

    Public Sub ShowChild() Implements Presenters.IChildView.ShowChild
        Show()
        Presenter.SendChangedViewEvent()
    End Sub

    Public Sub UpdateTopLevel(ByVal value As Boolean) Implements Presenters.IChildView.UpdateTopLevel
        SetTopLevel(value)
    End Sub

    Public Function GetMainPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetMainPanel
        Return PanelMain
    End Function

    Public Function GetToolstrip() As System.Windows.Forms.ToolStrip Implements Presenters.IChildView.GetToolstrip
        Return Nothing
    End Function

    Public Sub UpdateView(ByVal milestone As Business.DataHolders.Milestone) Implements Presenters.IViewFormMilestoneEditor.UpdateView
        TextBoxName.Text = milestone.Name
        TextBoxDescription.Text = milestone.Description
        DateTimePickerBeginDate.Value = milestone.BeginDate
        DateTimePickerEndDate.Value = milestone.EndDate
    End Sub
#End Region

End Class