﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Presenters
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Drawing
Imports System.Windows.Forms
Imports System.Text

Public Class FormProjectEditor
    Implements IViewFormProjectEditor
    Private m_Presenter As PresenterFormProjectEditor
    Private m_FormProjectScreen1 As FormProjectScreen1
    Private m_FormProjectScreen2 As FormProjectScreen2
    Private m_FormProjectScreen3 As FormProjectScreen3
    Private m_FormProjectScreen4 As FormProjectScreen4
    Private m_FormProjectScreen5 As FormProjectScreen5
    Private m_FormProjectScreen6 As FormProjectScreen6
    Private m_FormProjetccreenAuditor As FormProjectScreenAuditor
    Private m_ProjectScreenList As IList(Of IChildView)
    Private Const SCREEN_NAME As String = "Project Editor"

    Public Sub New(ByVal creatorId As String)
        'new project
        Initialize()
        Presenter.CreateNewProject(creatorId)
        InitializeForms()
    End Sub

    Public Sub New(ByVal project As ProjectProposalRevision)
        'edit existing project
        Initialize()
        Presenter.SetSelection(project)
        InitializeForms()
    End Sub

    Private Sub Initialize()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        'initialize presenter
        Presenter = New PresenterFormProjectEditor(Me)
    End Sub

    Private Sub InitializeForms()
        Dim isAuditor As Boolean = Presenter.IsAuditor()

        'keep this before the initialization of the child forms as the PresenterProjectScreenAuditor send an event on initiazation that overwrites this setting
        EnableToolStripButtonsSaveAndCancel(((Presenter.IsCreator AndAlso Not Presenter.IsReviewing) OrElse isAuditor) AndAlso Not Presenter.IsReadOnly)

        'initialize list
        ProjectScreenList = New List(Of IChildView)

        'initialize forms and add to list
        If (isAuditor) Then
            ProjectScreenList.Add(New FormProjectScreenAuditor())
        End If
        ProjectScreenList.Add(New FormProjectScreen1(isAuditor))
        ProjectScreenList.Add(New FormProjectScreen2())
        ProjectScreenList.Add(New FormProjectScreen3())
        ProjectScreenList.Add(New FormProjectScreen4())
        ProjectScreenList.Add(New FormProjectScreen5())
        ProjectScreenList.Add(New FormProjectScreen6())

        Presenter.AddChilViews(ProjectScreenList)

        'show 1st form in ProjectScreenList
        Presenter.Next()
    End Sub

    Private Sub ButtonPrevious_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonPrevious.Click
        Cursor.Current = Cursors.WaitCursor
        Presenter.Previous()
        Cursor.Current = Cursors.Default
    End Sub

    Private Sub ButtonNext_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonNext.Click
        Cursor.Current = Cursors.WaitCursor
        Presenter.Next()
        Cursor.Current = Cursors.Default
    End Sub

    Private Sub ToolStripButtonSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonSave.Click
        Try
            If (Presenter.AreRequiredFieldsFilledIn()) Then
                Cursor.Current = Cursors.WaitCursor
                Presenter.Save()
                ShowChild()
            Else
                Dim stringBuilder As StringBuilder = New StringBuilder()
                Dim enumerator As IEnumerator(Of String) = Presenter.GetRequiredFieldNames().GetEnumerator()
                While (enumerator.MoveNext())
                    stringBuilder.Append(String.Format("{0}, ", enumerator.Current))
                End While
                MessageBox.Show(String.Format("Mandatory fields are: {0}", stringBuilder.ToString().Substring(0, stringBuilder.ToString().Length - 2)))
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        Finally
            Cursor.Current = Cursors.Default
        End Try
    End Sub

    Private Sub ToolStripButtonCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonCancel.Click
        Try
            Cursor.Current = Cursors.WaitCursor
            Presenter.Cancel()
        Catch ex As Exception
            'ignore
        Finally
            Cursor.Current = Cursors.Default
        End Try
    End Sub

    Private Sub ToolStripButtonClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonClose.Click
        Try
            Cursor.Current = Cursors.WaitCursor
            CloseForm()
        Catch ex As Exception
            'ignore
        Finally
            Cursor.Current = Cursors.Default
        End Try
    End Sub

    Private Sub CloseForm()
        Presenter.Close()
    End Sub

    Private Property Presenter() As PresenterFormProjectEditor
        Get
            Return m_Presenter
        End Get
        Set(ByVal value As PresenterFormProjectEditor)
            m_Presenter = value
        End Set
    End Property

    Private Property ProjectScreenList() As IList(Of IChildView)
        Get
            Return m_ProjectScreenList
        End Get
        Set(ByVal value As IList(Of IChildView))
            m_ProjectScreenList = value
        End Set
    End Property

    Private Sub PanelNavigation_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles PanelNavigation.Resize
        PanelNavigationControls.Location = New Point((GetMainPanel().Width - PanelNavigationControls.Width) \ 2, PanelNavigationControls.Location.Y)
    End Sub

#Region "IViewFormProjectEditor"
    Public Function GetMainPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetMainPanel
        Return PanelFormProjectEditor
    End Function

    Public Function GetToolstrip() As System.Windows.Forms.ToolStrip Implements Presenters.IChildView.GetToolstrip
        Return ToolStripFormProjectEditor
    End Function

    Public Sub UpdateTopLevel(ByVal value As Boolean) Implements Presenters.IChildView.UpdateTopLevel
        SetTopLevel(value)
    End Sub

    Public Sub ShowChild() Implements Presenters.IChildView.ShowChild
        Show()
        Presenter.SendChangedViewEvent()
    End Sub

    Public Function GetChildPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetChildPanel
        Return PanelCurrentPage
    End Function

    Public Function GetText() As String Implements Presenters.IChildView.GetText
        Return SCREEN_NAME
    End Function

    Public Sub UpdateLabelCurrentPage(ByVal value As String) Implements Presenters.IViewFormProjectEditor.UpdateLabelCurrentPage
        LabelCurrentPage.Text = value
    End Sub

    Public Sub UpdateTextboxDescription(ByVal description As String) Implements Presenters.IViewFormProjectEditor.UpdateTextboxDescription
        TextBoxDescription.Text = description
    End Sub

    Protected Sub EnableToolStripButtonsSaveAndCancel(ByVal value As Boolean) Implements Presenters.IViewFormProjectEditor.EnableToolStripButtonsSaveAndCancel
        ToolStripButtonSave.Enabled = value
        ToolStripButtonCancel.Enabled = value
    End Sub
#End Region

End Class