﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Presenters
Imports System.Windows.Forms

Public Class FormProjectScreen6
    Implements IViewProjectScreen6, IViewDescription
    Private m_Presenter As PresenterFormProjectScreen6
    Private Const SCREEN_NAME As String = "Screen 6"

    Public Sub New()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        'initialize presenter
        Presenter = New PresenterFormProjectScreen6(Me)
    End Sub

    Private Sub ToolStripButtonShowPreview_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonShowPreview.Click
        Try
            Cursor.Current = Cursors.WaitCursor
            WebBrowserPreview.DocumentText = Presenter.GetPreview()
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        Finally
            Cursor.Current = Cursors.Default
        End Try
    End Sub

    Private Sub ToolStripButtonSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonSave.Click
        Try
            Cursor.Current = Cursors.WaitCursor
            Dim saveDialog As SaveFileDialog = New SaveFileDialog()
            saveDialog.Filter = "*.docx|*.docx"
            If (saveDialog.ShowDialog() = Windows.Forms.DialogResult.OK) Then
                Presenter.Save(saveDialog.FileName)
            End If
            MessageBox.Show("Document succesfully saved.")
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        Finally
            Cursor.Current = Cursors.Default
        End Try
    End Sub

    Private Sub ToolStripButtonShowChanges_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonShowChanges.Click
        Try
            Cursor.Current = Cursors.WaitCursor
            WebBrowserPreview.DocumentText = Presenter.ShowChanges()
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        Finally
            Cursor.Current = Cursors.Default
        End Try
    End Sub

    Private Sub IsEditable(ByVal value As Boolean)
        ToolStripButtonShowChanges.Enabled = value
    End Sub

    Private Sub ShowErrorMessage(ByVal message As String)
        MessageBox.Show(message)
    End Sub

    Private Property Presenter() As PresenterFormProjectScreen6
        Get
            Return m_Presenter
        End Get
        Set(ByVal value As PresenterFormProjectScreen6)
            m_Presenter = value
        End Set
    End Property

#Region "IViewProjectScreen6"
    Public Function GetChildPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetChildPanel
        Return Nothing
    End Function

    Public Sub ShowChild() Implements Presenters.IChildView.ShowChild
        Try
            Show()
            Presenter.SendChangedViewEvent()
            Presenter.RequestSelection()
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Public Sub UpdateTopLevel(ByVal value As Boolean) Implements Presenters.IChildView.UpdateTopLevel
        SetTopLevel(value)
    End Sub

    Public Function GetMainPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetMainPanel
        Return PanelMain
    End Function

    Public Function GetToolstrip() As System.Windows.Forms.ToolStrip Implements Presenters.IChildView.GetToolstrip
        Return Nothing
    End Function

    Public Function GetText() As String Implements Presenters.IChildView.GetText
        Return SCREEN_NAME
    End Function

    Public Function GetName() As String Implements Presenters.IViewFormProjectEditorChild.GetName
        Return Name
    End Function

    Public Sub UpdateView() Implements Presenters.IViewProjectScreen6.UpdateView
        IsEditable(Presenter.IsEditable)
    End Sub
#End Region

End Class