﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Presenters

Public Class FormRole
    Implements IViewFormRole
    Private m_Presenter As PresenterFormRole
    Private m_FormRoleOverview As FormRoleOverview
    Private m_FormRoleEditor As FormRoleEditor
    Private m_ChildViews As IDictionary(Of String, IChildView)
    Private Const SCREEN_NAME As String = "Role"

    Public Sub New()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        'initialize dictionary
        ChildViews = New Dictionary(Of String, IChildView)

        'initialize forms and add to list
        ChildViews.Add(PresenterFormRole.ROLE_OVERVIEW, New FormRoleOverview())
        ChildViews.Add(PresenterFormRole.ROLE_EDITOR, RoleEditor)

        'initialize presenter
        Presenter = New PresenterFormRole(Me, ChildViews)
    End Sub

    Private Property Presenter() As PresenterFormRole
        Get
            Return m_Presenter
        End Get
        Set(ByVal value As PresenterFormRole)
            m_Presenter = value
        End Set
    End Property

    Private Property ChildViews() As IDictionary(Of String, IChildView)
        Get
            Return m_ChildViews
        End Get
        Set(ByVal value As IDictionary(Of String, IChildView))
            m_ChildViews = value
        End Set
    End Property

    Private Property RoleEditor() As FormRoleEditor
        Get
            Return m_FormRoleEditor
        End Get
        Set(ByVal value As FormRoleEditor)
            m_FormRoleEditor = value
        End Set
    End Property

#Region "IViewFormCategory"
    Public Function GetChildPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetChildPanel
        Return PanelChild
    End Function

    Public Function GetText() As String Implements Presenters.IChildView.GetText
        Return SCREEN_NAME
    End Function

    Public Sub ShowChild() Implements Presenters.IChildView.ShowChild
        Show()
    End Sub

    Public Sub UpdateTopLevel(ByVal value As Boolean) Implements Presenters.IChildView.UpdateTopLevel
        SetTopLevel(value)
    End Sub

    Public Function GetMainPanel() As System.Windows.Forms.Panel Implements Presenters.IForm.GetMainPanel
        Return PanelMain
    End Function

    Public Function GetToolstrip() As System.Windows.Forms.ToolStrip Implements Presenters.IForm.GetToolstrip
        Return ToolStripMain
    End Function

    Public Sub CloseRoleEditor() Implements Presenters.IViewFormRole.CloseRoleEditor
        RoleEditor = Nothing
    End Sub

    Public Sub CreateRoleEditor(ByVal role As Business.DataHolders.Role) Implements Presenters.IViewFormRole.CreateRoleEditor
        If (role IsNot Nothing) Then
            RoleEditor = New FormRoleEditor(role)
        Else
            RoleEditor = New FormRoleEditor()
        End If
    End Sub

    Public Function GetRoleEditor() As Presenters.IViewFormRoleEditor Implements Presenters.IViewFormRole.GetRoleEditor
        Return RoleEditor
    End Function
#End Region
End Class