﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Presenters
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Windows.Forms

Public Class FormSubCategoryEditor
    Implements IViewFormSubCategoryEditor
    Private m_Presenter As PresenterFormSubCategoryEditor
    Private Const SCREEN_NAME As String = "Subcategory Editor"

    Public Sub New()
        Initialize()
    End Sub

    Public Sub New(ByVal subCategory As SubCategory)
        Initialize()
        Presenter.SetSelection(subCategory)
        UpdateView(subCategory)
    End Sub

    Private Sub Initialize()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        Presenter = New PresenterFormSubCategoryEditor(Me)

        'disable ToolStripButtonSave
        ToolStripButtonSaveEnabled(False)
    End Sub

    Private Sub ToolStripButtonSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonSave.Click
        Try
            Presenter.UpdateSelelection(TextBoxSubCategory.Text)
            Presenter.Save()
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try
    End Sub

    Private Sub ToolStripButtonCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonCancel.Click
        Presenter.Cancel()
    End Sub

    Private Sub TextBoxSubCategory_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles TextBoxSubCategory.TextChanged
        ToolStripButtonSaveEnabled(RequiredFieldsAreFilledIn())
    End Sub

    Private Sub ToolStripButtonSaveEnabled(ByVal enabled As Boolean)
        ToolStripButtonSave.Enabled = enabled
    End Sub

    Private Function RequiredFieldsAreFilledIn() As Boolean
        Return (TextBoxSubCategory.Text.Trim() IsNot Nothing) AndAlso (TextBoxSubCategory.Text.Trim().Length > 0)
    End Function

    Private Property Presenter() As PresenterFormSubCategoryEditor
        Get
            Return m_Presenter
        End Get
        Set(ByVal value As PresenterFormSubCategoryEditor)
            m_Presenter = value
        End Set
    End Property

#Region "IViewFormSubCategoryEditor"
    Public Function GetChildPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetChildPanel
        Return Nothing
    End Function

    Public Function GetMainPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetMainPanel
        Return PanelMain
    End Function

    Public Function GetText() As String Implements Presenters.IChildView.GetText
        Return SCREEN_NAME
    End Function

    Public Function GetToolstrip() As System.Windows.Forms.ToolStrip Implements Presenters.IChildView.GetToolstrip
        Return Nothing
    End Function

    Public Sub ShowChild() Implements Presenters.IChildView.ShowChild
        Show()
        Presenter.SendChangedViewEvent()
    End Sub

    Public Sub UpdateTopLevel(ByVal value As Boolean) Implements Presenters.IChildView.UpdateTopLevel
        SetTopLevel(value)
    End Sub

#Region "IViewFormSubCategoryEditor"
    Public Sub ResetView() Implements Presenters.IViewFormSubCategoryEditor.ResetView
        TextBoxSubCategory.Clear()
        TextBoxSubCategory.Tag = 0
    End Sub

    Public Sub UpdateView(ByVal subCategory As Business.DataHolders.SubCategory) Implements Presenters.IViewFormSubCategoryEditor.UpdateView
        LabelCategory.Text = subCategory.Category.Description
        TextBoxSubCategory.Text = subCategory.Description
        TextBoxSubCategory.Tag = subCategory.Id
    End Sub
#End Region

#End Region

End Class