﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Presenters
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Windows.Forms
Imports CustomControls

Public Class FormTeamMemberEditor
    Implements IViewFormTeamMemberEditor
    Private m_Presenter As PresenterFormTeamMemberEditor
    Private Const SCREEN_NAME As String = "Teammember Editor"

    Public Sub New()
        Initialize()
    End Sub

    Public Sub New(ByVal teamMember As TeamMember)
        Initialize()
        Presenter.SetSelection(teamMember)
        UpdateView(teamMember)
    End Sub

    Private Sub Initialize()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        'add ListViewComboBoxCoordinator listview handler
        AddHandler ListViewComboBoxTeamMember.ListView.KeyPress, AddressOf ListViewComboBoxTeamMember_KeyPress

        'add  ListViewComboBoxReportTo listview handler
        AddHandler ListViewComboBoxReportTo.ListView.KeyPress, AddressOf ListViewComboBoxReportTo_KeyPress

        'clear text
        LabelEmployeeName.Text = Nothing
        LabelDepartment.Text = Nothing
        LabelSubDepartment.Text = Nothing
        LabelEmailAddress.Text = Nothing

        Presenter = New PresenterFormTeamMemberEditor(Me)

        'disable ToolStripButtonApply
        ToolStripButtonApplyEnabled(False)
    End Sub

    Private Sub ToolStripButtonApply_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonApply.Click
        Presenter.UpdateTeamMember()
        Presenter.Apply()
    End Sub

    Private Sub ToolStripButtonCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonCancel.Click
        Presenter.Cancel()
    End Sub

    Private Sub ListViewComboBoxTeamMember_DropDown(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxTeamMember.DropDown
        ListViewComboBoxTeamMember.ListView.BeginUpdate()
        Presenter.RequestAllEmployeesForTeamMember()
        ListViewComboBoxTeamMember.ListView.EndUpdate()
    End Sub

    Private Sub ListViewComboBoxTeamMember_SelectionChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxTeamMember.SelectionChanged
        Dim listViewItem As ListViewItem = ListViewComboBoxTeamMember.SelectedListViewitem
        LabelEmployeeName.Text = listViewItem.SubItems(1).Text
        LabelDepartment.Text = listViewItem.SubItems(2).Text
        LabelSubDepartment.Text = listViewItem.SubItems(3).Text
        LabelEmailAddress.Text = listViewItem.SubItems(4).Text
        Presenter.UpdateTeamMemberInfo(ListViewComboBoxTeamMember.Text, LabelEmployeeName.Text, LabelDepartment.Text, LabelSubDepartment.Text, LabelEmailAddress.Text)
        ToolStripButtonApplyEnabled(RequiredFieldsAreFilledIn())
    End Sub

    Private Sub ListViewComboBoxTeamMember_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs)
        ListViewComboBoxKeyPressed(ListViewComboBoxTeamMember)
    End Sub

    Private Sub ListViewComboBoxRole_DropDown(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxRole.DropDown
        ListViewComboBoxRole.ListView.BeginUpdate()
        Presenter.RequestAllRoles()
        ListViewComboBoxRole.ListView.EndUpdate()
    End Sub

    Private Sub ListViewComboBoxRole_SelectionChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxRole.SelectionChanged
        Dim listViewItem As ListViewItem = ListViewComboBoxRole.SelectedListViewitem
        Presenter.UpdateRole(Convert.ToInt32(listViewItem.Tag))
        ToolStripButtonApplyEnabled(RequiredFieldsAreFilledIn())
    End Sub

    Private Sub ListViewComboBoxReportTo_DropDown(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxReportTo.DropDown
        ListViewComboBoxReportTo.ListView.BeginUpdate()
        Presenter.RequestAllTeamMembersForReportTo()
        ListViewComboBoxReportTo.ListView.EndUpdate()
    End Sub

    Private Sub ListViewComboBoxReportTo_SelectionChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxReportTo.SelectionChanged
        Dim listViewItem As ListViewItem = ListViewComboBoxReportTo.SelectedListViewitem
        Presenter.UpdateReportTo(listViewItem.Tag.ToString())
        ToolStripButtonApplyEnabled(RequiredFieldsAreFilledIn())
    End Sub

    Private Sub ListViewComboBoxReportTo_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs)
        ListViewComboBoxKeyPressed(ListViewComboBoxReportTo)
    End Sub

    Private Sub ListViewComboBoxKeyPressed(ByVal listviewCombobox As ListViewComboBox)
        Try
            Dim listViewItem As ListViewItem = listviewCombobox.ListView.FindItemWithText(listviewCombobox.GetSearchString(), True, 0)
            If (listViewItem IsNot Nothing) Then
                listviewCombobox.ListView.Items(listViewItem.Index).Selected = True
                listviewCombobox.ListView.Items(listViewItem.Index).EnsureVisible()
            End If
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub ShowErrorMessage(ByVal message As String)
        MessageBox.Show(message)
    End Sub

    Private Sub ToolStripButtonApplyEnabled(ByVal enabled As Boolean)
        ToolStripButtonApply.Enabled = enabled
    End Sub

    Private Function RequiredFieldsAreFilledIn() As Boolean
        Return ListViewComboBoxTeamMember.Text IsNot Nothing
    End Function

    Private Property Presenter() As PresenterFormTeamMemberEditor
        Get
            Return m_Presenter
        End Get
        Set(ByVal value As PresenterFormTeamMemberEditor)
            m_Presenter = value
        End Set
    End Property

#Region "IViewFormWorkPackageEditor"
    Public Function GetChildPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetChildPanel
        Return Nothing
    End Function

    Public Function GetMainPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetMainPanel
        Return PanelMain
    End Function

    Public Function GetText() As String Implements Presenters.IChildView.GetText
        Return SCREEN_NAME
    End Function

    Public Function GetToolstrip() As System.Windows.Forms.ToolStrip Implements Presenters.IChildView.GetToolstrip
        Return Nothing
    End Function

    Public Sub ShowChild() Implements Presenters.IChildView.ShowChild
        Show()
        Presenter.SendChangedViewEvent()
    End Sub

    Public Sub UpdateTopLevel(ByVal value As Boolean) Implements Presenters.IChildView.UpdateTopLevel
        SetTopLevel(value)
    End Sub

    Public Sub UpdateView(ByVal teamMember As Business.DataHolders.TeamMember) Implements Presenters.IViewFormTeamMemberEditor.UpdateView
        ListViewComboBoxTeamMember.Text = teamMember.EmployeeId
        LabelEmployeeName.Text = teamMember.EmployeeName
        LabelDepartment.Text = teamMember.Department
        LabelSubDepartment.Text = teamMember.SubDepartment
        If (teamMember.ReportTo IsNot Nothing) Then
            ListViewComboBoxReportTo.Text = teamMember.ReportTo.EmployeeName
        End If
        ListViewComboBoxRole.Text = teamMember.Role.Name
    End Sub

    Public Sub UpdateTeamMember() Implements Presenters.IViewFormTeamMemberEditor.UpdateTeamMember
        ListViewComboBoxTeamMember.Invalidate()
        ListViewComboBoxTeamMember.ResumeLayout()
    End Sub

    Public Sub UpdateReportTo() Implements Presenters.IViewFormTeamMemberEditor.UpdateReportTo
        ListViewComboBoxReportTo.Invalidate()
    End Sub

    Public Sub UpdateRole() Implements Presenters.IViewFormTeamMemberEditor.UpdateRole
        ListViewComboBoxRole.Invalidate()
    End Sub

    Public Sub ClearListViewComboBoxReportTo() Implements Presenters.IViewFormTeamMemberEditor.ClearListViewComboBoxReportTo
        ListViewComboBoxReportTo.ListView.VirtualModeCache.Clear()
    End Sub

    Public Sub AddToListViewComboBoxReportTo(ByVal fourStateListViewItems As IList(Of CustomControls.FourStateListViewItem)) Implements Presenters.IViewFormTeamMemberEditor.AddToListViewComboBoxReportTo
        ListViewComboBoxReportTo.ListView.VirtualModeCache.AddRange(fourStateListViewItems)
    End Sub

    Public Sub ClearListViewComboBoxRole() Implements Presenters.IViewFormTeamMemberEditor.ClearListViewComboBoxRole
        ListViewComboBoxRole.ListView.VirtualModeCache.Clear()
    End Sub

    Public Sub AddToListViewComboBoxRole(ByVal fourStateListViewItem As CustomControls.FourStateListViewItem) Implements Presenters.IViewFormTeamMemberEditor.AddToListViewComboBoxRole
        ListViewComboBoxRole.ListView.VirtualModeCache.Add(fourStateListViewItem)
    End Sub

    Public Sub ClearListViewComboBoxTeamMember() Implements Presenters.IViewFormTeamMemberEditor.ClearListViewComboBoxTeamMember
        ListViewComboBoxTeamMember.ListView.VirtualModeCache.Clear()
    End Sub

    Public Sub AddToListViewComboBoxTeamMember(ByVal fourStateListViewItems As IList(Of CustomControls.FourStateListViewItem)) Implements Presenters.IViewFormTeamMemberEditor.AddToListViewComboBoxTeamMember
        ListViewComboBoxTeamMember.ListView.VirtualModeCache.AddRange(fourStateListViewItems)
    End Sub
#End Region

End Class