﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Presenters
Imports System.Drawing
Imports System.Windows.Forms
Imports CustomControls

Public Class FormTiming_Team
    Implements IViewFormTiming_Team
    Private m_Presenter As PresenterFormTiming_Team
    Private m_MouseLocationWorkPackage As Point
    Private m_MouseLocationMilestone As Point
    Private m_MouseLocationTeamMember As Point
    Private m_ReadOnlyMode As Boolean
    Private Const SCREEN_NAME As String = "Timing - Team"

    Public Sub New()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        'initialize presenter
        Presenter = New PresenterFormTiming_Team(Me)
    End Sub

    Private Sub ToolStripButtonAddWorkPackage_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonAddWorkPackage.Click
        OpenWorkPackageEditor(Nothing)
    End Sub

    Private Sub ToolStripButtonRemoveWorkPackage_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonRemoveWorkPackage.Click
        Try
            Dim listViewItem As ListViewItem = ListViewWorkPackages.GetItemAt(MouseLocationWorkPackage.X, MouseLocationWorkPackage.Y)
            If (listViewItem IsNot Nothing) Then
                If (MessageBox.Show("Remove selected data?", "Question", MessageBoxButtons.YesNo) = Windows.Forms.DialogResult.Yes) Then
                    Presenter.RemoveWorkPackage(listViewItem.Text)
                    Presenter.SelectedWorkPackageName = String.Empty
                    Presenter.RequestAllWorkPackages()
                    Presenter.RequestMilestonesForWorkPackage(Presenter.SelectedWorkPackageName)
                End If
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try
    End Sub

    Private Sub ToolStripButtonAddMilestone_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonAddMilestone.Click
        OpenMilestoneEditor(Nothing)
    End Sub

    Private Sub ToolStripButtonAddTeamMember_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonAddTeamMember.Click
        OpenTeamMemberEditor(Nothing)
    End Sub

    Private Sub ToolStripButtonRemoveTeamMember_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonRemoveTeamMember.Click
        Try
            Dim listViewItem As ListViewItem = ListViewTeamMembers.GetItemAt(MouseLocationTeamMember.X, MouseLocationTeamMember.Y)
            If (listViewItem IsNot Nothing) Then
                If (MessageBox.Show("Remove selected data?", "Question", MessageBoxButtons.YesNo) = Windows.Forms.DialogResult.Yes) Then
                    Presenter.RemoveTeamMember(listViewItem.Text)
                    Presenter.RequestAllTeamMembers()
                End If
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try
    End Sub

    Private Sub OpenWorkPackageEditor(ByVal workPackageName As String)
        Presenter.ShowWorkPackageEditor(workPackageName)
    End Sub

    Private Sub OpenMilestoneEditor(ByVal milestoneName As String)
        Presenter.ShowMilestoneEditor(milestoneName)
    End Sub

    Private Sub OpenTeamMemberEditor(ByVal employeeId As String)
        Presenter.ShowTeamMemberEditor(employeeId)
    End Sub

    Private Sub ToolStripButtonAddWorkPackageEnabled(ByVal value As Boolean)
        ToolStripButtonAddWorkPackage.Enabled = value
    End Sub

    Private Sub ToolStripButtonRemoveWorkPackageEnabled(ByVal value As Boolean)
        ToolStripButtonRemoveWorkPackage.Enabled = value
    End Sub

    Private Sub ToolStripButtonAddMileStoneEnabled(ByVal value As Boolean)
        ToolStripButtonAddMilestone.Enabled = value
    End Sub

    Private Sub ToolStripButtonRemoveMileStoneEnabled(ByVal value As Boolean)
        ToolStripButtonRemoveMilestone.Enabled = value
    End Sub

    Private Sub ToolStripButtonAddTeamMemberEnabled(ByVal value As Boolean)
        ToolStripButtonAddTeamMember.Enabled = value
    End Sub

    Private Sub ToolStripButtonRemoveTeamMemberEnabled(ByVal value As Boolean)
        ToolStripButtonRemoveTeamMember.Enabled = value
    End Sub

    Private Sub ListViewWorkPackage_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewWorkPackages.DoubleClick
        If Not (IsReadOnlyMode) Then
            Try
                Dim listViewItem As ListViewItem = ListViewWorkPackages.GetItemAt(MouseLocationWorkPackage.X, MouseLocationWorkPackage.Y)
                If (listViewItem IsNot Nothing) Then
                    OpenWorkPackageEditor(listViewItem.Text)
                End If
            Catch ex As Exception
                ShowErrorMessage("Unable to open workpackage editor.")
            End Try
        End If
    End Sub

    Private Sub ListViewWorkPackage_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles ListViewWorkPackages.MouseDown
        Try
            MouseLocationWorkPackage = e.Location
            Dim hasSelection As Boolean
            Dim listViewItem As ListViewItem = ListViewWorkPackages.GetItemAt(MouseLocationWorkPackage.X, MouseLocationWorkPackage.Y)
            If (listViewItem IsNot Nothing) Then
                hasSelection = True
                Presenter.SelectedWorkPackageName = listViewItem.Text
            Else
                Presenter.SelectedWorkPackageName = String.Empty
            End If
            Presenter.RequestMilestonesForWorkPackage(Presenter.SelectedWorkPackageName)
            If Not (IsReadOnlyMode) Then
                ToolStripButtonRemoveWorkPackageEnabled(hasSelection)
                ToolStripButtonAddMileStoneEnabled(hasSelection)
            End If
        Catch ex As Exception
            ShowErrorMessage("Unable to retrieve milstone(s) for the selected package.")
        End Try
    End Sub

    Private Sub ListViewMilestones_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewMilestones.DoubleClick
        If Not (IsReadOnlyMode) Then
            Try
                Dim listViewItem As ListViewItem = ListViewMilestones.GetItemAt(MouseLocationMilestone.X, MouseLocationMilestone.Y)
                If (listViewItem IsNot Nothing) Then
                    OpenMilestoneEditor(listViewItem.Text)
                End If
            Catch ex As Exception
                ShowErrorMessage("Unable to open milestone editor.")
            End Try
        End If
    End Sub

    Private Sub ListViewMilestones_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles ListViewMilestones.MouseDown
        If Not (IsReadOnlyMode) Then
            MouseLocationMilestone = e.Location
            Dim hasSelection As Boolean
            If (ListViewMilestones.GetItemAt(MouseLocationMilestone.X, MouseLocationMilestone.Y) IsNot Nothing) Then
                hasSelection = True
            End If
            ToolStripButtonRemoveMileStoneEnabled(hasSelection)
        End If
    End Sub

    Private Sub ListViewTeam_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewTeamMembers.DoubleClick
        If Not (IsReadOnlyMode) Then
            Try
                Dim listViewItem As ListViewItem = ListViewTeamMembers.GetItemAt(MouseLocationTeamMember.X, MouseLocationTeamMember.Y)
                If (listViewItem IsNot Nothing) Then
                    OpenTeamMemberEditor(listViewItem.Text)
                End If
            Catch ex As Exception
                ShowErrorMessage("Unable to open teammember editor.")
            End Try
        End If
    End Sub

    Private Sub ListViewTeam_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles ListViewTeamMembers.MouseDown
        If Not (IsReadOnlyMode) Then
            MouseLocationTeamMember = e.Location
            Dim hasSelection As Boolean
            If (ListViewTeamMembers.GetItemAt(MouseLocationTeamMember.X, MouseLocationTeamMember.Y) IsNot Nothing) Then
                hasSelection = True
            End If
            ToolStripButtonRemoveTeamMemberEnabled(hasSelection)
        End If
    End Sub

    Private Sub IsEditable(ByVal value As Boolean)
        IsReadOnlyMode = Not (value)
        ToolStripButtonAddWorkPackageEnabled(value)
        ToolStripButtonRemoveWorkPackageEnabled(value AndAlso ListViewWorkPackages.SelectedIndices.Count > 0)
        ToolStripButtonAddMileStoneEnabled(value AndAlso ListViewWorkPackages.SelectedIndices.Count > 0)
        ToolStripButtonRemoveMileStoneEnabled(value AndAlso ListViewMilestones.SelectedIndices.Count > 0)
        ToolStripButtonAddTeamMemberEnabled(value)
        ToolStripButtonRemoveTeamMemberEnabled(value AndAlso ListViewTeamMembers.SelectedIndices.Count > 0)
    End Sub

    Private Sub ShowErrorMessage(ByVal message As String)
        MessageBox.Show(message)
    End Sub

    Private Property Presenter() As PresenterFormTiming_Team
        Get
            Return m_Presenter
        End Get
        Set(ByVal value As PresenterFormTiming_Team)
            m_Presenter = value
        End Set
    End Property

    Private Property MouseLocationWorkPackage() As Point
        Get
            Return m_MouseLocationWorkPackage
        End Get
        Set(ByVal value As Point)
            m_MouseLocationWorkPackage = value
        End Set
    End Property

    Private Property MouseLocationMilestone() As Point
        Get
            Return m_MouseLocationMilestone
        End Get
        Set(ByVal value As Point)
            m_MouseLocationMilestone = value
        End Set
    End Property

    Private Property MouseLocationTeamMember() As Point
        Get
            Return m_MouseLocationTeamMember
        End Get
        Set(ByVal value As Point)
            m_MouseLocationTeamMember = value
        End Set
    End Property

    Private Property IsReadOnlyMode() As Boolean
        Get
            Return m_ReadOnlyMode
        End Get
        Set(ByVal value As Boolean)
            m_ReadOnlyMode = value
        End Set
    End Property

#Region "IViewFormTiming_Team"
    Public Function GetChildPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetChildPanel
        Return Nothing
    End Function

    Public Function GetText() As String Implements Presenters.IChildView.GetText
        Return SCREEN_NAME
    End Function

    Public Sub ShowChild() Implements Presenters.IChildView.ShowChild
        Try
            Presenter.RequestAllWorkPackages()
            Presenter.RequestMilestonesForWorkPackage(Presenter.SelectedWorkPackageName)
            Presenter.RequestAllTeamMembers()
            Presenter.RequestSelection()
            Show()
            Presenter.SendChangedViewEvent()
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Public Sub UpdateTopLevel(ByVal value As Boolean) Implements Presenters.IChildView.UpdateTopLevel
        SetTopLevel(value)
    End Sub

    Public Function GetMainPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetMainPanel
        Return PanelMain
    End Function

    Public Function GetToolstrip() As System.Windows.Forms.ToolStrip Implements Presenters.IChildView.GetToolstrip
        Return Nothing
    End Function

    Public Sub UpdateWorkPackages() Implements Presenters.IViewFormTiming_Team.UpdateWorkPackages
        ListViewWorkPackages.Invalidate()
    End Sub

    Public Sub ClearExtendedListViewWorkPackages() Implements Presenters.IViewFormTiming_Team.ClearExtendedListViewWorkPackages
        ListViewWorkPackages.VirtualModeCache.Clear()
    End Sub

    Public Sub AddToExtendedListViewWorkPackages(ByVal fourStateListViewItem As CustomControls.FourStateListViewItem) Implements Presenters.IViewFormTiming_Team.AddToExtendedListViewWorkPackages
        ListViewWorkPackages.VirtualModeCache.Add(fourStateListViewItem)
    End Sub

    Public Sub UpdateMileStones() Implements Presenters.IViewFormTiming_Team.UpdateMileStones
        ListViewMilestones.Invalidate()
        ToolStripButtonRemoveMileStoneEnabled(Not IsReadOnlyMode AndAlso ListViewMilestones.VirtualModeCache.Count > 0 AndAlso ListViewMilestones.SelectedIndices.Count > 0)
    End Sub

    Public Sub ClearExtendedListViewMilestones() Implements Presenters.IViewFormTiming_Team.ClearExtendedListViewMilestones
        ListViewMilestones.VirtualModeCache.Clear()
    End Sub

    Public Sub AddToExtendedListViewMilestones(ByVal fourStateListViewItem As CustomControls.FourStateListViewItem) Implements Presenters.IViewFormTiming_Team.AddToExtendedListViewMilestones
        ListViewMilestones.VirtualModeCache.Add(fourStateListViewItem)
    End Sub

    Public Sub UpdateTeamMembers() Implements Presenters.IViewFormTiming_Team.UpdateTeamMembers
        ListViewTeamMembers.Invalidate()
    End Sub

    Public Sub ClearExtendedListViewTeamMembers() Implements Presenters.IViewFormTiming_Team.ClearExtendedListViewTeamMembers
        ListViewTeamMembers.VirtualModeCache.Clear()
    End Sub

    Public Sub AddToExtendedListViewTeamMembers(ByVal fourStateListViewItem As CustomControls.FourStateListViewItem) Implements Presenters.IViewFormTiming_Team.AddToExtendedListViewTeamMembers
        ListViewTeamMembers.VirtualModeCache.Add(fourStateListViewItem)
    End Sub

    Public Sub UpdateView() Implements Presenters.IViewFormTiming_Team.UpdateView
        IsEditable(Presenter.IsEditable)
        If (Presenter.SelectedWorkPackageName IsNot Nothing) Then
            Dim listViewItem As ListViewItem = ListViewWorkPackages.FindItemWithText(Presenter.SelectedWorkPackageName)
            If (listViewItem IsNot Nothing) Then
                ListViewWorkPackages.FocusedItem = listViewItem
                ListViewWorkPackages.Invalidate()
            End If
        End If
    End Sub
#End Region

End Class