﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Windows.Forms

Public Class PresenterFormCategory
    Public Const CATEGORY_OVERVIEW As String = "CategoryOverview"
    Public Const CATEGORY_EDITOR As String = "CategoryEditor"
    Public Const SUB_CATEGORY_EDITOR As String = "SubCategoryEditor"

    Private m_View As IViewFormCategory
    Private m_CurrentPanel As Panel
    Private m_ChildViews As IDictionary(Of String, IChildView)

    Public Sub New(ByVal view As IViewFormCategory, ByVal childViews As IDictionary(Of String, IChildView))
        MyClass.View = view
        MyClass.ChildViews = childViews

        'add listener for closed view
        AddHandler DispatcherClosedViewEvent.ClosedView, AddressOf ClosedView

        'add listener for open view
        AddHandler DispatcherOpenCategoryEditorViewEvent.OpenCategoryEditorView, AddressOf OpenCategoryEditorView

        'add listener for open view
        AddHandler DispatcherOpenSubCategoryEditorViewEvent.OpenSubCategoryEditorView, AddressOf OpenSubCategoryEditorView

        'default veiw
        ShowChildPanelAndToolStrip(MyClass.ChildViews(CATEGORY_OVERVIEW))
    End Sub

    Private Sub ShowChildPanelAndToolStrip(ByVal child As IChildView)
        MergeAndShowChildForm(View, child)
    End Sub

    Private Sub MergeAndShowChildForm(ByVal parent As IChildView, ByVal child As IChildView)
        'get child toolstrip
        If (child.GetToolstrip() IsNot Nothing) Then
            PanelAndToollStripManager.MergeToolStrips(View, child)
        End If

        'panel
        'get child panel
        If (Not parent.GetMainPanel().Contains(child.GetMainPanel())) Then
            PanelAndToollStripManager.MergePanels(parent, child)
        End If
        child.ShowChild()
        'update parent panel
        If (CurrentPanel IsNot Nothing) Then
            CurrentPanel.Hide()
        End If
        CurrentPanel = child.GetMainPanel()
        CurrentPanel.Show()
    End Sub

    Private Sub ClosedView(ByVal sender As Object, ByVal e As ClosedViewEvent)
        If (sender.GetType() Is GetType(PresenterFormCategoryEditor) OrElse _
            sender.GetType() Is GetType(PresenterFormSubCategoryEditor)) Then
            ShowChildPanelAndToolStrip(ChildViews(CATEGORY_OVERVIEW))
        End If
    End Sub

    Private Sub OpenCategoryEditorView(ByVal sender As Object, ByVal e As OpenEditorViewEvent(Of Category))
        Select Case e.FormName
            Case CATEGORY_EDITOR
                View.CreateCategoryEditor(e.SelectedData)
                ChildViews(CATEGORY_EDITOR) = View.GetCategoryEditor()
                ShowChildPanelAndToolStrip(ChildViews(CATEGORY_EDITOR))
        End Select
    End Sub

    Private Sub OpenSubCategoryEditorView(ByVal sender As Object, ByVal e As OpenEditorViewEvent(Of SubCategory))
        Select Case e.FormName
            Case SUB_CATEGORY_EDITOR
                View.CreateSubCategoryEditor(e.SelectedData)
                ChildViews(SUB_CATEGORY_EDITOR) = View.GetSubCategoryEditor()
                ShowChildPanelAndToolStrip(ChildViews(SUB_CATEGORY_EDITOR))
        End Select
    End Sub

    Private Property View() As IViewFormCategory
        Get
            Return m_View
        End Get
        Set(ByVal value As IViewFormCategory)
            m_View = value
        End Set
    End Property

    Private Property CurrentPanel() As Panel
        Get
            Return m_CurrentPanel
        End Get
        Set(ByVal value As Panel)
            m_CurrentPanel = value
        End Set
    End Property

    Private Property ChildViews() As IDictionary(Of String, IChildView)
        Get
            Return m_ChildViews
        End Get
        Set(ByVal value As IDictionary(Of String, IChildView))
            m_ChildViews = value
        End Set
    End Property
End Class
