﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Windows.Forms
Imports CustomControls

Public Class PresenterFormCategoryOverview
    Private m_View As IViewFormCategoryOverview
    Private m_SelectedCatagoryId As Integer

    Public Sub New(ByVal view As IViewFormCategoryOverview)
        MyClass.View = view
        SelectedCatagoryId = -1

        'add listener for closed view
        AddHandler DispatcherClosedViewEvent.ClosedView, AddressOf ClosedView
    End Sub

    Public Sub RequestAllCategories()
        Try
            View.ClearExtendedListViewCategory()
            Dim categories As IList(Of Category) = TaskCategory.Instance.GetAllCategories()
            If (categories IsNot Nothing) Then
                Dim enumerator As IEnumerator(Of Category) = categories.GetEnumerator()
                While (enumerator.MoveNext())
                    Dim listViewItem As FourStateListViewItem = New FourStateListViewItem()
                    listViewItem.Text = enumerator.Current.Description
                    listViewItem.Tag = enumerator.Current.Id
                    View.AddToExtendedListViewCategory(listViewItem)
                End While
            End If
            View.UpdateCategory()
        Catch ex As Exception
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Sub

    Public Sub RequestSubCategoriesForCategory(ByVal categoryId As Integer)
        Try
            View.ClearExtendedListViewSubCategory()
            Dim subCategories As IList(Of SubCategory) = TaskSubCategory.Instance.GetSubCategoriesByCategoryId(categoryId)
            If (subCategories IsNot Nothing) Then
                Dim enumerator As IEnumerator(Of SubCategory) = subCategories.GetEnumerator()
                While (enumerator.MoveNext())
                    Dim listViewItem As FourStateListViewItem = New FourStateListViewItem()
                    listViewItem.Text = enumerator.Current.Description
                    listViewItem.Tag = enumerator.Current.Id
                    View.AddToExtendedListViewSubCategory(listViewItem)
                End While
            End If
            View.UpdateSubCategory()
        Catch ex As Exception
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Sub

    Public Sub RemoveCategory(ByVal id As Integer)
        Try
            TaskCategory.Instance.Remove(id)
        Catch ex As Exception
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Sub

    Public Sub RemoveSubCategory(ByVal id As Integer)
        Try
            TaskSubCategory.Instance.Remove(id)
        Catch ex As Exception
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Sub

    Public Sub Close()
        DispatcherClosedViewEvent.SendClosedViewEvent(Me, New ClosedViewEvent())
    End Sub

    Public Sub ShowCategoryEditor(ByVal description As String)
        Dim category As Category = TaskCategory.Instance.GetCategoryByDescription(description)
        DispatcherOpenCategoryEditorViewEvent.SendOpenCategoryEditorViewEvent(Me, New OpenEditorViewEvent(Of Category)(PresenterFormCategory.CATEGORY_EDITOR, category))
    End Sub

    Public Sub ShowSubCategoryEditor(ByVal description As String)
        Dim subCategory As SubCategory = TaskSubCategory.Instance.GetSubCategoryByDescription(description)
        subCategory.Category = TaskCategory.Instance.GetCategoryById(SelectedCatagoryId)
        DispatcherOpenSubCategoryEditorViewEvent.SendOpenSubCategoryEditorViewEvent(Me, New OpenEditorViewEvent(Of SubCategory)(PresenterFormCategory.SUB_CATEGORY_EDITOR, subCategory))
    End Sub

    Public Sub SendChangedViewEvent()
        DispatcherChangedViewEvent.SendChangedViewEvent(Me, New ChangedViewEvent(View.GetText(), Nothing))
    End Sub

    Private Sub ClosedView(ByVal sender As Object, ByVal e As ClosedViewEvent)
        If (sender.GetType() Is GetType(PresenterFormCategoryEditor)) Then
            View.UpdateCategory()
        ElseIf (sender.GetType() Is GetType(PresenterFormSubCategoryEditor)) Then
            View.UpdateSubCategory()
        End If
    End Sub

    Public Property SelectedCatagoryId() As Integer
        Get
            Return m_SelectedCatagoryId
        End Get
        Set(ByVal value As Integer)
            m_SelectedCatagoryId = value
        End Set
    End Property

    Private Property View() As IViewFormCategoryOverview
        Get
            Return m_View
        End Get
        Set(ByVal value As IViewFormCategoryOverview)
            m_View = value
        End Set
    End Property

End Class
