﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Windows.Forms

Public Class PresenterFormPartDescription
    Public Const PART_DESCRIPTION_OVERVIEW As String = "PartDescriptionOverview"
    Public Const PART_DESCRIPTION_EDITOR As String = "PartDescriptionEditor"

    Private m_View As IViewFormPartDescription
    Private m_CurrentPanel As Panel
    Private m_ChildViews As IDictionary(Of String, IChildView)

    Public Sub New(ByVal view As IViewFormPartDescription, ByVal childViews As IDictionary(Of String, IChildView))
        MyClass.View = view
        MyClass.ChildViews = childViews

        'add listener for closed view
        AddHandler DispatcherClosedViewEvent.ClosedView, AddressOf ClosedView

        'add listener for open view
        AddHandler DispatcherOpenPartDescriptionEditorViewEvent.OpenPartDescriptionEditorView, AddressOf OpenPartDescriptionEditorView

        'default veiw
        ShowChildPanelAndToolStrip(MyClass.ChildViews(PART_DESCRIPTION_OVERVIEW))
    End Sub

    Private Sub ShowChildPanelAndToolStrip(ByVal child As IChildView)
        MergeAndShowChildForm(View, child)
    End Sub

    Private Sub MergeAndShowChildForm(ByVal parent As IChildView, ByVal child As IChildView)
        'get child toolstrip
        If (child.GetToolstrip() IsNot Nothing) Then
            PanelAndToollStripManager.MergeToolStrips(View, child)
        End If

        'panel
        'get child panel
        If (Not parent.GetMainPanel().Contains(child.GetMainPanel())) Then
            PanelAndToollStripManager.MergePanels(parent, child)
        End If
        child.ShowChild()

        'update parent panel
        If (CurrentPanel IsNot Nothing) Then
            CurrentPanel.Hide()
        End If
        CurrentPanel = child.GetMainPanel()
        CurrentPanel.Show()
    End Sub

    Private Sub ClosedView(ByVal sender As Object, ByVal e As ClosedViewEvent)
        If (sender.GetType() Is GetType(PresenterFormPartDescriptionEditor)) Then
            View.ClosePartDescriptionEditor()
            ShowChildPanelAndToolStrip(ChildViews(PART_DESCRIPTION_OVERVIEW))
        End If
    End Sub

    Private Sub OpenPartDescriptionEditorView(ByVal sender As Object, ByVal e As OpenEditorViewEvent(Of FormDescription))
        Select Case e.FormName
            Case PART_DESCRIPTION_EDITOR
                View.CreatePartDescriptionEditor(e.SelectedData)
                ChildViews(PART_DESCRIPTION_EDITOR) = View.GetPartDescriptionEditor()
                ShowChildPanelAndToolStrip(ChildViews(PART_DESCRIPTION_EDITOR))
        End Select
    End Sub

    Private Property View() As IViewFormPartDescription
        Get
            Return m_View
        End Get
        Set(ByVal value As IViewFormPartDescription)
            m_View = value
        End Set
    End Property

    Private Property CurrentPanel() As Panel
        Get
            Return m_CurrentPanel
        End Get
        Set(ByVal value As Panel)
            m_CurrentPanel = value
        End Set
    End Property

    Private Property ChildViews() As IDictionary(Of String, IChildView)
        Get
            Return m_ChildViews
        End Get
        Set(ByVal value As IDictionary(Of String, IChildView))
            m_ChildViews = value
        End Set
    End Property
End Class
