﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports Securex.FIT.Fitclient.Business
Imports System.Windows.Forms

Public Class PresenterFormProjectEditor
    Private m_View As IViewFormProjectEditor
    Private m_CurrentPanel As Panel
    Private m_ChildViews As IList(Of IChildView)
    Private m_SelectedScreenIndex As Integer

    Public Sub New(ByVal view As IViewFormProjectEditor)
        MyClass.View = view

        'add listener for changed view
        AddHandler DispatcherChangedViewEvent.ChangedView, AddressOf ChangedView

        'add listener for RefreshViewOnSave event
        AddHandler DispatcherSubEditorOpenedEvent.SubEditorOpened, AddressOf SubEditorOpened

        'add listener for AuditorCanSave event
        AddHandler DispatcherAuditorCanSaveEvent.AuditorCanSave, AddressOf AuditorCanSave

        SelectedScreenIndex = -1
    End Sub

    Public Sub AddChilViews(ByVal childViews As IList(Of IChildView))
        MyClass.ChildViews = childViews
    End Sub

    Public Sub CreateNewProject(ByVal creatorId As String)
        Try
            TaskProjectProposalRevision.Instance.UpdateCreator(creatorId)
            Dim currentRevisionStatus As RevisionStatus = TaskRevisionStatus.Instance.GetRevisionStatusByName(TaskRevisionStatus.DRAFT)
            TaskProjectProposalRevision.Instance.UpdateRevisionStatus(currentRevisionStatus)
            TaskProjectProposalRevision.Instance.UpdateRevisionLastModified(Now)
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public Sub [Next]()
        If (SelectedScreenIndex < ChildViews.Count - 1) Then
            SelectedScreenIndex += 1
            ShowChildPanel(View, ChildViews(SelectedScreenIndex))
            UpdateLabelCurrentPanel()
        End If
    End Sub

    Public Sub Previous()
        If (SelectedScreenIndex > 0) Then
            SelectedScreenIndex -= 1
            ShowChildPanel(View, ChildViews(SelectedScreenIndex))
            UpdateLabelCurrentPanel()
        End If
    End Sub

    Public Sub Save()
        Try
            TaskRevision.Instance.CreateAndSaveNewRevision(TaskProjectProposalRevision.Instance.GetSelection())
            Dim newProjectRevision As ProjectProposalRevision = TaskProjectProposalRevision.Instance.GetSelection()
            DispatcherRefreshViewOnSaveEvent.SendRefreshViewOnSaveEvent(Me, New RefreshViewOnSaveEvent())
            EnableToolStripButtonsSaveAndCancel(((TaskInformation.Instance.IsCreator(newProjectRevision.Creator) AndAlso Not TaskRevisionStatus.Instance.IsReviewing(newProjectRevision)) OrElse TaskInformation.Instance.IsAuditor(newProjectRevision.Auditor)) AndAlso Not TaskRevisionStatus.Instance.IsReadOnly(newProjectRevision))
            'send mail
            TaskMail.Instance.SendMail(newProjectRevision)
        Catch ex As Exception
            Throw New Exception(ex.Message)
        End Try
    End Sub

    Public Sub Cancel()
        TaskProjectProposalRevision.Instance.Cancel()
        DispatcherCancelEditorEvent.SendCancelEditorEvent(Me, New CancelEditorEvent())
    End Sub

    Public Sub SetSelection(ByVal project As ProjectProposalRevision)
        TaskProjectProposalRevision.Instance.Add(project)
    End Sub

    Public Sub Close()
        TaskProjectProposalRevision.Instance.ClearSelection()
        DispatcherClosedViewEvent.SendClosedViewEvent(Me, New ClosedViewEvent())
    End Sub

    Public Sub SendChangedViewEvent()
        If (CurrentPanel IsNot Nothing) Then
            DispatcherChangedViewEvent.SendChangedViewEvent(Me, New ChangedViewEvent(ChildViews(SelectedScreenIndex).GetText(), Nothing))
        End If
    End Sub

    Public Function AreRequiredFieldsFilledIn() As Boolean
        Return TaskProjectProposalRevision.Instance.AreRequiredFieldsFilledIn()
    End Function

    Public Function GetRequiredFieldNames() As IList(Of String)
        Return TaskProjectProposalRevision.Instance.GetRequiredFieldNames()
    End Function

    Public ReadOnly Property IsReadOnly() As Boolean
        Get
            Return TaskRevisionStatus.Instance.IsReadOnly(TaskProjectProposalRevision.Instance.GetSelection())
        End Get
    End Property

    Public ReadOnly Property IsReviewing() As Boolean
        Get
            Return TaskRevisionStatus.Instance.IsReviewing(TaskProjectProposalRevision.Instance.GetSelection())
        End Get
    End Property

    Public ReadOnly Property IsCreator() As Boolean
        Get
            Return TaskInformation.Instance.IsCreator(TaskProjectProposalRevision.Instance.GetSelection().Creator)
        End Get
    End Property

    Public ReadOnly Property IsAuditor() As Boolean
        Get
            Return TaskInformation.Instance.IsAuditor(TaskProjectProposalRevision.Instance.GetSelection().Auditor)
        End Get
    End Property

    Private Sub ThrowException(ByVal exception As Exception)
        Throw New Exception(exception.Message)
    End Sub

    Private Sub ShowChildPanel(ByVal parent As IChildView, ByVal child As IChildView)
        MergeAndShowChildForm(parent, child)
    End Sub

    Private Sub MergeAndShowChildForm(ByVal parent As IChildView, ByVal child As IChildView)
        'get child panel
        If (Not parent.GetChildPanel().Contains(child.GetMainPanel())) Then
            PanelAndToollStripManager.MergePanels(parent, child)
        End If
        child.ShowChild()
        'update parent panel
        If (CurrentPanel IsNot Nothing) Then
            CurrentPanel.Hide()
        End If
        CurrentPanel = child.GetMainPanel()
        CurrentPanel.Show()
    End Sub

    Private Sub UpdateLabelCurrentPanel()
        Dim index As Integer
        If (ChildViews.Count > 0) Then
            index = SelectedScreenIndex + 1
        End If
        View.UpdateLabelCurrentPage(String.Format("{0}\{1}", index.ToString(), (ChildViews.Count).ToString()))
    End Sub

    Private Sub EnableToolStripButtonsSaveAndCancel(ByVal value As Boolean)
        View.EnableToolStripButtonsSaveAndCancel(value)
    End Sub

    Private Sub ChangedView(ByVal sender As Object, ByVal e As ChangedViewEvent)
        If (sender.GetType() Is GetType(PresenterFormProjectScreen1) OrElse _
            sender.GetType() Is GetType(PresenterFormProjectScreen2) OrElse _
            sender.GetType() Is GetType(PresenterFormProjectScreen3) OrElse _
            sender.GetType() Is GetType(PresenterFormProjectScreen4) OrElse _
            sender.GetType() Is GetType(PresenterFormProjectScreen5) OrElse _
            sender.GetType() Is GetType(PresenterFormProjectScreen6) OrElse _
            sender.GetType() Is GetType(PresenterFormProjectScreenAuditor)) Then
            View.UpdateTextboxDescription(e.Description)
        End If
    End Sub

    Private Sub SubEditorOpened(ByVal sender As Object, ByVal e As SubEditorOpenedEvent)
        EnableToolStripButtonsSaveAndCancel(e.Opened)
    End Sub

    Private Sub AuditorCanSave(ByVal sender As Object, ByVal e As AuditorCanSaveEvent)
        EnableToolStripButtonsSaveAndCancel(e.CanSave)
    End Sub

    Private Property View() As IViewFormProjectEditor
        Get
            Return m_View
        End Get
        Set(ByVal value As IViewFormProjectEditor)
            m_View = value
        End Set
    End Property

    Private Property CurrentPanel() As Panel
        Get
            Return m_CurrentPanel
        End Get
        Set(ByVal value As Panel)
            m_CurrentPanel = value
        End Set
    End Property

    Private Property ChildViews() As IList(Of IChildView)
        Get
            Return m_ChildViews
        End Get
        Set(ByVal value As IList(Of IChildView))
            m_ChildViews = value
        End Set
    End Property

    Private Property SelectedScreenIndex() As Integer
        Get
            Return m_SelectedScreenIndex
        End Get
        Set(ByVal value As Integer)
            m_SelectedScreenIndex = value
        End Set
    End Property
End Class
