﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Windows.Forms
Imports CustomControls

Public Class PresenterFormProjectScreenAuditor
    Private m_View As IViewProjectScreenAuditor

    Public Sub New(ByVal view As IViewProjectScreenAuditor)
        MyClass.View = view

        'add listener for RefreshViewOnSave event
        AddHandler DispatcherRefreshViewOnSaveEvent.RefreshViewOnSave, AddressOf RefreshViewOnSave

        SendAuditorCanSaveEvent()
    End Sub

    Public Sub SendChangedViewEvent()
        DispatcherChangedViewEvent.SendChangedViewEvent(Me, New ChangedViewEvent(View.GetText(), TaskFormDescription.Instance.GetDescriptionByFormName(View.GetName()).Description))
    End Sub

    Public Sub RequestSelection()
        View.UpdateView(TaskProjectProposalRevision.Instance.GetSelection())
    End Sub

    Public Sub RequestAllRevisionStatuses()
        Try
            View.ClearListViewComboBoxAuditorRevisionStatus()
            Dim revisionStatuses As IList(Of RevisionStatus) = TaskRevisionStatus.Instance.GetAllRevisionStatuses()
            Dim auditorRevisionStatuses As IList(Of FourStateListViewItem) = New List(Of FourStateListViewItem)()
            If (revisionStatuses IsNot Nothing) Then
                Dim enumerator As IEnumerator(Of RevisionStatus) = revisionStatuses.GetEnumerator()
                While (enumerator.MoveNext())
                    Dim addToCache As Boolean = True
                    If (enumerator.Current.Name.Equals(TaskRevisionStatus.DRAFT)) Then
                        addToCache = False
                    End If
                    If (addToCache) Then
                        Dim listViewItem As FourStateListViewItem = New FourStateListViewItem()
                        listViewItem.Text = enumerator.Current.Name
                        listViewItem.Tag = enumerator.Current.Id
                        auditorRevisionStatuses.Add(listViewItem)
                    End If
                End While
            End If
            View.AddToListViewComboBoxAuditorRevisionStatus(auditorRevisionStatuses)
            View.UpdateRevisionStatus()
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public Sub UpdateRevisionStatus(ByVal revisionName As String, ByVal revisionId As Integer)
        Try
            Dim currentRevision As RevisionStatus = New RevisionStatus() With {.Id = revisionId, .Name = revisionName}
            TaskProjectProposalRevision.Instance.UpdateRevisionStatus(currentRevision)
            TaskProjectProposalRevision.Instance.UpdateRevisedBy(TaskProjectProposalRevision.Instance.GetSelection().Auditor.ToString())
            SendAuditorCanSaveEvent()
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public Sub UpdateAuditorDescription(ByVal auditorDescription As String)
        Try
            TaskProjectProposalRevision.Instance.UpdateAuditorDescription(auditorDescription)
            SendAuditorCanSaveEvent()
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public ReadOnly Property IsEditable() As Boolean
        Get
            Return TaskProjectProposalRevision.Instance.IsReviewed() AndAlso Not TaskRevisionStatus.Instance.IsReadOnly(TaskProjectProposalRevision.Instance.GetSelection())
        End Get
    End Property

    Private Sub SendAuditorCanSaveEvent()
        Dim canSave As Boolean
        If (TaskProjectProposalRevision.Instance.GetSelection().RevisionInformation.Description IsNot Nothing AndAlso TaskProjectProposalRevision.Instance.GetSelection().RevisionInformation.Description.Trim().Length > 0 AndAlso (TaskProjectProposalRevision.Instance.GetSelection().RevisionInformation.RevisionStatus.Name.Equals(TaskRevisionStatus.REJECTED) OrElse TaskProjectProposalRevision.Instance.GetSelection().RevisionInformation.RevisionStatus.Name.Equals(TaskRevisionStatus.RELEASED))) Then
            canSave = True
        End If
        DispatcherAuditorCanSaveEvent.SendAuditorCanSaveEvent(Me, New AuditorCanSaveEvent(canSave))
    End Sub

    Private Sub RefreshViewOnSave(ByVal sender As Object, ByVal e As RefreshViewOnSaveEvent)
        View.UpdateView(TaskProjectProposalRevision.Instance.GetSelection())
    End Sub

    Private Sub ThrowException(ByVal exception As Exception)
        Throw New Exception(exception.Message)
    End Sub

    Private Property View() As IViewProjectScreenAuditor
        Get
            Return m_View
        End Get
        Set(ByVal value As IViewProjectScreenAuditor)
            m_View = value
        End Set
    End Property

End Class
