﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports NHibernate
Imports NHibernate.Criterion

Public Class SqlCategory
    Implements ICategoryDao
    Private Const ID As String = "Id"
    Private Const DESCRIPTION As String = "Description"
    Private m_GenericNhbDao As GenericNhbDao(Of Category)

    Public Sub New()
        m_GenericNhbDao = New GenericNhbDao(Of Category)()
    End Sub

#Region "ICategoryDao"
    ''' <summary>
    ''' Get all categories
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <returns>Returns an IList of categories</returns>
    ''' <remarks></remarks>
    Public Function GetAllCategories(ByVal session As Business.ISessionDao) As System.Collections.Generic.IList(Of Business.DataHolders.Category) Implements Business.ICategoryDao.GetAllCategories
        Try
            GetAllCategories = m_GenericNhbDao.GetAll(DirectCast(session, SqlSession).Session)
        Catch ex As NHibernateException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Function

    ''' <summary>
    ''' Get all categories for description
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <param name="description">Returns an IList of categories</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetCategoryByDescription(ByVal session As Business.ISessionDao, ByVal description As String) As System.Collections.Generic.IList(Of Business.DataHolders.Category) Implements Business.ICategoryDao.GetCategoryByDescription
        Try
            Dim orders As Order() = {New Order(SqlCategory.DESCRIPTION, True)}
            Dim criterions As ICriterion() = {New LikeExpression(SqlCategory.DESCRIPTION, description, MatchMode.Exact)}
            Return m_GenericNhbDao.GetForCriteria(DirectCast(session, SqlSession).Session, criterions, orders)
        Catch ex As NHibernateException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Function

    ''' <summary>
    ''' Get category by row identifier
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <param name="id">Row identifier</param>
    ''' <returns>Returns an IList of categories</returns>
    ''' <remarks></remarks>
    Public Function GetCategoryById(ByVal session As Business.ISessionDao, ByVal id As Integer) As System.Collections.Generic.IList(Of Business.DataHolders.Category) Implements Business.ICategoryDao.GetCategoryById
        Try
            Dim orders As Order() = {New Order(SqlCategory.ID, True)}
            Dim criterions As ICriterion() = {New LikeExpression(SqlCategory.ID, id.ToString(), MatchMode.Exact)}
            Return m_GenericNhbDao.GetForCriteria(DirectCast(session, SqlSession).Session, criterions, orders)
        Catch ex As NHibernateException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Function

    ''' <summary>
    ''' Remove a categorie
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <param name="category">Category object to be removed</param>
    ''' <remarks></remarks>
    Public Sub Remove(ByVal session As Business.ISessionDao, ByVal category As Business.DataHolders.Category) Implements Business.ICategoryDao.Remove
        Try
            m_GenericNhbDao.Delete(DirectCast(session, SqlSession).Session, category)
        Catch ex As NHibernateException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Sub

    ''' <summary>
    ''' Save changes
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <param name="category">Category object to be saved or updated</param>
    ''' <remarks></remarks>
    Public Sub Save(ByVal session As Business.ISessionDao, ByVal category As Business.DataHolders.Category) Implements Business.ICategoryDao.Save
        Try
            m_GenericNhbDao.SaveOrUpdate(DirectCast(session, SqlSession).Session, category)
        Catch ex As NHibernateException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Sub
#End Region
End Class
