﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports NHibernate
Imports NHibernate.Criterion

Public Class SqlFormDescription
    Implements IFormDescriptionDao
    Private Const ID As String = "Id"
    Private Const FORMNAME As String = "FormName"
    Private m_GenericNhbDao As GenericNhbDao(Of FormDescription)

    Public Sub New()
        m_GenericNhbDao = New GenericNhbDao(Of FormDescription)()
    End Sub

#Region "IFormDescriptionDao"
    ''' <summary>
    ''' Get all form descriptions
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <returns>Returns an IList of formdescriptions</returns>
    ''' <remarks></remarks>
    Public Function GetAllFormDescriptions(ByVal session As Business.ISessionDao) As System.Collections.Generic.IList(Of Business.DataHolders.FormDescription) Implements Business.IFormDescriptionDao.GetAllFormDescriptions
        Try
            GetAllFormDescriptions = m_GenericNhbDao.GetAll(DirectCast(session, SqlSession).Session)
        Catch ex As NHibernateException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Function

    ''' <summary>
    ''' Get form description by form name
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <param name="formName"></param>
    ''' <returns>Returns an IList of formdescriptions</returns>
    ''' <remarks></remarks>
    Public Function GetFormDescriptionByFormName(ByVal session As Business.ISessionDao, ByVal formName As String) As System.Collections.Generic.IList(Of Business.DataHolders.FormDescription) Implements Business.IFormDescriptionDao.GetFormDescriptionByFormName
        Try
            Dim orders As Order() = {New Order(SqlFormDescription.FORMNAME, True)}
            Dim criterions As ICriterion() = {New LikeExpression(SqlFormDescription.FORMNAME, formName, MatchMode.Exact)}
            Return m_GenericNhbDao.GetForCriteria(DirectCast(session, SqlSession).Session, criterions, orders)
        Catch ex As RepositoryException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Function

    ''' <summary>
    ''' Get form description by row identifier
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <param name="id">Row identifier</param>
    ''' <returns>Returns an IList of formdescriptions</returns>
    ''' <remarks></remarks>
    Public Function GetFormDescriptionById(ByVal session As Business.ISessionDao, ByVal id As Integer) As System.Collections.Generic.IList(Of Business.DataHolders.FormDescription) Implements Business.IFormDescriptionDao.GetFormDescriptionById
        Try
            Dim orders As Order() = {New Order(SqlFormDescription.ID, True)}
            Dim criterions As ICriterion() = {New LikeExpression(SqlFormDescription.ID, id.ToString(), MatchMode.Exact)}
            Return m_GenericNhbDao.GetForCriteria(DirectCast(session, SqlSession).Session, criterions, orders)
        Catch ex As RepositoryException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Function

    ''' <summary>
    ''' Remove a form description
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <param name="formDescription">Formdescription object to be removed</param>
    ''' <remarks></remarks>
    Public Sub Remove(ByVal session As Business.ISessionDao, ByVal formDescription As Business.DataHolders.FormDescription) Implements Business.IFormDescriptionDao.Remove
        Try
            m_GenericNhbDao.Delete(DirectCast(session, SqlSession).Session, formDescription)
        Catch ex As RepositoryException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Sub

    ''' <summary>
    ''' Save changes
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <param name="formDescription">Formdescription object to be saved or updated</param>
    ''' <remarks></remarks>
    Public Sub Save(ByVal session As ISessionDao, ByVal formDescription As Business.DataHolders.FormDescription) Implements Business.IFormDescriptionDao.Save
        Try
            m_GenericNhbDao.SaveOrUpdate(DirectCast(session, SqlSession).Session, formDescription)
        Catch ex As RepositoryException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Sub
#End Region
End Class
