﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports NHibernate
Imports NHibernate.Criterion

Public Class SqlProjectProposalRevision
    Implements IProjectProposalRevisionDao
    Private Const ID As String = "Id"
    Private Const PROJECT_ID As String = "Project.Id"
    Private m_GenericNhbDao As GenericNhbDao(Of ProjectProposalRevision)

    Public Sub New()
        m_GenericNhbDao = New GenericNhbDao(Of ProjectProposalRevision)()
    End Sub

#Region "IProjectProposalRevision"
    ''' <summary>
    ''' Get all revisions for the specified project
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <param name="projectProposalNameId">Project proposal name row identifier</param>
    ''' <param name="properties">Needed properties</param>
    ''' <returns>Returns an IList of projectProposalRevisions</returns>
    ''' <remarks></remarks>
    Public Function GetAllRevisionsByProjectPrososalNameId(ByVal session As Business.ISessionDao, ByVal projectProposalNameId As Integer, ByVal properties As System.Collections.Generic.IList(Of String)) As System.Collections.Generic.IList(Of Business.DataHolders.ProjectProposalRevision) Implements Business.IProjectProposalRevisionDao.GetAllRevisionsByProjectPrososalNameId
        Try
            Dim orders As Order() = {New Order(SqlProjectProposalRevision.PROJECT_ID, True)}
            Dim criterions As ICriterion() = {New LikeExpression(SqlProjectProposalRevision.PROJECT_ID, projectProposalNameId.ToString(), MatchMode.Exact)}

            If (properties IsNot Nothing) Then
                Dim count As Integer = properties.Count - 1
                Dim projectionsArray(count) As IProjection
                Dim propertieEnumerator As IEnumerator(Of String) = properties.GetEnumerator()
                Dim index As Integer = -1
                While (propertieEnumerator.MoveNext())
                    index += 1
                    projectionsArray(index) = Projections.Alias(Projections.Property(propertieEnumerator.Current), propertieEnumerator.Current)
                End While
                GetAllRevisionsByProjectPrososalNameId = m_GenericNhbDao.GetForCriteriaAndProjection(DirectCast(session, SqlSession).Session, criterions, orders, projectionsArray)
            Else
                GetAllRevisionsByProjectPrososalNameId = m_GenericNhbDao.GetForCriteria(DirectCast(session, SqlSession).Session, criterions, orders)
            End If
        Catch ex As NHibernateException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Function

    ''' <summary>
    ''' Get the revision by row identifier
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <param name="id">Row identifier</param>
    ''' <returns>Returns an IList of projectProposalRevisions</returns>
    ''' <remarks></remarks>
    Public Function GetProjectProposalRevisionById(ByVal session As ISessionDao, ByVal id As Integer) As System.Collections.Generic.IList(Of Business.DataHolders.ProjectProposalRevision) Implements Business.IProjectProposalRevisionDao.GetProjectProposalRevisionById
        Try
            Dim orders As Order() = {New Order(SqlProjectProposalRevision.ID, True)}
            Dim criterions As ICriterion() = {New LikeExpression(SqlProjectProposalRevision.ID, id.ToString(), MatchMode.Exact)}
            Return m_GenericNhbDao.GetForCriteria(DirectCast(session, SqlSession).Session, criterions, orders)
        Catch ex As NHibernateException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Function

    ''' <summary>
    ''' Save changes
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <param name="projectProposalRevision">ProjectProposalRevision object to be saved or updated</param>
    ''' <returns>Returns an IList of projectProposalRevisions</returns>
    ''' <remarks></remarks>
    Public Function Save(ByVal session As Business.ISessionDao, ByVal projectProposalRevision As Business.DataHolders.ProjectProposalRevision) As Business.DataHolders.ProjectProposalRevision Implements Business.IProjectProposalRevisionDao.Save
        Try
            'if the cache contains another copy with the same reference use merge
            projectProposalRevision = m_GenericNhbDao.Merge(DirectCast(session, SqlSession).Session, projectProposalRevision)
            Return projectProposalRevision
        Catch ex As NHibernateException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Function
#End Region
End Class
