﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports NHibernate
Imports NHibernate.Criterion

Public Class SqlRole
    Implements IRoleDao
    Private Const ID As String = "Id"
    Private Const NAME As String = "Name"
    Private m_GenericNhbDao As GenericNhbDao(Of Role)

    Public Sub New()
        m_GenericNhbDao = New GenericNhbDao(Of Role)()
    End Sub

#Region "IRole"
    ''' <summary>
    ''' Get all roles
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <returns>Returns an IList of roles</returns>
    ''' <remarks></remarks>
    Public Function GetAllRoles(ByVal session As Business.ISessionDao) As System.Collections.Generic.IList(Of Business.DataHolders.Role) Implements Business.IRoleDao.GetAllRoles
        Try
            GetAllRoles = m_GenericNhbDao.GetAll(DirectCast(session, SqlSession).Session)
        Catch ex As NHibernateException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Function

    ''' <summary>
    ''' Get all roles for name
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <param name="name">Returns an IList of roles</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetRoleByName(ByVal session As Business.ISessionDao, ByVal name As String) As System.Collections.Generic.IList(Of Business.DataHolders.Role) Implements Business.IRoleDao.GetRoleByName
        Try
            Dim orders As Order() = {New Order(SqlRole.NAME, True)}
            Dim criterions As ICriterion() = {New LikeExpression(SqlRole.NAME, name, MatchMode.Exact)}
            Return m_GenericNhbDao.GetForCriteria(DirectCast(session, SqlSession).Session, criterions, orders)
        Catch ex As NHibernateException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Function

    ''' <summary>
    ''' Get role by row identifier
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <param name="id">Row identifier</param>
    ''' <returns>Returns an IList of roles</returns>
    ''' <remarks></remarks>
    Public Function GetRoleById(ByVal session As Business.ISessionDao, ByVal id As Integer) As System.Collections.Generic.IList(Of Business.DataHolders.Role) Implements Business.IRoleDao.GetRoleById
        Try
            Dim orders As Order() = {New Order(SqlRole.ID, True)}
            Dim criterions As ICriterion() = {New LikeExpression(SqlRole.ID, id.ToString(), MatchMode.Exact)}
            Return m_GenericNhbDao.GetForCriteria(DirectCast(session, SqlSession).Session, criterions, orders)
        Catch ex As NHibernateException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Function

    ''' <summary>
    ''' Remove a role
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <param name="role">Role object to be removed</param>
    ''' <remarks></remarks>
    Public Sub Remove(ByVal session As Business.ISessionDao, ByVal role As Business.DataHolders.Role) Implements Business.IRoleDao.Remove
        Try
            m_GenericNhbDao.Delete(DirectCast(session, SqlSession).Session, role)
        Catch ex As NHibernateException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Sub

    ''' <summary>
    ''' Save changes
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <param name="role">Role object to be saved or updated</param>
    ''' <remarks></remarks>
    Public Sub Save(ByVal session As Business.ISessionDao, ByVal role As Business.DataHolders.Role) Implements Business.IRoleDao.Save
        Try
            m_GenericNhbDao.SaveOrUpdate(DirectCast(session, SqlSession).Session, role)
        Catch ex As NHibernateException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Sub

#End Region

End Class
