﻿Option Strict On
Option Explicit On
Option Infer Off
Imports NHibernate
Imports NHibernate.Criterion

Public Class GenericNhbDao(Of T As Class)
    Public Overridable Function SaveOrUpdate(ByVal session As ISession, ByVal obj As T) As T
        Try
            session.SaveOrUpdate(obj)
            Return obj
        Catch ex As Exception
            ThrowNHibernateException("Unable to save or update", ex)
        End Try
    End Function

    Public Overridable Function Merge(ByVal session As ISession, ByVal obj As T) As T
        Try
            Return session.Merge(obj)
        Catch ex As Exception
            ThrowNHibernateException("Unable to merge", ex)
        End Try
    End Function

    Public Overridable Sub Delete(ByVal session As ISession, ByVal obj As T)
        Try
            session.Delete(obj)
        Catch ex As Exception
            ThrowNHibernateException("Unable to delete", ex)
        End Try
    End Sub

    Public Overridable Function GetAll(ByVal session As ISession) As IList(Of T)
        Try
            Return GetForCriteria(session, Nothing, Nothing)
        Catch ex As Exception
            ThrowNHibernateException("Unable to get all", ex)
        End Try
    End Function

    Public Overridable Function GetForCriteria(ByVal session As ISession, ByVal criterions As Criterion.ICriterion(), ByVal orders As Criterion.Order()) As IList(Of T)
        Try
            Return CreateCriteria(session, criterions, orders, Nothing).List(Of T)()
        Catch ex As Exception
            ThrowNHibernateException("Unable to get result for criteria", ex)
        End Try
    End Function

    Public Overridable Function GetForCriteriaAndProjection(ByVal session As ISession, ByVal criterions As Criterion.ICriterion(), ByVal orders As Criterion.Order(), ByVal projection As Criterion.IProjection()) As IList(Of T)
        Try
            Return CreateCriteria(session, criterions, orders, projection).SetResultTransformer(New Transform.AliasToBeanResultTransformer(GetType(T))).List(Of T)()
        Catch ex As Exception
            ThrowNHibernateException("Unable to get result for criteria", ex)
        End Try
    End Function

    Private Function CreateCriteria(ByVal session As ISession, ByVal criterions As Criterion.ICriterion(), ByVal orders As Criterion.Order(), ByVal projection As Criterion.IProjection()) As ICriteria
        Dim criteria As ICriteria = session.CreateCriteria(Of T)()
        If criterions IsNot Nothing Then
            For Each criterion As ICriterion In criterions
                criteria.Add(criterion)
            Next
        End If
        If orders IsNot Nothing Then
            For Each order As Criterion.Order In orders
                criteria.AddOrder(order)
            Next
        End If
        If projection IsNot Nothing Then
            criteria.SetProjection(projection)
        End If
        Return criteria
    End Function

    Private Sub ThrowNHibernateException(ByVal message As String, ByVal ex As Exception)
        Throw New NHibernateException(String.Format("{0} {1}", message, Me.GetType().ToString()), ex)
    End Sub
End Class
