﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports DocumentFormat.OpenXml
Imports DocumentFormat.OpenXml.Packaging
Imports DocumentFormat.OpenXml.Wordprocessing
Imports System.IO
Imports System.Xml
Imports System.Xml.Xsl
Imports System.Reflection

Public Class IoWord
    Implements IWordDao
    Private Const TRANSFORMATIONFILE As String = "DocX2Html.xslt"
    Private Const WORD_TEMPLATE As String = "Project_Charter_Template.docx"
    Private m_MemoryStream As MemoryStream
    Private m_WordDocument As WordprocessingDocument

    ''' <summary>
    ''' Open template file.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub OpenTemplate() Implements Business.IWordDao.OpenTemplate
        Dim filename As String = WORD_TEMPLATE
        Try
            filename = String.Format("{0}\{1}", My.Application.Info.DirectoryPath, WORD_TEMPLATE)
            Dim byteArray As Byte() = File.ReadAllBytes(filename)
            InMemoryStream = New MemoryStream()
            InMemoryStream.Write(byteArray, 0, byteArray.Length)
            WordDocument = WordprocessingDocument.Open(InMemoryStream, True)
        Catch ex As Exception
            Throw New WordExecption(String.Format("Unable to open file {0}.", filename), ex.InnerException)
        End Try
    End Sub

    ''' <summary>
    ''' Save the document to file.
    ''' </summary>
    ''' <param name="filename"></param>
    ''' <remarks></remarks>
    Public Sub SaveAs(ByVal filename As String) Implements Business.IWordDao.SaveAs
        Try
            Dim fileStream As FileStream = New FileStream(filename, FileMode.Create)
            InMemoryStream.WriteTo(fileStream)
            fileStream.Flush()
            fileStream.Close()
            fileStream.Dispose()
        Catch ex As Exception
            Throw New WordExecption(String.Format("Unable to save file {0}.", filename), ex.InnerException)
        End Try
    End Sub

    ''' <summary>
    ''' Close the document.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub CloseTemplate()
        Try
            InMemoryStream.Close()
            InMemoryStream.Dispose()
        Catch ex As Exception
            Throw New WordExecption("Unable to close template file.", ex.InnerException)
        End Try
    End Sub

    ''' <summary>
    ''' Convert document to html.
    ''' the html page is returned as a string.
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function PreviewDocumentAsHtml() As String Implements IWordDao.PreviewDocumentAsHtml
        Try
            Dim mainPart As XmlDocument = New XmlDocument()
            mainPart.LoadXml(WordDocument.MainDocumentPart.Document.OuterXml)
            Dim xsltTransformation As XslCompiledTransform = New XslCompiledTransform()
            xsltTransformation.Load(TRANSFORMATIONFILE, XsltSettings.TrustedXslt, New EmbeddedResource())
            Dim memoryStream As MemoryStream = New MemoryStream()
            Dim transformedOutputStream As XmlWriter = XmlWriter.Create(memoryStream)
            xsltTransformation.Transform(mainPart, transformedOutputStream)
            transformedOutputStream.Flush()
            Dim streamReader As StreamReader = New StreamReader(memoryStream)
            memoryStream.Seek(0, SeekOrigin.Begin)
            PreviewDocumentAsHtml = streamReader.ReadToEnd()
            transformedOutputStream.Close()
            streamReader.Close()
            memoryStream.Close()
            streamReader.Dispose()
            transformedOutputStream = Nothing
            memoryStream.Dispose()
            mainPart = Nothing
        Catch ex As Exception
            Throw New WordExecption(String.Format("Unable to convert word document to html.", ex.InnerException))
        End Try
    End Function

    ''' <summary>
    ''' Is there a documenet loaded?
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property HasWordDocument() As Boolean Implements IWordDao.HasWordDocument
        Get
            Return WordDocument IsNot Nothing
        End Get
    End Property

    Private Sub FindAndReplacePlaceHolder(ByVal find As String, ByVal replaceWith As String)
        Dim elementEnumerator As IEnumerator(Of OpenXmlElement) = FindMainDocumentPartElements(find).GetEnumerator()
        While (elementEnumerator.MoveNext())
            elementEnumerator.Current.InnerXml = elementEnumerator.Current.InnerXml.Replace(find, replaceWith)
        End While
    End Sub

    Private Sub FindAndReplacePlaceHolder(ByVal find As String, ByVal table As Table)
        Dim elementEnumerator As IEnumerator(Of OpenXmlElement) = FindMainDocumentPartElements(find).GetEnumerator()
        While (elementEnumerator.MoveNext())
            WordDocument.MainDocumentPart.Document.FirstChild.ReplaceChild(table, elementEnumerator.Current)
        End While
    End Sub

    Private Function FindMainDocumentPartElements(ByVal find As String) As IList(Of OpenXmlElement)
        FindMainDocumentPartElements = New List(Of OpenXmlElement)()
        Dim index As Integer = -1
        Dim originalElement As OpenXmlElement = Nothing
        Dim elements As IEnumerator(Of OpenXmlElement) = WordDocument.MainDocumentPart.Document.FirstChild.ChildElements.GetEnumerator()
        While (elements.MoveNext())
            index += 1
            If (elements.Current.InnerText.Contains(find)) Then
                FindMainDocumentPartElements.Add(elements.Current)
            End If
        End While
    End Function

    Private Function CreateTable(ByVal tableData As WordTableData) As Table
        Dim borderStyle As EnumValue(Of BorderValues) = BorderValues.Single
        Dim borderSize As UInt32Value = 6
        Dim table As Table = New Table()
        'borders
        'top
        Dim topBorder As TopBorder = New TopBorder()
        topBorder.Val = borderStyle
        topBorder.Size = borderSize
        'buttom
        Dim bottomBorder As BottomBorder = New BottomBorder()
        bottomBorder.Val = borderStyle
        bottomBorder.Size = borderSize
        'left
        Dim leftBorder As LeftBorder = New LeftBorder()
        leftBorder.Val = borderStyle
        leftBorder.Size = borderSize
        'right
        Dim rightBorder As RightBorder = New RightBorder()
        rightBorder.Val = borderStyle
        rightBorder.Size = borderSize
        'inside horizontal
        Dim insideHorizontalBorder As InsideHorizontalBorder = New InsideHorizontalBorder()
        insideHorizontalBorder.Val = borderStyle
        insideHorizontalBorder.Size = borderSize
        'inside vertical
        Dim insideVerticalBorder As InsideVerticalBorder = New InsideVerticalBorder()
        insideVerticalBorder.Val = borderStyle
        insideVerticalBorder.Size = borderSize
        Dim tableBorders As TableBorders = New TableBorders(topBorder, bottomBorder, leftBorder, rightBorder, insideHorizontalBorder, insideVerticalBorder)
        Dim tableProperties As TableProperties = New TableProperties(tableBorders)
        table.Append(tableProperties)

        'table type
        table.Append(New TableGrid())

        'headers
        Dim rowElements As IList(Of OpenXmlElement) = New List(Of OpenXmlElement)()
        Dim headerEnumerator As IEnumerator(Of String) = tableData.TableHeaders.GetEnumerator()
        While (headerEnumerator.MoveNext())
            Dim tableCell As TableCell = New TableCell()
            Dim paragraph As Paragraph = New Paragraph()
            Dim run As Run = New Run()
            Dim runProperties As RunProperties = New RunProperties()
            Dim tableCellProperties As TableCellProperties = New TableCellProperties()
            Dim bold As Bold = New Bold()
            Dim text As Text = New Text() With {.Text = headerEnumerator.Current}
            Dim shading As Shading = New Shading() With {.Color = "auto", .Fill = "#909090"}
            tableCellProperties.Append(shading)
            runProperties.Append(bold)
            run.Append(runProperties)
            run.Append(text)
            paragraph.Append(run)
            tableCell.Append(paragraph)
            tableCell.Append(tableCellProperties)
            rowElements.Add(tableCell)
        End While
        AddRow(table, rowElements)
        rowElements.Clear()

        'table data
        Dim cellEnumerator As IEnumerator(Of String) = tableData.TableData.GetEnumerator()
        Dim columnCounter As Integer = 0
        While (cellEnumerator.MoveNext())
            columnCounter += 1
            Dim tableCell As TableCell = New TableCell(New Paragraph(New Run(New Text(cellEnumerator.Current))))
            rowElements.Add(tableCell)
            'if column counter = header count then add row to table, reset the column counter and clear the rowElement list
            If (columnCounter = tableData.TableHeaders.Count) Then
                columnCounter = 0
                AddRow(table, rowElements)
                rowElements.Clear()
            End If
        End While
        Return table
    End Function

    Private Sub AddRow(ByVal table As Table, ByVal rowElements As IList(Of OpenXmlElement))
        Dim tableRow As TableRow = New TableRow(rowElements)
        Dim tableRowHeight As TableRowHeight = New TableRowHeight() With {.Val = CType(284, UInt32Value), .HeightType = HeightRuleValues.Exact}
        Dim tableRowProperties As TableRowProperties = New TableRowProperties()
        tableRowProperties.Append(tableRowHeight)
        tableRow.TableRowProperties = tableRowProperties
        table.Append(tableRow)
    End Sub

    Private Property InMemoryStream() As MemoryStream
        Get
            Return m_MemoryStream
        End Get
        Set(ByVal value As MemoryStream)
            m_MemoryStream = value
        End Set
    End Property

    Private Property WordDocument() As WordprocessingDocument
        Get
            Return m_WordDocument
        End Get
        Set(ByVal value As WordprocessingDocument)
            m_WordDocument = value
        End Set
    End Property

#Region "IWordDao"
    Public Sub GenerateMainDocument(ByVal wordData As IList(Of AbstractWordData)) Implements Business.IWordDao.GenerateMainDocument
        Try
            Dim wordDataEnumerator As IEnumerator(Of AbstractWordData) = wordData.GetEnumerator()
            While (wordDataEnumerator.MoveNext())
                Select Case wordDataEnumerator.Current.DataType
                    Case Business.DataHolders.AbstractWordData.WordDataType.Text
                        FindAndReplacePlaceHolder(wordDataEnumerator.Current.SearchValue, DirectCast(wordDataEnumerator.Current, WordTextData).ReplaceWith)
                    Case Business.DataHolders.AbstractWordData.WordDataType.Table
                        Dim tableData As WordTableData = DirectCast(wordDataEnumerator.Current, WordTableData)
                        Dim table As Table = CreateTable(tableData)
                        FindAndReplacePlaceHolder(wordDataEnumerator.Current.SearchValue, table)
                End Select
            End While
            WordDocument.MainDocumentPart.Document.Save()
        Catch ex As Exception
            Throw New WordExecption("Unable to genererate main document.", ex.InnerException)
        End Try
    End Sub

    Public Sub GenerateFooterDocument(ByVal wordData As System.Collections.Generic.IList(Of Business.DataHolders.AbstractWordData)) Implements Business.IWordDao.GenerateFooterDocument
        Try
            Dim footerPartEnumerator As IEnumerator(Of FooterPart) = WordDocument.MainDocumentPart.FooterParts.GetEnumerator()
            While (footerPartEnumerator.MoveNext())
                Dim wordDataEnumerator As IEnumerator(Of AbstractWordData) = wordData.GetEnumerator()
                While (wordDataEnumerator.MoveNext())
                    Select Case wordDataEnumerator.Current.DataType
                        Case Business.DataHolders.AbstractWordData.WordDataType.Text
                            Dim find As String = wordDataEnumerator.Current.SearchValue
                            Dim replaceWith As String = DirectCast(wordDataEnumerator.Current, WordTextData).ReplaceWith
                            footerPartEnumerator.Current.Footer.InnerXml = footerPartEnumerator.Current.Footer.InnerXml.Replace(find, replaceWith)
                            footerPartEnumerator.Current.Footer.Save()
                    End Select
                End While
            End While
        Catch ex As Exception
            Throw New WordExecption("Unable to genererate footer.", ex.InnerException)
        End Try
    End Sub

    Public Sub GenerateHeaderDocument(ByVal wordData As System.Collections.Generic.IList(Of Business.DataHolders.AbstractWordData)) Implements Business.IWordDao.GenerateHeaderDocument
        Try
            Dim headerPartEnumerator As IEnumerator(Of HeaderPart) = WordDocument.MainDocumentPart.HeaderParts.GetEnumerator()
            While (headerPartEnumerator.MoveNext())
                Dim wordDataEnumerator As IEnumerator(Of AbstractWordData) = wordData.GetEnumerator()
                While (wordDataEnumerator.MoveNext())
                    Select Case wordDataEnumerator.Current.DataType
                        Case Business.DataHolders.AbstractWordData.WordDataType.Text
                            Dim find As String = wordDataEnumerator.Current.SearchValue
                            Dim replaceWith As String = DirectCast(wordDataEnumerator.Current, WordTextData).ReplaceWith
                            headerPartEnumerator.Current.Header.InnerXml = headerPartEnumerator.Current.Header.InnerXml.Replace(find, replaceWith)
                            headerPartEnumerator.Current.Header.Save()
                    End Select
                End While
            End While
        Catch ex As Exception
            Throw New WordExecption("Unable to genererate header.", ex.InnerException)
        End Try
    End Sub
#End Region

End Class
