﻿Option Strict On
Option Explicit On
Option Infer Off
Imports System.Reflection
Imports Securex.FIT.Fitclient.Business.DataHolders

Public MustInherit Class GenericSelection(Of T)
    Private m_Selection As T
    Private m_OriginalSelection As T

    Public Sub New()
        Clear()
    End Sub

    ''' <summary>
    ''' Add a object to selection
    ''' </summary>
    ''' <param name="selectedObject">The object that the selction will contains</param>
    ''' <remarks></remarks>
    Public Overridable Sub Add(ByVal selectedObject As T)
        m_Selection = selectedObject
        m_OriginalSelection = GenericClone.DeepClone(Of T)(selectedObject)
    End Sub

    ''' <summary>
    ''' Undo changes
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub Undo()
        m_Selection = GenericClone.DeepClone(Of T)(m_OriginalSelection)
    End Sub

    ''' <summary>
    ''' Clear the selection
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub Clear()
        m_Selection = Activator.CreateInstance(Of T)()
        m_OriginalSelection = Activator.CreateInstance(Of T)()
    End Sub

    ''' <summary>
    ''' Returns the object in the selection
    ''' </summary>
    ''' <returns>Returns the object in the selection</returns>
    ''' <remarks>The returned object is type safe</remarks>
    Public MustOverride Function GetSelection() As T

    Protected ReadOnly Property Selection() As T
        Get
            Return m_Selection
        End Get
    End Property

    Protected ReadOnly Property OriginalSelection() As T
        Get
            Return m_OriginalSelection
        End Get
    End Property
End Class
