﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Text.RegularExpressions
Imports System.Collections
Imports System.Text
Imports System.Xml
Imports Utils


Public Class TaskDiff
    Private Shared m_TaskDiff As TaskDiff = New TaskDiff()
    Private Const ROOT_NODE_NAME As String = "TextDiff"
    Private Const DEFAULT_CHILD_NODE_NAME As String = "TextNode"
    Private Const DELIMITER As String = " - "

    Private Sub New()
    End Sub

    Public Shared ReadOnly Property Instance() As TaskDiff
        Get
            Return m_TaskDiff
        End Get
    End Property

    ''' <summary>
    ''' Show diffrences between this and the previous project
    ''' </summary>
    ''' <returns>Returns the difference html page as string</returns>
    ''' <remarks></remarks>
    Public Function ShowChanges() As String
        Dim stringbuilder As StringBuilder = New StringBuilder()
        Try
            Dim currentRevision As ProjectProposalRevision = TaskProjectProposalRevision.Instance.GetSelection()
            Dim previousRevision As ProjectProposalRevision = TaskProjectProposalRevision.Instance.GetProjectProposalRevisionById(currentRevision.PreviousRevisionId.Value)
            stringbuilder.Append("<html>")
            stringbuilder.Append("<body>")
            stringbuilder.Append("<div>")
            stringbuilder.Append(GenerateDifference("ProjectName", currentRevision.Project.Name, previousRevision.Project.Name))
            stringbuilder.Append(GenerateDifference("Description", currentRevision.ProjectDescription, previousRevision.ProjectDescription))
            '(sub)category
            Dim currentSubCategoryDescription As String = Nothing
            Dim currentCategoryDescription As String = Nothing
            Dim previousSubCategoryDescription As String = Nothing
            Dim previousCategoryDescription As String = Nothing
            If (currentRevision.SubCategory IsNot Nothing) Then
                currentSubCategoryDescription = currentRevision.SubCategory.Description
                currentCategoryDescription = currentRevision.SubCategory.Category.Description
            End If
            If (previousRevision.SubCategory IsNot Nothing) Then
                previousSubCategoryDescription = previousRevision.SubCategory.Description
                previousCategoryDescription = previousRevision.SubCategory.Category.Description
            End If
            stringbuilder.Append(GenerateDifference("Category", GetSafeValue(currentCategoryDescription), GetSafeValue(previousCategoryDescription)))
            stringbuilder.Append(GenerateDifference("SubCategory", GetSafeValue(currentSubCategoryDescription), GetSafeValue(previousSubCategoryDescription)))
            stringbuilder.Append(GenerateDifference("Creator", currentRevision.Creator.ToString(), previousRevision.Creator.ToString()))
            stringbuilder.Append(GenerateDifference("Auditor", currentRevision.Auditor.ToString(), previousRevision.Auditor.ToString()))
            stringbuilder.Append(GenerateDifference("SponsorName", GetSafeValue(currentRevision.SponsorName), GetSafeValue(previousRevision.SponsorName)))
            stringbuilder.Append(GenerateDifference("PusherName", GetSafeValue(currentRevision.PusherName), GetSafeValue(previousRevision.PusherName)))
            stringbuilder.Append(GenerateDifference("CoordinatorName", GetSafeValue(currentRevision.CoordinatorName), GetSafeValue(previousRevision.CoordinatorName)))
            stringbuilder.Append(GenerateDifference("Budget", GetSafeValue(currentRevision.Budget), GetSafeValue(previousRevision.Budget)))
            stringbuilder.Append(GenerateDifference("Why", GetSafeValue(currentRevision.Why), GetSafeValue(previousRevision.Why)))
            stringbuilder.Append(GenerateDifference("Include", GetSafeValue(currentRevision.Include), GetSafeValue(previousRevision.Include)))
            stringbuilder.Append(GenerateDifference("Exclude", GetSafeValue(currentRevision.Exclude), GetSafeValue(previousRevision.Exclude)))
            stringbuilder.Append(GenerateDifference("AsIs", GetSafeValue(currentRevision.AsIs), GetSafeValue(previousRevision.AsIs)))
            stringbuilder.Append(GenerateDifference("ToBe", GetSafeValue(currentRevision.ToBe), GetSafeValue(previousRevision.ToBe)))
            stringbuilder.Append(GenerateDifference("Objectifs", GetSafeValue(currentRevision.Objectifs), GetSafeValue(previousRevision.Objectifs)))
            stringbuilder.Append(GenerateDifference("Cost", GetSafeValue(currentRevision.Cost), GetSafeValue(previousRevision.Cost)))
            stringbuilder.Append(GenerateDifference("Benefits", GetSafeValue(currentRevision.Benefits), GetSafeValue(previousRevision.Benefits)))
            stringbuilder.Append(GenerateDifference("Alignation", GetSafeValue(currentRevision.Alignation), GetSafeValue(previousRevision.Alignation)))
            stringbuilder.Append(GenerateDifference("Preconditions", GetSafeValue(currentRevision.Preconditions), GetSafeValue(previousRevision.Preconditions)))
            stringbuilder.Append(GenerateDifference("Constraints", GetSafeValue(currentRevision.Constraints), GetSafeValue(previousRevision.Constraints)))
            stringbuilder.Append(GenerateDifference("Dependencies", GetSafeValue(currentRevision.Dependencies), GetSafeValue(previousRevision.Dependencies)))
            stringbuilder.Append(GenerateDifference("Risk", GetSafeValue(currentRevision.Risk), GetSafeValue(previousRevision.Risk)))
            stringbuilder.Append(GenerateDifference("Support", GetSafeValue(currentRevision.Support), GetSafeValue(previousRevision.Support)))
            'workpackage(s)
            Dim currentWorkpackages As String = Nothing
            Dim previousWorkpackages As String = Nothing
            If (currentRevision.TeamMembers IsNot Nothing) Then
                currentWorkpackages = ConvertWorkPackage(currentRevision.WorkPackages)
            End If
            If (previousRevision.TeamMembers IsNot Nothing) Then
                previousWorkpackages = ConvertWorkPackage(previousRevision.WorkPackages)
            End If
            stringbuilder.Append(GenerateDifference("Workpackages", GetSafeValue(currentWorkpackages), GetSafeValue(previousWorkpackages)))

            'teammember(s)
            Dim currentTeammembers As String = Nothing
            Dim previousTeammembers As String = Nothing
            If (currentRevision.TeamMembers IsNot Nothing) Then
                currentTeammembers = ConvertTeamMembers(currentRevision.TeamMembers)
            End If
            If (previousRevision.TeamMembers IsNot Nothing) Then
                previousTeammembers = ConvertTeamMembers(previousRevision.TeamMembers)
            End If
            stringbuilder.Append(GenerateDifference("Teammembers", GetSafeValue(currentTeammembers), GetSafeValue(previousTeammembers)))
            stringbuilder.Append("</div>")
            stringbuilder.Append("</body>")
            stringbuilder.Append("</html>")
        Catch ex As Exception
            Throw New Exception("Unable to generate diffence view.", ex.InnerException)
        End Try
        Return stringbuilder.ToString()
    End Function

    Private Function GetSafeValue(ByVal value As String) As String
        GetSafeValue = ""
        If (value IsNot Nothing) Then
            GetSafeValue = value
        End If
    End Function

    Private Function ConvertWorkPackage(ByVal list As IList(Of WorkPackage)) As String
        Dim stringbuilder As StringBuilder = New StringBuilder()
        Dim enumerator As IEnumerator(Of WorkPackage) = list.GetEnumerator()
        While (enumerator.MoveNext())
            stringbuilder.Append(enumerator.Current.Name)
            stringbuilder.Append(DELIMITER)
            stringbuilder.Append(enumerator.Current.EstimatedTime)
            stringbuilder.Append(DELIMITER)
            stringbuilder.Append(enumerator.Current.ResponsibleName)
            stringbuilder.Append(DELIMITER)
            stringbuilder.Append(enumerator.Current.DueDate.ToShortDateString())
            stringbuilder.Append(vbLf)
            Dim milestoneEnumerator As IEnumerator(Of Milestone) = enumerator.Current.Milestones.GetEnumerator()
            While (milestoneEnumerator.MoveNext())
                stringbuilder.Append(milestoneEnumerator.Current.Name)
                stringbuilder.Append(DELIMITER)
                stringbuilder.Append(milestoneEnumerator.Current.Description)
                stringbuilder.Append(DELIMITER)
                stringbuilder.Append(milestoneEnumerator.Current.BeginDate.ToShortDateString())
                stringbuilder.Append(DELIMITER)
                stringbuilder.Append(milestoneEnumerator.Current.EndDate.ToShortDateString())
                stringbuilder.Append(vbLf)
            End While
        End While
        Return stringbuilder.ToString()
    End Function

    Private Function ConvertTeamMembers(ByVal list As IList(Of TeamMember)) As String
        Dim stringbuilder As StringBuilder = New StringBuilder()
        Dim enumerator As IEnumerator(Of TeamMember) = list.GetEnumerator()
        While (enumerator.MoveNext())
            stringbuilder.Append(enumerator.Current.Role.Name)
            stringbuilder.Append(DELIMITER)
            stringbuilder.Append(enumerator.Current.EmployeeName)
            stringbuilder.Append(DELIMITER)
            stringbuilder.Append(enumerator.Current.Department)
            stringbuilder.Append(DELIMITER)
            stringbuilder.Append(enumerator.Current.EmailAddress)
            stringbuilder.Append(vbLf)
        End While
        Return stringbuilder.ToString()
    End Function

    Private Function GenerateDifference(ByVal propertyName As String, ByVal current As String, ByVal previous As String) As String
        Dim stringbuilder As StringBuilder = New StringBuilder()
        stringbuilder.Append("<h3>")
        stringbuilder.Append(propertyName)
        stringbuilder.Append("</h3>")
        Dim docXml As XmlDocument = FindDifference(current, previous)
        Dim nodeEnumerator As IEnumerator = docXml.SelectSingleNode(ROOT_NODE_NAME).ChildNodes.GetEnumerator()
        While (nodeEnumerator.MoveNext())
            Dim node As XmlNode = DirectCast(nodeEnumerator.Current, XmlNode)
            stringbuilder.Append("<p>")
            Dim span As String = "<span>"
            If (node.Attributes.Count > 0) Then
                span = String.Format("<span style=background-color:{0}>", node.Attributes("bgcolor").InnerXml)
            End If
            stringbuilder.Append(span)
            stringbuilder.Append(node.InnerText)
            stringbuilder.Append("</span>")
            stringbuilder.Append("</p>")
        End While
        Return stringbuilder.ToString()
    End Function

    Private Function FindDifference(ByVal current As String, ByVal previous As String) As XmlDocument
        Dim xmlDoc As XmlDocument = New XmlDocument()
        'encoding
        xmlDoc.AppendChild(xmlDoc.CreateXmlDeclaration("1.0", "UTF-8", "no"))
        'root node
        Dim rootNode As XmlElement = xmlDoc.CreateElement(ROOT_NODE_NAME)
        xmlDoc.AppendChild(rootNode)
        Dim xmlNodeCounter As Integer

        Dim diffItemArray As Diff.Item() = Diff.DiffText(previous, current, True, True, True)
        Dim compareWithLines As String() = Regex.Split(previous, "\n")
        Dim currentLines As String() = Regex.Split(current, "\n")
        Dim currentIndex As Integer
        For index As Integer = 0 To diffItemArray.Length - 1
            Dim diffItem As Diff.Item = diffItemArray(index)
            'same
            While (currentIndex < diffItem.MainIndex) AndAlso (currentIndex < currentLines.Length)
                AddNode(xmlDoc, xmlNodeCounter, rootNode, DEFAULT_CHILD_NODE_NAME, currentLines(currentIndex), Nothing)
                currentIndex += 1
            End While
            'deleted
            For compareWithIndex As Integer = 0 To diffItem.CompareWithDeleteCount - 1
                AddNode(xmlDoc, xmlNodeCounter, rootNode, DEFAULT_CHILD_NODE_NAME, compareWithLines(diffItem.CompareWithIndex + compareWithIndex), "#FF8888")
            Next
            'insert
            While (currentIndex < diffItem.MainIndex + diffItem.MainInsertCount)
                AddNode(xmlDoc, xmlNodeCounter, rootNode, DEFAULT_CHILD_NODE_NAME, currentLines(currentIndex), "LightGreen")
                currentIndex += 1
            End While
        Next
        'rest of unchanged lines
        While (currentIndex < currentLines.Length)
            AddNode(xmlDoc, xmlNodeCounter, rootNode, DEFAULT_CHILD_NODE_NAME, currentLines(currentIndex), Nothing)
            currentIndex += 1
        End While
        Return DirectCast(xmlDoc.Clone(), XmlDocument)
    End Function

    Private Sub AddNode(ByVal xmlDoc As XmlDocument, ByRef xmlNodeCounter As Integer, ByVal rootNode As XmlElement, ByVal defaultChildNodeName As String, ByVal text As String, ByVal colorName As String)
        xmlNodeCounter += 1
        Dim childNodeName As String = String.Format("{0}{1}", defaultChildNodeName, xmlNodeCounter.ToString())
        'create new node
        Dim newNode As XmlNode = xmlDoc.CreateNode(XmlNodeType.Element, childNodeName, Nothing)
        If (text IsNot Nothing) Then
            newNode.InnerText = text
        End If
        If (colorName IsNot Nothing) Then
            'assign attribute
            DirectCast(newNode, XmlElement).SetAttribute("bgcolor", colorName)
        End If
        'add node
        rootNode.AppendChild(newNode)
    End Sub
End Class
