﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business.DataHolders

Public Class TaskInformation
    Implements IInformation
    Private Shared m_Information As TaskInformation = New TaskInformation()
    Private m_UserID As String
    Private m_ApplicationName As String
    Private m_ApplicationVersion As String
    Private m_IsDevelopment As Boolean
    Private m_Privileges As Privileges

    Private Sub New()
    End Sub

    Public Shared ReadOnly Property Instance() As TaskInformation
        Get
            Return m_Information
        End Get
    End Property

    ''' <summary>
    ''' Is the current person an administrator
    ''' </summary>
    ''' <returns>Returns true or false</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property IsAdmin() As Boolean
        Get
            Return m_Privileges.IsAdmin
        End Get
    End Property

    ''' <summary>
    ''' Get the current person's employee id
    ''' </summary>
    ''' <returns>Returns the employee id</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property UserId() As String
        Get
            Return m_UserID
        End Get
    End Property

    ''' <summary>
    ''' Is the current person the project owner
    ''' </summary>
    ''' <param name="creator">creator object</param>
    ''' <returns>Returns true or false</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property IsCreator(ByVal creator As Creator) As Boolean
        Get
            Return UserId.Equals(creator.EmployeeId)
        End Get
    End Property

    ''' <summary>
    ''' Is the current person the reviewer for this project
    ''' </summary>
    ''' <param name="auditor">Auditor object</param>
    ''' <returns>Returns true or false</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property IsAuditor(ByVal auditor As Auditor) As Boolean
        Get
            Return auditor IsNot Nothing AndAlso auditor.EmployeeId IsNot Nothing AndAlso auditor.EmployeeId.Equals(UserId)
        End Get
    End Property

    ''' <summary>
    ''' Get the application name
    ''' </summary>
    ''' <returns>Returns the application name</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property ApplicationName() As String
        Get
            Return m_ApplicationName
        End Get
    End Property

    ''' <summary>
    ''' Get the application version
    ''' </summary>
    ''' <returns>Returns the application version</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property ApplicationVersion() As String
        Get
            Return m_ApplicationVersion
        End Get
    End Property

    ''' <summary>
    ''' Is the program started in debug mode
    ''' </summary>
    ''' <returns>Returns true or false</returns>
    ''' <remarks>
    ''' True = Acceptation
    ''' False = Production
    ''' </remarks>
    Public ReadOnly Property IsDevelopment() As Boolean
        Get
            Return m_IsDevelopment
        End Get
    End Property

    ''' <summary>
    ''' Set the privileges for this person
    ''' </summary>
    ''' <value>The privilage object for this person</value>
    ''' <remarks></remarks>
    Public WriteOnly Property SetPrivileges() As Privileges
        Set(ByVal value As Privileges)
            m_Privileges = value
        End Set
    End Property

#Region "IUserInformation"
    ''' <summary>
    ''' Set the application name
    ''' </summary>
    ''' <value>The application name</value>
    ''' <remarks></remarks>
    Public WriteOnly Property SetApplicationName() As String Implements IInformation.SetApplicationName
        Set(ByVal value As String)
            m_ApplicationName = value
        End Set
    End Property

    ''' <summary>
    ''' Set the application version
    ''' </summary>
    ''' <value>The application version</value>
    ''' <remarks></remarks>
    Public WriteOnly Property SetApplicationVersion() As String Implements IInformation.SetApplicationVersion
        Set(ByVal value As String)
            m_ApplicationVersion = value
        End Set
    End Property

    ''' <summary>
    ''' Set the employee id
    ''' </summary>
    ''' <value>Employee id</value>
    ''' <remarks></remarks>
    Public WriteOnly Property SetUserID() As String Implements IInformation.SetUserID
        Set(ByVal value As String)
            m_UserID = value
        End Set
    End Property

    ''' <summary>
    ''' Set the running environment
    ''' </summary>
    ''' <value>True or false</value>
    ''' <remarks>
    ''' True = Acceptation
    ''' False = Production
    ''' </remarks>
    Public WriteOnly Property SetIsDevelopment() As Boolean Implements IInformation.SetIsDevelopment
        Set(ByVal value As Boolean)
            m_IsDevelopment = value
        End Set
    End Property
#End Region

End Class
