﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Presenters
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports CustomControls
Imports System.Windows.Forms
Imports System.Drawing

Public Class FormCategoryOverview
    Implements IViewFormCategoryOverview
    Private m_Presenter As PresenterFormCategoryOverview
    Private m_MouseLocationCategory As Point
    Private m_MouseLocationSubCategory As Point
    Private m_CacheComparator As ExtendedListviewComparator
    Private Const SCREEN_NAME As String = "Category Overview"

    Public Sub New()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        Presenter = New PresenterFormCategoryOverview(Me)
        CacheComparator = New ExtendedListviewComparator()

        'disable ToolStripButtonRemoveCategory ,ToolStripButtonRemoveSubCategory and ToolStripButtonAddSubCategory
        ToolStripButtonRemoveCategoryEnabled(False)
        ToolStripButtonRemoveSubCategoryEnabled(False)
        ToolStripButtonAddSubCategoryEnabled(False)
    End Sub

    Private Sub ToolStripButtonAddCategory_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonAddCategory.Click
        OpenCategoryEditor(Nothing)
    End Sub

    Private Sub ToolStripButtonRemoveCategory_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonRemoveCategory.Click
        Try
            Dim listViewItem As ListViewItem = ListViewCategory.GetItemAt(MouseLocationCategory.X, MouseLocationCategory.Y)
            If (listViewItem IsNot Nothing) Then
                If (MessageBox.Show("Remove selected data?", "Question", MessageBoxButtons.YesNo) = Windows.Forms.DialogResult.Yes) Then
                    Presenter.RemoveCategory(Convert.ToInt32(listViewItem.Tag))
                    Presenter.SelectedCatagoryId = -1
                    Presenter.RequestAllCategories()
                    Presenter.RequestSubCategoriesForCategory(Presenter.SelectedCatagoryId)
                End If
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try
    End Sub

    Private Sub ToolStripButtonAddSubCategory_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonAddSubCategory.Click
        Dim listViewItem As ListViewItem = ListViewCategory.GetItemAt(MouseLocationCategory.X, MouseLocationCategory.Y)
        If (listViewItem IsNot Nothing) Then
            OpenSubCategoryEditor(listViewItem.Text)
        End If
    End Sub

    Private Sub ToolStripButtonRemoveSubCategory_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonRemoveSubCategory.Click
        Try
            Dim listViewItem As ListViewItem = ListViewSubCategory.GetItemAt(MouseLocationSubCategory.X, MouseLocationSubCategory.Y)
            If (listViewItem IsNot Nothing) Then
                If (MessageBox.Show("Remove selected data?", "Question", MessageBoxButtons.YesNo) = Windows.Forms.DialogResult.Yes) Then
                    Presenter.RemoveSubCategory(Convert.ToInt32(listViewItem.Tag))
                    Presenter.RequestSubCategoriesForCategory(Presenter.SelectedCatagoryId)
                End If
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try
    End Sub

    Private Sub ToolStripButtonClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonClose.Click
        Presenter.Close()
    End Sub

    Private Sub ListViewCategory_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewCategory.DoubleClick
        Dim listViewItem As ListViewItem = ListViewCategory.GetItemAt(MouseLocationCategory.X, MouseLocationCategory.Y)
        If (listViewItem IsNot Nothing) Then
            OpenCategoryEditor(listViewItem.Text)
        End If
    End Sub

    Private Sub ListViewCategory_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles ListViewCategory.MouseDown
        MouseLocationCategory = e.Location
        Dim hasSelection As Boolean
        Dim listViewItem As ListViewItem = ListViewCategory.GetItemAt(MouseLocationCategory.X, MouseLocationCategory.Y)
        If (listViewItem IsNot Nothing) Then
            hasSelection = True
            Presenter.SelectedCatagoryId = Convert.ToInt32(listViewItem.Tag)
            Presenter.RequestSubCategoriesForCategory(Presenter.SelectedCatagoryId)
        End If
        ToolStripButtonRemoveCategoryEnabled(hasSelection)
        ToolStripButtonAddSubCategoryEnabled(hasSelection)
    End Sub

    Private Sub ListViewSubCategory_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewSubCategory.DoubleClick
        Dim listViewItem As ListViewItem = ListViewSubCategory.GetItemAt(MouseLocationSubCategory.X, MouseLocationSubCategory.Y)
        If (listViewItem IsNot Nothing) Then
            OpenSubCategoryEditor(listViewItem.Text)
        End If
    End Sub

    Private Sub ListViewSubCategory_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles ListViewSubCategory.MouseDown
        MouseLocationSubCategory = e.Location
        Dim hasSelection As Boolean
        If (ListViewSubCategory.GetItemAt(MouseLocationSubCategory.X, MouseLocationSubCategory.Y) IsNot Nothing) Then
            hasSelection = True
        End If
        ToolStripButtonRemoveSubCategoryEnabled(hasSelection)
    End Sub

    Private Sub SortListViewCategory(ByVal sender As System.Object, ByVal e As CustomControls.SortVirtualExtendedListviewEventArgs) Handles ListViewCategory.SortVirtualExtendedListview
        CacheComparator.Order = e.Order
        CacheComparator.ColumnIndex = e.ColumnIndex
        ListViewCategory.VirtualModeCache.Sort(CacheComparator)
        'Presenter.ListViewCategoryItemCache.Sort(CacheComparator)
        ListViewCategory.Invalidate()
    End Sub

    Private Sub SortListViewSubCategory(ByVal sender As System.Object, ByVal e As CustomControls.SortVirtualExtendedListviewEventArgs) Handles ListViewSubCategory.SortVirtualExtendedListview
        CacheComparator.Order = e.Order
        CacheComparator.ColumnIndex = e.ColumnIndex
        ListViewSubCategory.VirtualModeCache.Sort(CacheComparator)
        ListViewSubCategory.Invalidate()
    End Sub

    Private Sub OpenCategoryEditor(ByVal description As String)
        ToolStripButtonCloseEnabled(False)
        Presenter.ShowCategoryEditor(description)
    End Sub

    Private Sub OpenSubCategoryEditor(ByVal description As String)
        ToolStripButtonCloseEnabled(False)
        Presenter.ShowSubCategoryEditor(description)
    End Sub

    Private Sub ToolStripButtonRemoveCategoryEnabled(ByVal enable As Boolean)
        ToolStripButtonRemoveCategory.Enabled = enable
    End Sub

    Private Sub ToolStripButtonRemoveSubCategoryEnabled(ByVal enable As Boolean)
        ToolStripButtonRemoveSubCategory.Enabled = enable
    End Sub

    Private Sub ToolStripButtonAddSubCategoryEnabled(ByVal enable As Boolean)
        ToolStripButtonAddSubCategory.Enabled = enable
    End Sub

    Private Sub ToolStripButtonCloseEnabled(ByVal enable As Boolean)
        ToolStripButtonClose.Enabled = enable
    End Sub

    Private Property Presenter() As PresenterFormCategoryOverview
        Get
            Return m_Presenter
        End Get
        Set(ByVal value As PresenterFormCategoryOverview)
            m_Presenter = value
        End Set
    End Property

    Private Property MouseLocationCategory() As Point
        Get
            Return m_MouseLocationCategory
        End Get
        Set(ByVal value As Point)
            m_MouseLocationCategory = value
        End Set
    End Property

    Private Property MouseLocationSubCategory() As Point
        Get
            Return m_MouseLocationSubCategory
        End Get
        Set(ByVal value As Point)
            m_MouseLocationSubCategory = value
        End Set
    End Property

    Private Property CacheComparator() As ExtendedListviewComparator
        Get
            Return m_CacheComparator
        End Get
        Set(ByVal value As ExtendedListviewComparator)
            m_CacheComparator = value
        End Set
    End Property

#Region "IViewFormCategoryOverview"
    Public Function GetChildPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetChildPanel
        Return Nothing
    End Function

    Public Function GetMainPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetMainPanel
        Return PanelMain
    End Function

    Public Function GetText() As String Implements Presenters.IChildView.GetText
        Return SCREEN_NAME
    End Function

    Public Function GetToolstrip() As System.Windows.Forms.ToolStrip Implements Presenters.IChildView.GetToolstrip
        Return ToolStripMain
    End Function

    Public Sub ShowChild() Implements Presenters.IChildView.ShowChild
        Try
            'populate listview
            Presenter.RequestAllCategories()
            If Not (Presenter.SelectedCatagoryId.Equals(-1)) Then
                Presenter.RequestSubCategoriesForCategory(Presenter.SelectedCatagoryId)
            End If
            Show()
            Presenter.SendChangedViewEvent()
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try
    End Sub

    Public Sub UpdateTopLevel(ByVal value As Boolean) Implements Presenters.IChildView.UpdateTopLevel
        SetTopLevel(value)
    End Sub

    Public Sub UpdateCategory() Implements Presenters.IViewFormCategoryOverview.UpdateCategory
        UpdateView(ListViewCategory)
    End Sub

    Public Sub UpdateSubCategory() Implements Presenters.IViewFormCategoryOverview.UpdateSubCategory
        UpdateView(ListViewSubCategory)
    End Sub

    Private Sub UpdateView(ByVal listView As ExtendedListView)
        ToolStripButtonRemoveCategoryEnabled(False)
        ToolStripButtonRemoveSubCategoryEnabled(False)
        ToolStripButtonCloseEnabled(True)
        listView.Sort()
        listView.Invalidate()
    End Sub

    Public Sub ClearExtendedListViewCategory() Implements Presenters.IViewFormCategoryOverview.ClearExtendedListViewCategory
        ListViewCategory.VirtualModeCache.Clear()
    End Sub

    Public Sub AddToExtendedListViewCategory(ByVal fourStateListViewItem As CustomControls.FourStateListViewItem) Implements Presenters.IViewFormCategoryOverview.AddToExtendedListViewCategory
        ListViewCategory.VirtualModeCache.Add(fourStateListViewItem)
    End Sub

    Public Sub ClearExtendedListViewSubCategory() Implements Presenters.IViewFormCategoryOverview.ClearExtendedListViewSubCategory
        ListViewSubCategory.VirtualModeCache.Clear()
    End Sub

    Public Sub AddToExtendedListViewSubCategory(ByVal fourStateListViewItem As CustomControls.FourStateListViewItem) Implements Presenters.IViewFormCategoryOverview.AddToExtendedListViewSubCategory
        ListViewSubCategory.VirtualModeCache.Add(fourStateListViewItem)
    End Sub
#End Region

End Class