﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Presenters
Imports System.Windows.Forms

Public Class FormMain
    Implements IViewFormMain
    Private m_Presenter As PresenterFormMain
    Private m_FormProjectEditor As FormProjectEditor
    Private m_FormOverview As FormOverview
    Private m_FormCategory As FormCategory
    Private m_FormPartDescription As FormPartDescription
    Private m_FormRole As FormRole
    Private m_ChildViews As IDictionary(Of String, IChildView)

    Public Sub New()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        'initialize dictionary
        ChildViews = New Dictionary(Of String, IChildView)

        'initialize forms and add to list
        ChildViews.Add(PresenterFormMain.OVERVIEW, New FormOverview())
        ChildViews.Add(PresenterFormMain.PROJECT_EDITOR, ProjectEditor())
        ChildViews.Add(PresenterFormMain.CATEGORIES, Categories())
        ChildViews.Add(PresenterFormMain.PART_DESCRIPTION, PartDescription())
        ChildViews.Add(PresenterFormMain.ROLES, Role())

        'initialize presenter
        Presenter = New PresenterFormMain(Me, ChildViews)

        'reclame grip padding
        StatusStripMain.Padding = New Padding(StatusStripMain.Padding.Left, StatusStripMain.Padding.Top, 0, StatusStripMain.Padding.Bottom)
    End Sub

    Private Property Presenter() As PresenterFormMain
        Get
            Return m_Presenter
        End Get
        Set(ByVal value As PresenterFormMain)
            m_Presenter = value
        End Set
    End Property

    Private Property ChildViews() As IDictionary(Of String, IChildView)
        Get
            Return m_ChildViews
        End Get
        Set(ByVal value As IDictionary(Of String, IChildView))
            m_ChildViews = value
        End Set
    End Property

    Private Property ProjectEditor() As FormProjectEditor
        Get
            Return m_FormProjectEditor
        End Get
        Set(ByVal value As FormProjectEditor)
            m_FormProjectEditor = value
        End Set
    End Property

    Private Property Categories() As FormCategory
        Get
            Return m_FormCategory
        End Get
        Set(ByVal value As FormCategory)
            m_FormCategory = value
        End Set
    End Property

    Private Property PartDescription() As FormPartDescription
        Get
            Return m_FormPartDescription
        End Get
        Set(ByVal value As FormPartDescription)
            m_FormPartDescription = value
        End Set
    End Property

    Private Property Role() As FormRole
        Get
            Return m_FormRole
        End Get
        Set(ByVal value As FormRole)
            m_FormRole = value
        End Set
    End Property

#Region "IViewFormMain"
    Public Function GetPanel() As System.Windows.Forms.Panel Implements Presenters.IForm.GetMainPanel
        Return PanelMain
    End Function

    Public Function GetToolStrip() As System.Windows.Forms.ToolStrip Implements Presenters.IViewFormMain.GetToolstrip
        Return ToolStripMain
    End Function

    Public Sub UpdateText(ByVal value As String) Implements Presenters.IViewFormMain.UpdateText
        Text = value
    End Sub

    Public Sub CloseProjectEditor() Implements Presenters.IViewFormMain.CloseProjectEditor
        ProjectEditor = Nothing
    End Sub

    Public Sub CreateProjectEditor(ByVal project As Business.DataHolders.ProjectProposalRevision) Implements Presenters.IViewFormMain.CreateProjectEditor
        If (project IsNot Nothing) Then
            ProjectEditor = New FormProjectEditor(project)
        Else
            ProjectEditor = New FormProjectEditor(Presenter.UserId)
        End If
    End Sub

    Public Function GetProjectEditor() As IViewFormProjectEditor Implements Presenters.IViewFormMain.GetProjectEditor
        Return ProjectEditor()
    End Function

    Public Sub CloseCategories() Implements Presenters.IViewFormMain.CloseCategories
        Categories = Nothing
    End Sub

    Public Sub CreateCategories() Implements Presenters.IViewFormMain.CreateCategories
        Categories = New FormCategory()
    End Sub

    Public Function GetCategories() As Presenters.IViewFormCategory Implements Presenters.IViewFormMain.GetCategories
        Return Categories
    End Function

    Public Sub ClosePartDescriptionEditor() Implements Presenters.IViewFormMain.ClosePartDescription
        PartDescription = Nothing
    End Sub

    Public Sub CreatePartDescriptionEditor() Implements Presenters.IViewFormMain.CreatePartDescription
        PartDescription = New FormPartDescription()
    End Sub

    Public Function GetPartDescriptionEditor() As Presenters.IViewFormPartDescription Implements Presenters.IViewFormMain.GetPartDescription
        Return PartDescription
    End Function

    Public Sub RefreshForm() Implements Presenters.IViewFormMain.RefreshForm
        Refresh()
    End Sub

    Public Sub SetApplicationVersion(ByVal version As String) Implements Presenters.IViewFormMain.SetApplicationVersion
        ToolStripStatusLabelVersion.Text = version
    End Sub

    Public Sub CloseRole() Implements Presenters.IViewFormMain.CloseRole
        Role = Nothing
    End Sub

    Public Sub CreateRole() Implements Presenters.IViewFormMain.CreateRole
        Role = New FormRole()
    End Sub

    Public Function GetRole() As Presenters.IViewFormRole Implements Presenters.IViewFormMain.GetRole
        Return Role
    End Function
#End Region

End Class
