﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports Securex.FIT.Fitclient.Presenters
Imports System.Windows.Forms
Imports System.Drawing
Imports CustomControls

Public Class FormOverview
    Implements IViewFormOverview
    Private m_Presenter As PresenterFormOverview
    Private m_MouseLocationName As Point
    Private m_MouseLocationRevision As Point
    Private m_CacheComparator As ExtendedListviewComparator
    Private Const SCREEN_NAME As String = "Overview"

    Public Sub New()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        'initialize presenter
        Presenter = New PresenterFormOverview(Me)
        CacheComparator = New ExtendedListviewComparator()
    End Sub

    Private Sub ToolStripButtonNew_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonNew.Click
        Presenter.SendOpenProjectEditor(Nothing)
    End Sub

    Private Sub ToolStripButtonCategories_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonCategories.Click
        Presenter.SendOpenCategories()
    End Sub

    Private Sub ToolStripButtonPartDescription_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonPartDescription.Click
        Presenter.SendOpenPartDescription()
    End Sub

    Private Sub ToolStripButtonRoles_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonRoles.Click
        Presenter.SendOpenRoles()
    End Sub

    Private Sub ListViewName_MouseClick(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles ListViewName.MouseClick
        If (e.Button = Windows.Forms.MouseButtons.Left) Then
            Try
                Cursor.Current = Cursors.WaitCursor
                Dim listViewItem As ListViewItem = ListViewName.GetItemAt(MouseLocationName.X, MouseLocationName.Y)
                If (listViewItem IsNot Nothing) Then
                    Presenter.SelectedProjectNameId = Convert.ToInt32(listViewItem.Tag)
                    Presenter.RequestAllRevisionsForProject(Presenter.SelectedProjectNameId)
                End If
            Catch ex As Exception
                MessageBox.Show(ex.Message)
            Finally
                Cursor.Current = Cursors.Default
            End Try
        End If
    End Sub

    Private Sub ListViewName_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles ListViewName.MouseDown
        MouseLocationName = e.Location
    End Sub

    Private Sub ListViewRevision_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewRevision.DoubleClick
        Try
            Cursor.Current = Cursors.WaitCursor
            Dim listViewItemProjectName As ListViewItem = ListViewName.GetItemAt(MouseLocationName.X, MouseLocationName.Y)
            Dim projectName As ProjectProposalName = New ProjectProposalName()
            projectName.Id = Convert.ToInt32(listViewItemProjectName.Tag)
            projectName.Name = listViewItemProjectName.Text
            Dim listViewItemRevision As ListViewItem = ListViewRevision.GetItemAt(MouseLocationRevision.X, MouseLocationRevision.Y)
            If (listViewItemRevision IsNot Nothing) Then
                Presenter.ShowProjectEditor(Convert.ToInt32(listViewItemRevision.Tag))
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        Finally
            Cursor.Current = Cursors.Default
        End Try
    End Sub

    Private Sub ListViewRevision_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles ListViewRevision.MouseDown
        MouseLocationRevision = e.Location
    End Sub

    Private Sub ToolStripMenuItemNewRevision_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripMenuItemNewRevision.Click
        'create new revision based on previous one
        Try
            Cursor.Current = Cursors.WaitCursor
            Presenter.CreateNewRevision()
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        Finally
            Cursor.Current = Cursors.Default
        End Try
    End Sub

    Private Sub SortListViewName(ByVal sender As System.Object, ByVal e As CustomControls.SortVirtualExtendedListviewEventArgs) Handles ListViewName.SortVirtualExtendedListview
        CacheComparator.Order = e.Order
        CacheComparator.ColumnIndex = e.ColumnIndex
        ListViewName.VirtualModeCache.Sort(CacheComparator)
        ListViewName.Invalidate()
    End Sub

    Private Property Presenter() As PresenterFormOverview
        Get
            Return m_Presenter
        End Get
        Set(ByVal value As PresenterFormOverview)
            m_Presenter = value
        End Set
    End Property

    Private Property MouseLocationName() As Point
        Get
            Return m_MouseLocationName
        End Get
        Set(ByVal value As Point)
            m_MouseLocationName = value
        End Set
    End Property

    Private Property MouseLocationRevision() As Point
        Get
            Return m_MouseLocationRevision
        End Get
        Set(ByVal value As Point)
            m_MouseLocationRevision = value
        End Set
    End Property

    Private Property CacheComparator() As ExtendedListviewComparator
        Get
            Return m_CacheComparator
        End Get
        Set(ByVal value As ExtendedListviewComparator)
            m_CacheComparator = value
        End Set
    End Property

#Region "IViewFormOverview"
    Public Function GetChildPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetChildPanel
        Return Nothing
    End Function

    Public Sub ShowChild() Implements Presenters.IChildView.ShowChild
        Try
            Cursor.Current = Cursors.WaitCursor
            'has the current user an admin function
            Presenter.RequestIsAdmin()
            '
            Presenter.RequestAllProjectNames()
            Presenter.RequestAllRevisionsForProject(Presenter.SelectedProjectNameId)
            Show()
            Presenter.SendChangedViewEvent()
        Catch ex As Exception
            'ignore
        Finally
            Cursor.Current = Cursors.Default
        End Try
    End Sub

    Public Sub UpdateTopLevel(ByVal value As Boolean) Implements Presenters.IChildView.UpdateTopLevel
        SetTopLevel(value)
    End Sub

    Public Function GetMainPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetMainPanel
        Return PanelMain
    End Function

    Public Function GetToolstrip() As System.Windows.Forms.ToolStrip Implements Presenters.IChildView.GetToolstrip
        Return ToolStripMain
    End Function

    Public Function GetText() As String Implements Presenters.IChildView.GetText
        Return SCREEN_NAME
    End Function

    Public Sub UpdateNames() Implements Presenters.IViewFormOverview.UpdateNames
        ListViewName.Sort()
        ListViewName.Invalidate()
    End Sub

    Public Sub UpdateRevisions() Implements Presenters.IViewFormOverview.UpdateRevisions
        ListViewRevision.Invalidate()
    End Sub

    Public Sub AdminFunction(ByVal enabled As Boolean) Implements Presenters.IViewFormOverview.AdminFunction
        ToolStripButtonCategories.Enabled = enabled
        ToolStripButtonCategories.Visible = enabled
        '
        ToolStripButtonPartDescription.Enabled = enabled
        ToolStripButtonPartDescription.Visible = enabled
        '
        ToolStripButtonRoles.Enabled = enabled
        ToolStripButtonRoles.Visible = enabled
    End Sub

    Public Sub ClearExtendedListViewName() Implements Presenters.IViewFormOverview.ClearExtendedListViewName
        ListViewName.VirtualModeCache.Clear()
    End Sub

    Public Sub AddToExtendedListViewName(ByVal fourStateListViewItem As CustomControls.FourStateListViewItem) Implements Presenters.IViewFormOverview.AddToExtendedListViewName
        ListViewName.VirtualModeCache.Add(fourStateListViewItem)
    End Sub

    Public Sub ClearExtendedListViewRevision() Implements Presenters.IViewFormOverview.ClearExtendedListViewRevision
        ListViewRevision.VirtualModeCache.Clear()
    End Sub

    Public Sub AddToExtendedListViewRevision(ByVal fourStateListViewItem As CustomControls.FourStateListViewItem) Implements Presenters.IViewFormOverview.AddToExtendedListViewRevision
        ListViewRevision.VirtualModeCache.Add(fourStateListViewItem)
    End Sub
#End Region

End Class