﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Presenters
Imports Securex.FIT.Fitclient.Business.DataHolders

Public Class FormPartDescription
    Implements IViewFormPartDescription
    Private m_Presenter As PresenterFormPartDescription
    Private m_FormPartDescriptionOverview As FormPartDescriptionOverview
    Private m_FormPartDescriptionEditor As FormPartDescriptionEditor
    Private m_ChildViews As IDictionary(Of String, IChildView)
    Private Const SCREEN_NAME As String = "Form Description"

    Public Sub New()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        'initialize dictionary
        ChildViews = New Dictionary(Of String, IChildView)

        'initialize forms and add to list
        ChildViews.Add(PresenterFormPartDescription.PART_DESCRIPTION_OVERVIEW, New FormPartDescriptionOverview())
        ChildViews.Add(PresenterFormPartDescription.PART_DESCRIPTION_EDITOR, PartDescriptionEditor)

        'initialize presenter
        Presenter = New PresenterFormPartDescription(Me, ChildViews)
    End Sub

    Private Property Presenter() As PresenterFormPartDescription
        Get
            Return m_Presenter
        End Get
        Set(ByVal value As PresenterFormPartDescription)
            m_Presenter = value
        End Set
    End Property

    Private Property ChildViews() As IDictionary(Of String, IChildView)
        Get
            Return m_ChildViews
        End Get
        Set(ByVal value As IDictionary(Of String, IChildView))
            m_ChildViews = value
        End Set
    End Property

    Private Property PartDescriptionEditor() As FormPartDescriptionEditor
        Get
            Return m_FormPartDescriptionEditor
        End Get
        Set(ByVal value As FormPartDescriptionEditor)
            m_FormPartDescriptionEditor = value
        End Set
    End Property

#Region "IViewFormPartDescription"
    Public Function GetChildPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetChildPanel
        Return PanelChild
    End Function

    Public Function GetMainPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetMainPanel
        Return PanelMain
    End Function

    Public Function GetText() As String Implements Presenters.IChildView.GetText
        Return SCREEN_NAME
    End Function

    Public Function GetToolstrip() As System.Windows.Forms.ToolStrip Implements Presenters.IChildView.GetToolstrip
        Return ToolStripMain
    End Function

    Public Sub ShowChild() Implements Presenters.IChildView.ShowChild
        Show()
    End Sub

    Public Sub UpdateTopLevel(ByVal value As Boolean) Implements Presenters.IChildView.UpdateTopLevel
        SetTopLevel(value)
    End Sub

#Region "IViewFormPartDescription"
    Public Sub ClosePartDescriptionEditor() Implements Presenters.IViewFormPartDescription.ClosePartDescriptionEditor
        PartDescriptionEditor = Nothing
    End Sub

    Public Sub CreatePartDescriptionEditor(ByVal formDescription As FormDescription) Implements Presenters.IViewFormPartDescription.CreatePartDescriptionEditor
        If (formDescription IsNot Nothing) Then
            PartDescriptionEditor = New FormPartDescriptionEditor(formDescription)
        Else
            PartDescriptionEditor = New FormPartDescriptionEditor()
        End If
    End Sub

    Public Function GetPartDescriptionEditor() As Presenters.IViewFormPartDescriptionEditor Implements Presenters.IViewFormPartDescription.GetPartDescriptionEditor
        Return PartDescriptionEditor
    End Function
#End Region

#End Region
End Class