﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Presenters
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Windows.Forms

Public Class FormPartDescriptionEditor
    Implements IViewFormPartDescriptionEditor
    Private m_Presenter As PresenterFormPartDescriptionEditor
    Private Const SCREEN_NAME As String = "Form Description Editor"

    Public Sub New()
        Initialize()
    End Sub

    Public Sub New(ByVal formDescription As FormDescription)
        Initialize()
        Presenter.SetSelection(formDescription)
        UpdateView(formDescription)
    End Sub

    Private Sub Initialize()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        Presenter = New PresenterFormPartDescriptionEditor(Me)

        'populate combobox
        ComboBoxFormNames.Items.AddRange(Presenter.GetAllProjectFormNames().ToArray())

        'disable ToolStripButtonSave
        ToolStripButtonSaveEnabled(False)
    End Sub

    Private Sub ToolStripButtonSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonSave.Click
        Try
            Presenter.UpdateSelelection(TextBoxDescription.Text, ComboBoxFormNames.SelectedItem.ToString())
            Presenter.Save()
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try
    End Sub

    Private Sub ToolStripButtonCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonCancel.Click
        Presenter.Cancel()
    End Sub

    Private Sub ComboBoxFormNames_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ComboBoxFormNames.SelectedIndexChanged
        Presenter.RequestDescriptionForForm(ComboBoxFormNames.SelectedItem.ToString())
        ToolStripButtonSaveEnabled(RequiredFieldsAreFilledIn())
    End Sub

    Private Sub TextBoxDescription_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles TextBoxDescription.TextChanged
        ToolStripButtonSaveEnabled(RequiredFieldsAreFilledIn())
    End Sub

    Private Sub ToolStripButtonSaveEnabled(ByVal enabled As Boolean)
        ToolStripButtonSave.Enabled = enabled
    End Sub

    Private Function RequiredFieldsAreFilledIn() As Boolean
        Return (ComboBoxFormNames.SelectedIndex > -1) AndAlso (TextBoxDescription.Text.Trim() IsNot Nothing) AndAlso (TextBoxDescription.Text.Trim().Length > 0)
    End Function

    Private Property Presenter() As PresenterFormPartDescriptionEditor
        Get
            Return m_Presenter
        End Get
        Set(ByVal value As PresenterFormPartDescriptionEditor)
            m_Presenter = value
        End Set
    End Property

#Region "IViewFormPartDescription"
    Public Function GetChildPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetChildPanel
        Return Nothing
    End Function

    Public Function GetMainPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetMainPanel
        Return PanelMain
    End Function

    Public Function GetText() As String Implements Presenters.IChildView.GetText
        Return SCREEN_NAME
    End Function

    Public Function GetToolstrip() As System.Windows.Forms.ToolStrip Implements Presenters.IChildView.GetToolstrip
        Return Nothing
    End Function

    Public Sub ShowChild() Implements Presenters.IChildView.ShowChild
        Show()
        Presenter.SendChangedViewEvent()
    End Sub

    Public Sub UpdateTopLevel(ByVal value As Boolean) Implements Presenters.IChildView.UpdateTopLevel
        SetTopLevel(value)
    End Sub

#Region "IViewFormPartDescriptionEditor"
    Public Sub ResetView() Implements Presenters.IViewFormPartDescriptionEditor.ResetView
        TextBoxDescription.Clear()
        ComboBoxFormNames.Tag = 0
    End Sub

    Public Sub UpdateView(ByVal formDescription As Business.DataHolders.FormDescription) Implements Presenters.IViewFormPartDescriptionEditor.UpdateView
        TextBoxDescription.Text = formDescription.Description
        ComboBoxFormNames.Text = formDescription.FormName
        ComboBoxFormNames.Tag = formDescription.Id
    End Sub
#End Region

#End Region
End Class