﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Presenters
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Windows.Forms
Imports System.Drawing

Public Class FormPartDescriptionOverview
    Implements IViewFormPartDescriptionOverview
    Private m_Presenter As PresenterFormPartDescriptionOverview
    Private m_MouseLocation As Point
    Private m_CacheComparator As ExtendedListviewComparator
    Private Const SCREEN_NAME As String = "Form Description Overview"

    Public Sub New()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        Presenter = New PresenterFormPartDescriptionOverview(Me)
        CacheComparator = New ExtendedListviewComparator()

        'disable ToolStripButtonRemove
        ToolStripButtonRemoveEnabled(False)
    End Sub

    Private Sub ToolStripButtonAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonAdd.Click
        OpenPartDescriptionEditor(Nothing)
    End Sub

    Private Sub ToolStripButtonRemove_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonRemove.Click
        Try
            Dim listViewItem As ListViewItem = ListViewPartDescription.GetItemAt(MouseLocation.X, MouseLocation.Y)
            If (listViewItem IsNot Nothing) Then
                If (MessageBox.Show("Remove selected data?", "Question", MessageBoxButtons.YesNo) = Windows.Forms.DialogResult.Yes) Then
                    Presenter.Remove(Convert.ToInt32(listViewItem.Tag))
                    Presenter.RequestAllPartDescriptions()
                End If
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try
    End Sub

    Private Sub ToolStripButtonClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonClose.Click
        Presenter.Close()
    End Sub

    Private Sub ListViewPartDescription_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewPartDescription.DoubleClick
        Dim listViewItem As ListViewItem = ListViewPartDescription.GetItemAt(MouseLocation.X, MouseLocation.Y)
        If (listViewItem IsNot Nothing) Then
            OpenPartDescriptionEditor(Convert.ToInt32(listViewItem.Tag))
        End If
    End Sub

    Private Sub ListViewPartDescription_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles ListViewPartDescription.MouseDown
        MouseLocation = e.Location
        Dim hasSelection As Boolean
        If (ListViewPartDescription.GetItemAt(MouseLocation.X, MouseLocation.Y) IsNot Nothing) Then
            hasSelection = True
        End If
        ToolStripButtonRemoveEnabled(hasSelection)
    End Sub

    Private Sub SortListViewPartDescription(ByVal sender As System.Object, ByVal e As CustomControls.SortVirtualExtendedListviewEventArgs) Handles ListViewPartDescription.SortVirtualExtendedListview
        CacheComparator.Order = e.Order
        CacheComparator.ColumnIndex = e.ColumnIndex
        ListViewPartDescription.VirtualModeCache.Sort(CacheComparator)
        ListViewPartDescription.Invalidate()
    End Sub

    Private Sub OpenPartDescriptionEditor(ByVal id As Integer)
        ToolStripButtonCloseEnabled(False)
        Presenter.ShowPartDescriptionEditor(id)
    End Sub

    Private Sub ToolStripButtonRemoveEnabled(ByVal enable As Boolean)
        ToolStripButtonRemove.Enabled = enable
    End Sub

    Private Sub ToolStripButtonCloseEnabled(ByVal enable As Boolean)
        ToolStripButtonClose.Enabled = enable
    End Sub

    Private Property Presenter() As PresenterFormPartDescriptionOverview
        Get
            Return m_Presenter
        End Get
        Set(ByVal value As PresenterFormPartDescriptionOverview)
            m_Presenter = value
        End Set
    End Property

    Private Property MouseLocation() As Point
        Get
            Return m_MouseLocation
        End Get
        Set(ByVal value As Point)
            m_MouseLocation = value
        End Set
    End Property

    Private Property CacheComparator() As ExtendedListviewComparator
        Get
            Return m_CacheComparator
        End Get
        Set(ByVal value As ExtendedListviewComparator)
            m_CacheComparator = value
        End Set
    End Property

#Region "IViewFormPartDescriptionOverview"
    Public Function GetChildPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetChildPanel
        Return Nothing
    End Function

    Public Function GetMainPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetMainPanel
        Return PanelMain
    End Function

    Public Function GetText() As String Implements Presenters.IChildView.GetText
        Return SCREEN_NAME
    End Function

    Public Function GetToolstrip() As System.Windows.Forms.ToolStrip Implements Presenters.IChildView.GetToolstrip
        Return ToolStripMain
    End Function

    Public Sub ShowChild() Implements Presenters.IChildView.ShowChild
        Try
            'populate listview
            Presenter.RequestAllPartDescriptions()
            Show()
            Presenter.SendChangedViewEvent()
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try
    End Sub

    Public Sub UpdateTopLevel(ByVal value As Boolean) Implements Presenters.IChildView.UpdateTopLevel
        SetTopLevel(value)
    End Sub

    Public Sub UpdateView() Implements Presenters.IViewFormPartDescriptionOverview.UpdateView
        ToolStripButtonRemoveEnabled(False)
        ToolStripButtonCloseEnabled(True)
        ListViewPartDescription.Sort()
        ListViewPartDescription.Invalidate()
    End Sub

    Public Sub ClearExtendedListViewPartDescription() Implements Presenters.IViewFormPartDescriptionOverview.ClearExtendedListViewPartDescription
        ListViewPartDescription.VirtualModeCache.Clear()
    End Sub

    Public Sub AddToExtendedListViewPartDescription(ByVal fourStateListViewItem As CustomControls.FourStateListViewItem) Implements Presenters.IViewFormPartDescriptionOverview.AddToExtendedListViewPartDescription
        ListViewPartDescription.VirtualModeCache.Add(fourStateListViewItem)
    End Sub
#End Region

End Class