﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Presenters
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Windows.Forms
Imports CustomControls

Public Class FormProjectScreen1
    Implements IViewProjectScreen1, IViewDescription
    Private m_Presenter As PresenterFormProjectScreen1
    Private Const SCREEN_NAME As String = "Screen 1"

    Public Sub New(ByVal isAuditor As Boolean)
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.

        'add ListViewComboBoxCategory listview handler
        AddHandler ListViewComboBoxCategory.ListView.KeyPress, AddressOf ListViewComboBoxCategory_KeyPress

        'add ListViewComboBoxCategory listview handler
        AddHandler ListViewComboBoxSubCategory.ListView.KeyPress, AddressOf ListViewComboBoxSubCategory_KeyPress

        'add ListViewComboBoxSponsor listview handler
        AddHandler ListViewComboBoxSponsor.ListView.KeyPress, AddressOf ListViewComboBoxSponsor_KeyPress

        'add ListViewComboBoxPusher listview handler
        AddHandler ListViewComboBoxPusher.ListView.KeyPress, AddressOf ListViewComboBoxPusher_KeyPress

        'add ListViewComboBoxCoordinator listview handler
        AddHandler ListViewComboBoxCoordinator.ListView.KeyPress, AddressOf ListViewComboBoxCoordinator_KeyPress

        'add ListViewComboBoxStatus listview handler
        AddHandler ListViewComboBoxRevisionStatus.ListView.KeyPress, AddressOf ListViewComboBoxRevisionStatus_KeyPress

        'add ListViewComboBoxAuditor listview handler
        AddHandler ListViewComboBoxAuditor.ListView.KeyPress, AddressOf ListViewComboBoxAuditor_KeyPress

        LabelCoordinator.Text = Nothing
        LabelCreator.Text = Nothing
        LabelPusher.Text = Nothing
        LabelSponsor.Text = Nothing
        LabelAuditor.Text = Nothing

        'initialize presenter
        Presenter = New PresenterFormProjectScreen1(Me, isAuditor)
    End Sub

    Private Sub TextBoxProjectName_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles TextBoxProjectName.TextChanged
        Try
            Presenter.UpdateProjectName(TextBoxProjectName.Text)
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub RichTextBoxDescription_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles RichTextBoxDescription.TextChanged
        Try
            Presenter.UpdateProjectDescription(RichTextBoxDescription.Text)
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub ListViewComboBoxAuditor_DropDown(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxAuditor.DropDown
        Try
            ListViewComboBoxAuditor.ListView.BeginUpdate()
            Presenter.RequestAllEmployeesForAuditor()
            ListViewComboBoxAuditor.ListView.EndUpdate()
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub ListViewComboBoxAuditor_SelectionChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxAuditor.SelectionChanged
        Try
            Dim listViewItem As ListViewItem = ListViewComboBoxAuditor.SelectedListViewitem
            LabelAuditor.Text = listViewItem.SubItems(1).Text
            Presenter.UpdateAuditor(ListViewComboBoxAuditor.Text)
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub ListViewComboBoxAuditor_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs)
        ListViewComboBoxKeyPressed(ListViewComboBoxAuditor)
    End Sub

    Private Sub ListViewComboBoxCategory_DropDown(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxCategory.DropDown
        Try
            ListViewComboBoxCategory.ListView.BeginUpdate()
            Presenter.RequestAllCategories()
            ListViewComboBoxCategory.ListView.EndUpdate()
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub ListViewComboBoxCategory_SelectionChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxCategory.SelectionChanged
        Try
            Dim listViewItem As ListViewItem = ListViewComboBoxCategory.SelectedListViewitem
            Dim category As Category = New Category()
            category.Id = Convert.ToInt32(listViewItem.Tag)
            category.Description = listViewItem.Text
            Presenter.UpdateCategory(category)
            Presenter.RequestSubCategoryForSelectedCategory(category.Id)
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub ListViewComboBoxCategory_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs)
        ListViewComboBoxKeyPressed(ListViewComboBoxCategory)
    End Sub

    Private Sub ListViewComboBoxSubCategory_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs)
        ListViewComboBoxKeyPressed(ListViewComboBoxSubCategory)
    End Sub

    Private Sub ListViewComboBoxSubCategory_SelectionChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxSubCategory.SelectionChanged
        Try
            Dim listViewItem As ListViewItem = ListViewComboBoxSubCategory.SelectedListViewitem
            Dim subCategory As SubCategory = New SubCategory()
            subCategory.Id = Convert.ToInt32(listViewItem.Tag)
            subCategory.Description = listViewItem.Text
            Presenter.UpdateSubCategory(subCategory)
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub ListViewComboBoxSubCategory_Click(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles ListViewComboBoxSubCategory.MouseClick
        If (e.Button = Windows.Forms.MouseButtons.Left AndAlso ListViewComboBoxSubCategory.SelectedListViewitem Is Nothing) Then
            Try
                Presenter.RequestSubCategoryForSelectedCategory(ListViewComboBoxCategory.Text)
            Catch ex As Exception
                ShowErrorMessage(ex.Message)
            End Try
        End If
    End Sub

    Private Sub ListViewComboBoxSponsor_DropDown(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxSponsor.DropDown
        Try
            ListViewComboBoxSponsor.ListView.BeginUpdate()
            Presenter.RequestAllEmployeesForSponsor()
            ListViewComboBoxSponsor.ListView.EndUpdate()
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub ListViewComboBoxSponsor_SelectionChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxSponsor.SelectionChanged
        Try
            Dim listViewItem As ListViewItem = ListViewComboBoxSponsor.SelectedListViewitem
            LabelSponsor.Text = listViewItem.SubItems(1).Text
            Presenter.UpdateSponsor(ListViewComboBoxSponsor.Text, LabelSponsor.Text)
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub ListViewComboBoxSponsor_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs)
        ListViewComboBoxKeyPressed(ListViewComboBoxSponsor)
    End Sub

    Private Sub ListViewComboBoxPusher_DropDown(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxPusher.DropDown
        Try
            ListViewComboBoxPusher.ListView.BeginUpdate()
            Presenter.RequestAllEmployeesForPusher()
            ListViewComboBoxPusher.ListView.EndUpdate()
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub ListViewComboBoxPusher_SelectionChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxPusher.SelectionChanged
        Try
            Dim listViewItem As ListViewItem = ListViewComboBoxPusher.SelectedListViewitem
            LabelPusher.Text = listViewItem.SubItems(1).Text
            Presenter.UpdatePusher(ListViewComboBoxPusher.Text, LabelPusher.Text)
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub ListViewComboBoxPusher_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs)
        ListViewComboBoxKeyPressed(ListViewComboBoxPusher)
    End Sub

    Private Sub ListViewComboBoxCoordinator_DropDown(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxCoordinator.DropDown
        Try
            ListViewComboBoxCoordinator.ListView.BeginUpdate()
            Presenter.RequestAllEmployeesForCoordinator()
            ListViewComboBoxCoordinator.ListView.EndUpdate()
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub ListViewComboBoxCoordinator_SelectionChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxCoordinator.SelectionChanged
        Try
            Dim listViewItem As ListViewItem = ListViewComboBoxCoordinator.SelectedListViewitem
            LabelCoordinator.Text = listViewItem.SubItems(1).Text
            Presenter.UpdateCoordinator(ListViewComboBoxCoordinator.Text, LabelCoordinator.Text)
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub ListViewComboBoxCoordinator_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs)
        ListViewComboBoxKeyPressed(ListViewComboBoxCoordinator)
    End Sub

    Private Sub ListViewComboBoxRevisionStatus_DropDown(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxRevisionStatus.DropDown
        Try
            ListViewComboBoxRevisionStatus.ListView.BeginUpdate()
            Presenter.RequestAllRevisionStatuses()
            ListViewComboBoxRevisionStatus.ListView.EndUpdate()
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub ListViewComboBoxRevisionStatus_SelectionChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxRevisionStatus.SelectionChanged
        Try
            Dim listViewItem As ListViewItem = ListViewComboBoxRevisionStatus.SelectedListViewitem
            Presenter.UpdateCurrentRevisionStatus(listViewItem.Text, Convert.ToInt32(listViewItem.Tag))
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub ListViewComboBoxRevisionStatus_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs)
        ListViewComboBoxKeyPressed(ListViewComboBoxRevisionStatus)
    End Sub

    Private Sub TextBoxBudget_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles TextBoxBudget.TextChanged
        Try
            Presenter.UpdateBudget(TextBoxBudget.Text)
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub ListViewComboBoxKeyPressed(ByVal listviewCombobox As ListViewComboBox)
        Try
            Dim listViewItem As ListViewItem = listviewCombobox.ListView.FindItemWithText(listviewCombobox.GetSearchString(), True, 0)
            If (listViewItem IsNot Nothing) Then
                listviewCombobox.ListView.Items(listViewItem.Index).Selected = True
                listviewCombobox.ListView.Items(listViewItem.Index).EnsureVisible()
            End If
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub Editable(ByVal value As Boolean)
        TextBoxProjectName.ReadOnly = Not value
        RichTextBoxDescription.ReadOnly = Not value
        ListViewComboBoxCategory.ReadOnly = Not value
        ListViewComboBoxSubCategory.ReadOnly = Not value
        ListViewComboBoxAuditor.ReadOnly = Not value
        ListViewComboBoxSponsor.ReadOnly = Not value
        ListViewComboBoxPusher.ReadOnly = Not value
        ListViewComboBoxCoordinator.ReadOnly = Not value
        ListViewComboBoxRevisionStatus.ReadOnly = Not value
        TextBoxBudget.ReadOnly = Not value
    End Sub

    Private Sub ShowErrorMessage(ByVal message As String)
        MessageBox.Show(message)
    End Sub

    Private Property Presenter() As PresenterFormProjectScreen1
        Get
            Return m_Presenter
        End Get
        Set(ByVal value As PresenterFormProjectScreen1)
            m_Presenter = value
        End Set
    End Property

#Region "IViewProjectScreen1"
    Public Function GetMainPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetMainPanel
        Return PanelFormProjectScreen1
    End Function

    Public Function GetToolstrip() As System.Windows.Forms.ToolStrip Implements Presenters.IChildView.GetToolstrip
        Return Nothing
    End Function

    Public Sub UpdateTopLevel(ByVal value As Boolean) Implements Presenters.IChildView.UpdateTopLevel
        SetTopLevel(value)
    End Sub

    Public Sub ShowChild() Implements Presenters.IChildView.ShowChild
        Try
            Show()
            Presenter.SendChangedViewEvent()
            Presenter.RequestSelection()
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Public Function GetChildPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetChildPanel
        Return Nothing
    End Function

    Public Function GetText() As String Implements Presenters.IChildView.GetText
        Return SCREEN_NAME
    End Function

    Public Sub UpdateCatagory() Implements Presenters.IViewProjectScreen1.UpdateCatagory
        Try
            UpdateListViewComboBox(ListViewComboBoxCategory)
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Public Sub UpdateSubCatagory() Implements Presenters.IViewProjectScreen1.UpdateSubCatagory
        Try
            UpdateListViewComboBox(ListViewComboBoxSubCategory)
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Public Sub UpdateCreator(ByVal creator As Business.DataHolders.Creator) Implements Presenters.IViewProjectScreen1.UpdateCreator
        Try
            LabelCreatorID.Text = creator.EmployeeId
            LabelCreator.Text = creator.ToString()
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Public Sub UpdateAuditor() Implements Presenters.IViewProjectScreen1.UpdateAuditor
        Try
            UpdateListViewComboBox(ListViewComboBoxAuditor)
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Public Sub UpdateSponsor() Implements Presenters.IViewProjectScreen1.UpdateSponsor
        Try
            UpdateListViewComboBox(ListViewComboBoxSponsor)
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Public Sub UpdatePusher() Implements Presenters.IViewProjectScreen1.UpdatePusher
        Try
            UpdateListViewComboBox(ListViewComboBoxPusher)
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Public Sub UpdateCoordinator() Implements Presenters.IViewProjectScreen1.UpdateCoordinator
        Try
            UpdateListViewComboBox(ListViewComboBoxCoordinator)
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Public Sub UpdateRevisionStatus() Implements Presenters.IViewProjectScreen1.UpdateRevisionStatus
        Try
            UpdateListViewComboBox(ListViewComboBoxRevisionStatus)
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub UpdateListViewComboBox(ByVal listViewComboBox As ListViewComboBox)
        Try
            listViewComboBox.Invalidate()
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Public Sub UpdateView(ByVal project As Business.DataHolders.ProjectProposalRevision) Implements Presenters.IViewProjectScreen1.UpdateView
        Try
            TextBoxProjectName.Text = project.Project.Name
            RichTextBoxDescription.Text = project.ProjectDescription
            If (project.SubCategory IsNot Nothing) Then
                ListViewComboBoxCategory.Text = project.SubCategory.Category.Description
                ListViewComboBoxSubCategory.Text = project.SubCategory.Description
            End If
            LabelCreatorID.Text = project.Creator.EmployeeId
            LabelCreator.Text = project.Creator.ToString()
            ListViewComboBoxAuditor.Text = project.Auditor.EmployeeId
            LabelAuditor.Text = project.Auditor.ToString()
            ListViewComboBoxSponsor.Text = project.SponsorId
            LabelSponsor.Text = project.SponsorName
            ListViewComboBoxPusher.Text = project.PusherId
            LabelPusher.Text = project.PusherName
            ListViewComboBoxCoordinator.Text = project.CoordinatorId
            LabelCoordinator.Text = project.CoordinatorName
            TextBoxBudget.Text = project.Budget
            ListViewComboBoxRevisionStatus.Text = project.RevisionInformation.RevisionStatus.Name
            ListViewComboBoxRevisionStatus.Tag = project.RevisionInformation.RevisionStatus.Id
            LabelRevisionNumber.Text = project.RevisionInformation.RevisionNumber
            Editable(Presenter.IsEditable)
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Public Function GetName() As String Implements Presenters.IViewFormProjectEditorChild.GetName
        Return Name
    End Function

    Public Sub ClearListViewComboBoxAuditor() Implements Presenters.IViewProjectScreen1.ClearListViewComboBoxAuditor
        ListViewComboBoxAuditor.ListView.VirtualModeCache.Clear()
    End Sub

    Public Sub AddToListViewComboBoxAuditor(ByVal fourStateListViewItems As IList(Of CustomControls.FourStateListViewItem)) Implements Presenters.IViewProjectScreen1.AddToListViewComboBoxAuditor
        ListViewComboBoxAuditor.ListView.VirtualModeCache.AddRange(fourStateListViewItems)
    End Sub

    Public Sub ClearListViewComboBoxCategory() Implements Presenters.IViewProjectScreen1.ClearListViewComboBoxCategory
        ListViewComboBoxCategory.ListView.VirtualModeCache.Clear()
    End Sub

    Public Sub AddToListViewComboBoxCategory(ByVal fourStateListViewItem As CustomControls.FourStateListViewItem) Implements Presenters.IViewProjectScreen1.AddToListViewComboBoxCategory
        ListViewComboBoxCategory.ListView.VirtualModeCache.Add(fourStateListViewItem)
    End Sub

    Public Sub ClearListViewComboBoxSubCategory() Implements Presenters.IViewProjectScreen1.ClearListViewComboBoxSubCategory
        ListViewComboBoxSubCategory.ListView.VirtualModeCache.Clear()
    End Sub

    Public Sub AddToListViewComboBoxSubCategory(ByVal fourStateListViewItems As IList(Of CustomControls.FourStateListViewItem)) Implements Presenters.IViewProjectScreen1.AddToListViewComboBoxSubCategory
        ListViewComboBoxSubCategory.ListView.VirtualModeCache.AddRange(fourStateListViewItems)
    End Sub

    Public Sub ClearListViewComboBoxCoordinator() Implements Presenters.IViewProjectScreen1.ClearListViewComboBoxCoordinator
        ListViewComboBoxCoordinator.ListView.VirtualModeCache.Clear()
    End Sub

    Public Sub AddToListViewComboBoxCoordinator(ByVal fourStateListViewItems As IList(Of CustomControls.FourStateListViewItem)) Implements Presenters.IViewProjectScreen1.AddToListViewComboBoxCoordinator
        ListViewComboBoxCoordinator.ListView.VirtualModeCache.AddRange(fourStateListViewItems)
    End Sub

    Public Sub ClearListViewComboBoxPusher() Implements Presenters.IViewProjectScreen1.ClearListViewComboBoxPusher
        ListViewComboBoxPusher.ListView.VirtualModeCache.Clear()
    End Sub

    Public Sub AddToListViewComboBoxPusher(ByVal fourStateListViewItems As IList(Of CustomControls.FourStateListViewItem)) Implements Presenters.IViewProjectScreen1.AddToListViewComboBoxPusher
        ListViewComboBoxPusher.ListView.VirtualModeCache.AddRange(fourStateListViewItems)
    End Sub

    Public Sub ClearListViewComboBoxRevisionStatus() Implements Presenters.IViewProjectScreen1.ClearListViewComboBoxRevisionStatus
        ListViewComboBoxRevisionStatus.ListView.VirtualModeCache.Clear()
    End Sub

    Public Sub AddToListViewComboBoxRevisionStatus(ByVal fourStateListViewItem As CustomControls.FourStateListViewItem) Implements Presenters.IViewProjectScreen1.AddToListViewComboBoxRevisionStatus
        ListViewComboBoxRevisionStatus.ListView.VirtualModeCache.Add(fourStateListViewItem)
    End Sub

    Public Sub ClearListViewComboBoxSponsor() Implements Presenters.IViewProjectScreen1.ClearListViewComboBoxSponsor
        ListViewComboBoxSponsor.ListView.VirtualModeCache.Clear()
    End Sub

    Public Sub AddToListViewComboBoxSponsor(ByVal fourStateListViewItems As IList(Of CustomControls.FourStateListViewItem)) Implements Presenters.IViewProjectScreen1.AddToListViewComboBoxSponsor
        ListViewComboBoxSponsor.ListView.VirtualModeCache.AddRange(fourStateListViewItems)
    End Sub
#End Region



End Class