﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Presenters
Imports System.Windows.Forms
Imports CustomControls

Public Class FormProjectScreenAuditor
    Implements IViewProjectScreenAuditor, IViewDescription
    Private m_Presenter As PresenterFormProjectScreenAuditor
    Private Const SCREEN_NAME As String = "Screen Auditor"

    Public Sub New()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.

        'initialize presenter
        Presenter = New PresenterFormProjectScreenAuditor(Me)
    End Sub

    Private Sub TextBoxAuditorDescription_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RichTextBoxAuditorDescription.TextChanged
        Try
            Presenter.UpdateAuditorDescription(RichTextBoxAuditorDescription.Text)
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub ListViewComboBoxAuditorRevisionStatus_DropDown(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxAuditorRevisionStatus.DropDown
        Try
            Presenter.RequestAllRevisionStatuses()
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub ListViewComboBoxAuditorRevisionStatus_SelectionChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxAuditorRevisionStatus.SelectionChanged
        Try
            Dim listViewItem As ListViewItem = ListViewComboBoxAuditorRevisionStatus.SelectedListViewitem
            Presenter.UpdateRevisionStatus(listViewItem.Text, Convert.ToInt32(listViewItem.Tag))
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub UpdateListViewComboBox(ByVal listViewComboBox As ListViewComboBox)
        Try
            listViewComboBox.Invalidate()
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub Editable(ByVal value As Boolean)
        RichTextBoxAuditorDescription.ReadOnly = Not value
        ListViewComboBoxAuditorRevisionStatus.ReadOnly = Not value
    End Sub

    Private Sub ShowErrorMessage(ByVal message As String)
        MessageBox.Show(message)
    End Sub

    Private Property Presenter() As PresenterFormProjectScreenAuditor
        Get
            Return m_Presenter
        End Get
        Set(ByVal value As PresenterFormProjectScreenAuditor)
            m_Presenter = value
        End Set
    End Property

#Region "IViewProjectScreenAuditor"
    Public Function GetChildPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetChildPanel
        Return Nothing
    End Function

    Public Function GetText() As String Implements Presenters.IChildView.GetText
        Return SCREEN_NAME
    End Function

    Public Sub ShowChild() Implements Presenters.IChildView.ShowChild
        Try
            Show()
            Presenter.SendChangedViewEvent()
            Presenter.RequestSelection()
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Public Sub UpdateTopLevel(ByVal value As Boolean) Implements Presenters.IChildView.UpdateTopLevel
        SetTopLevel(value)
    End Sub

    Public Function GetMainPanel() As System.Windows.Forms.Panel Implements Presenters.IForm.GetMainPanel
        Return PanelMain
    End Function

    Public Function GetToolstrip() As System.Windows.Forms.ToolStrip Implements Presenters.IForm.GetToolstrip
        Return Nothing
    End Function

    Public Function GetName() As String Implements Presenters.IViewFormProjectEditorChild.GetName
        Return Name
    End Function

    Public Sub UpdateRevisionStatus() Implements Presenters.IViewProjectScreenAuditor.UpdateRevisionStatus
        Try
            UpdateListViewComboBox(ListViewComboBoxAuditorRevisionStatus)
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Public Sub UpdateView(ByVal project As Business.DataHolders.ProjectProposalRevision) Implements Presenters.IViewProjectScreenAuditor.UpdateView
        ListViewComboBoxAuditorRevisionStatus.Text = project.RevisionInformation.RevisionStatus.Name
        ListViewComboBoxAuditorRevisionStatus.Tag = project.RevisionInformation.RevisionStatus.Id
        RichTextBoxAuditorDescription.Text = project.RevisionInformation.Description
        Editable(Presenter.IsEditable)
    End Sub

    Public Sub AddToListViewComboBoxAuditorRevisionStatus(ByVal fourStateListViewItems As System.Collections.Generic.IList(Of CustomControls.FourStateListViewItem)) Implements Presenters.IViewProjectScreenAuditor.AddToListViewComboBoxAuditorRevisionStatus
        ListViewComboBoxAuditorRevisionStatus.ListView.VirtualModeCache.AddRange(fourStateListViewItems)
    End Sub

    Public Sub ClearListViewComboBoxAuditorRevisionStatus() Implements Presenters.IViewProjectScreenAuditor.ClearListViewComboBoxAuditorRevisionStatus
        ListViewComboBoxAuditorRevisionStatus.ListView.VirtualModeCache.Clear()
    End Sub
#End Region

End Class