﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Presenters
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Windows.Forms

Public Class FormRoleEditor
    Implements IViewFormRoleEditor
    Private m_Presenter As PresenterFormRoleEditor
    Private Const SCREEN_NAME As String = "Role Editor"

    Public Sub New()
        Initialize()
    End Sub

    Public Sub New(ByVal role As Role)
        Initialize()
        Presenter.SetSelection(role)
        UpdateView(role)
    End Sub

    Private Sub Initialize()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        Presenter = New PresenterFormRoleEditor(Me)

        'disable ToolStripButtonSave
        ToolStripButtonSaveEnabled(False)
    End Sub

    Private Sub ToolStripButtonSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonSave.Click
        Try
            Presenter.UpdateSelelection(TextBoxRole.Text)
            Presenter.Save()
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try
    End Sub

    Private Sub ToolStripButtonCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonCancel.Click
        Presenter.Cancel()
    End Sub

    Private Sub TextBoxRole_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles TextBoxRole.TextChanged
        ToolStripButtonSaveEnabled(RequiredFieldsAreFilledIn())
    End Sub

    Private Sub ToolStripButtonSaveEnabled(ByVal enabled As Boolean)
        ToolStripButtonSave.Enabled = enabled
    End Sub

    Private Function RequiredFieldsAreFilledIn() As Boolean
        Return (TextBoxRole.Text.Trim() IsNot Nothing) AndAlso (TextBoxRole.Text.Trim().Length > 0)
    End Function

    Private Property Presenter() As PresenterFormRoleEditor
        Get
            Return m_Presenter
        End Get
        Set(ByVal value As PresenterFormRoleEditor)
            m_Presenter = value
        End Set
    End Property

#Region "IViewFormCategoryEditor"
    Public Function GetChildPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetChildPanel
        Return Nothing
    End Function

    Public Function GetText() As String Implements Presenters.IChildView.GetText
        Return SCREEN_NAME
    End Function

    Public Sub ShowChild() Implements Presenters.IChildView.ShowChild
        Show()
        Presenter.SendChangedViewEvent()
    End Sub

    Public Sub UpdateTopLevel(ByVal value As Boolean) Implements Presenters.IChildView.UpdateTopLevel
        SetTopLevel(value)
    End Sub

    Public Function GetMainPanel() As System.Windows.Forms.Panel Implements Presenters.IForm.GetMainPanel
        Return PanelMain
    End Function

    Public Function GetToolstrip() As System.Windows.Forms.ToolStrip Implements Presenters.IForm.GetToolstrip
        Return Nothing
    End Function

    Public Sub ResetView() Implements Presenters.IViewFormRoleEditor.ResetView
        TextBoxRole.Clear()
        TextBoxRole.Tag = 0
    End Sub

    Public Sub UpdateView(ByVal role As Business.DataHolders.Role) Implements Presenters.IViewFormRoleEditor.UpdateView
        TextBoxRole.Text = role.Name
        TextBoxRole.Tag = role.Id
    End Sub
#End Region

End Class