﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Presenters
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports CustomControls
Imports System.Windows.Forms
Imports System.Drawing

Public Class FormRoleOverview
    Implements IViewFormRoleOverview
    Private m_Presenter As PresenterFormRoleOverview
    Private m_MouseLocationRole As Point
    Private m_CacheComparator As ExtendedListviewComparator
    Private Const SCREEN_NAME As String = "Role Overview"

    Public Sub New()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        Presenter = New PresenterFormRoleOverview(Me)
        CacheComparator = New ExtendedListviewComparator()

        'disable ToolStripButtonRemoveEnabled 
        ToolStripButtonRemoveEnabled(False)
    End Sub

    Private Sub ToolStripButtonAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonAdd.Click
        OpenRoleEditor(Nothing)
    End Sub

    Private Sub ToolStripButtonRemove_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonRemove.Click
        Try
            Dim listViewItem As ListViewItem = ExtendedListViewRole.GetItemAt(MouseLocationRole.X, MouseLocationRole.Y)
            If (listViewItem IsNot Nothing) Then
                If (MessageBox.Show("Remove selected data?", "Question", MessageBoxButtons.YesNo) = Windows.Forms.DialogResult.Yes) Then
                    Presenter.RemoveRole(Convert.ToInt32(listViewItem.Tag))
                    Presenter.SelectedRoleId = -1
                    Presenter.RequestAllRoles()
                End If
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try
    End Sub

    Private Sub ToolStripButtonClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonClose.Click
        Presenter.Close()
    End Sub

    Private Sub ExtendedListViewRole_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles ExtendedListViewRole.DoubleClick
        Dim listViewItem As ListViewItem = ExtendedListViewRole.GetItemAt(MouseLocationRole.X, MouseLocationRole.Y)
        If (listViewItem IsNot Nothing) Then
            OpenRoleEditor(listViewItem.Text)
        End If
    End Sub

    Private Sub ExtendedListViewRole_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles ExtendedListViewRole.MouseDown
        MouseLocationRole = e.Location
        Dim hasSelection As Boolean
        Dim listViewItem As ListViewItem = ExtendedListViewRole.GetItemAt(MouseLocationRole.X, MouseLocationRole.Y)
        If (listViewItem IsNot Nothing) Then
            hasSelection = True
            Presenter.SelectedRoleId = Convert.ToInt32(listViewItem.Tag)
        End If
        ToolStripButtonRemoveEnabled(hasSelection)
    End Sub

    'Private Sub ExtendedListViewRole_RetrieveVirtualItem(ByVal sender As Object, ByVal e As System.Windows.Forms.RetrieveVirtualItemEventArgs) Handles ExtendedListViewRole.RetrieveVirtualItem
    '    RetrieveVirtualItem(Presenter.ListViewRoleItemCache, e)
    'End Sub

    'Private Sub RetrieveVirtualItem(ByVal listViewCache As List(Of ListViewItem), ByVal e As System.Windows.Forms.RetrieveVirtualItemEventArgs)
    '    If (listViewCache IsNot Nothing AndAlso listViewCache.Count > 0 AndAlso listViewCache.Count > e.ItemIndex) Then
    '        e.Item = listViewCache(e.ItemIndex)
    '    Else
    '        'if there isn't any data
    '        'add empty listviewitem to listview
    '        Dim listViewItem As ListViewItem = New ListViewItem()
    '        e.Item = listViewItem
    '    End If
    'End Sub

    Private Sub SortExtendedListViewRole(ByVal sender As System.Object, ByVal e As CustomControls.SortVirtualExtendedListviewEventArgs) Handles ExtendedListViewRole.SortVirtualExtendedListview
        CacheComparator.Order = e.Order
        CacheComparator.ColumnIndex = e.ColumnIndex
        ExtendedListViewRole.VirtualModeCache.Sort(CacheComparator)
        'Presenter.ListViewRoleItemCache.Sort(CacheComparator)
        ExtendedListViewRole.Invalidate()
    End Sub

    Private Sub OpenRoleEditor(ByVal name As String)
        ToolStripButtonCloseEnabled(False)
        Presenter.ShowRoleEditor(name)
    End Sub

    Private Sub ToolStripButtonAddEnabled(ByVal enable As Boolean)
        ToolStripButtonAdd.Enabled = enable
    End Sub

    Private Sub ToolStripButtonRemoveEnabled(ByVal enable As Boolean)
        ToolStripButtonRemove.Enabled = enable
    End Sub

    Private Sub ToolStripButtonCloseEnabled(ByVal enable As Boolean)
        ToolStripButtonClose.Enabled = enable
    End Sub

    Private Property Presenter() As PresenterFormRoleOverview
        Get
            Return m_Presenter
        End Get
        Set(ByVal value As PresenterFormRoleOverview)
            m_Presenter = value
        End Set
    End Property

    Private Property MouseLocationRole() As Point
        Get
            Return m_MouseLocationRole
        End Get
        Set(ByVal value As Point)
            m_MouseLocationRole = value
        End Set
    End Property

    Private Property CacheComparator() As ExtendedListviewComparator
        Get
            Return m_CacheComparator
        End Get
        Set(ByVal value As ExtendedListviewComparator)
            m_CacheComparator = value
        End Set
    End Property

#Region "IViewFormCategories"
    Public Function GetChildPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetChildPanel
        Return Nothing
    End Function

    Public Function GetText() As String Implements Presenters.IChildView.GetText
        Return SCREEN_NAME
    End Function

    Public Sub ShowChild() Implements Presenters.IChildView.ShowChild
        Try
            'populate listview
            Presenter.RequestAllRoles()
            Show()
            Presenter.SendChangedViewEvent()
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try
    End Sub

    Public Sub UpdateTopLevel(ByVal value As Boolean) Implements Presenters.IChildView.UpdateTopLevel
        SetTopLevel(value)
    End Sub

    Public Function GetMainPanel() As System.Windows.Forms.Panel Implements Presenters.IForm.GetMainPanel
        Return PanelMain
    End Function

    Public Function GetToolstrip() As System.Windows.Forms.ToolStrip Implements Presenters.IForm.GetToolstrip
        Return ToolStripMain
    End Function

    Public Sub UpdateRole() Implements Presenters.IViewFormRoleOverview.UpdateRole
        UpdateView(ExtendedListViewRole)
    End Sub

    Private Sub UpdateView(ByVal listView As ExtendedListView)
        ToolStripButtonRemoveEnabled(False)
        ToolStripButtonCloseEnabled(True)
        'listView.VirtualListSize = count
        listView.Sort()
        listView.Invalidate()
    End Sub

    Public Sub ClearExtendedListViewRole() Implements Presenters.IViewFormRoleOverview.ClearExtendedListViewRole
        ExtendedListViewRole.VirtualModeCache.Clear()
    End Sub

    Public Sub AddToExtendedListViewRole(ByVal fourStateListViewItem As CustomControls.FourStateListViewItem) Implements Presenters.IViewFormRoleOverview.AddToExtendedListViewRole
        ExtendedListViewRole.VirtualModeCache.Add(fourStateListViewItem)
    End Sub

#End Region

End Class