﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Presenters
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Windows.Forms
Imports CustomControls

Public Class FormWorkPackageEditor
    Implements IViewFormWorkPackageEditor
    Private m_Presenter As PresenterFormWorkPackageEditor
    Private Const SCREEN_NAME As String = "Workpackage Editor"

    Public Sub New()
        Initialize()
    End Sub

    Public Sub New(ByVal workPackage As WorkPackage)
        Initialize()
        Presenter.SetSelection(workPackage)
        UpdateView(workPackage)
    End Sub

    Private Sub Initialize()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.

        'add ListViewComboBoxCoordinator listview handlers
        AddHandler ListViewComboBoxResponsible.ListView.KeyPress, AddressOf ListViewComboBoxResponsible_KeyPress

        'clear text
        LabelResponsible.Text = Nothing

        Presenter = New PresenterFormWorkPackageEditor(Me)

        'disable ToolStripButtonApply
        ToolStripButtonApplyEnabled(False)
    End Sub

    Private Sub ToolStripButtonApply_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonApply.Click
        Presenter.UpdateWorkPackage()
        Presenter.Apply()
    End Sub

    Private Sub ToolStripButtonCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonCancel.Click
        Presenter.Cancel()
    End Sub

    Private Sub TextBoxWorkPackage_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles TextBoxWorkPackage.TextChanged
        Presenter.UpdateName(TextBoxWorkPackage.Text)
        ToolStripButtonApplyEnabled(RequiredFieldsAreFilledIn())
    End Sub

    Private Sub TextBoxEstimatedTime_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles TextBoxEstimatedTime.TextChanged
        Presenter.UpdateEstimatedTime(TextBoxEstimatedTime.Text)
        ToolStripButtonApplyEnabled(RequiredFieldsAreFilledIn())
    End Sub

    Private Sub DateTimePickerDueDate_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DateTimePickerDueDate.ValueChanged
        Presenter.UpdateDueDate(DateTimePickerDueDate.Value)
        ToolStripButtonApplyEnabled(RequiredFieldsAreFilledIn())
    End Sub

    Private Sub ListViewComboBoxResponsible_DropDown(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxResponsible.DropDown
        ListViewComboBoxResponsible.ListView.BeginUpdate()
        Presenter.RequestAllTeamMembers()
        ListViewComboBoxResponsible.ListView.EndUpdate()
    End Sub

    Private Sub ListViewComboBoxResponsible_SelectionChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListViewComboBoxResponsible.SelectionChanged
        Dim listViewItem As ListViewItem = ListViewComboBoxResponsible.SelectedListViewitem
        LabelResponsible.Text = listViewItem.SubItems(1).Text
        Presenter.UpdateResponsible(ListViewComboBoxResponsible.Text, LabelResponsible.Text)
        ToolStripButtonApplyEnabled(RequiredFieldsAreFilledIn())
    End Sub

    Private Sub ListViewComboBoxResponsible_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs)
        ListViewComboBoxKeyPressed(ListViewComboBoxResponsible)
    End Sub

    Private Sub ListViewComboBoxKeyPressed(ByVal listviewCombobox As ListViewComboBox)
        Try
            Dim listViewItem As ListViewItem = listviewCombobox.ListView.FindItemWithText(listviewCombobox.GetSearchString(), True, 0)
            If (listViewItem IsNot Nothing) Then
                listviewCombobox.ListView.Items(listViewItem.Index).Selected = True
                listviewCombobox.ListView.Items(listViewItem.Index).EnsureVisible()
            End If
        Catch ex As Exception
            ShowErrorMessage(ex.Message)
        End Try
    End Sub

    Private Sub ToolStripButtonApplyEnabled(ByVal enabled As Boolean)
        ToolStripButtonApply.Enabled = enabled
    End Sub

    Private Function RequiredFieldsAreFilledIn() As Boolean
        Return TextBoxWorkPackage.Text.Trim() IsNot Nothing AndAlso TextBoxEstimatedTime.Text.Trim() IsNot Nothing AndAlso ListViewComboBoxResponsible.Text IsNot Nothing
    End Function

    Private Sub ShowErrorMessage(ByVal message As String)
        MessageBox.Show(message)
    End Sub

    Private Property Presenter() As PresenterFormWorkPackageEditor
        Get
            Return m_Presenter
        End Get
        Set(ByVal value As PresenterFormWorkPackageEditor)
            m_Presenter = value
        End Set
    End Property

#Region "IViewFormWorkPackageEditor"
    Public Function GetChildPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetChildPanel
        Return Nothing
    End Function

    Public Function GetText() As String Implements Presenters.IChildView.GetText
        Return SCREEN_NAME
    End Function

    Public Sub ShowChild() Implements Presenters.IChildView.ShowChild
        Show()
        Presenter.SendChangedViewEvent()
    End Sub

    Public Sub UpdateTopLevel(ByVal value As Boolean) Implements Presenters.IChildView.UpdateTopLevel
        SetTopLevel(value)
    End Sub

    Public Function GetMainPanel() As System.Windows.Forms.Panel Implements Presenters.IChildView.GetMainPanel
        Return PanelMain
    End Function

    Public Function GetToolstrip() As System.Windows.Forms.ToolStrip Implements Presenters.IChildView.GetToolstrip
        Return Nothing
    End Function

    Public Sub UpdateView(ByVal workPackage As Business.DataHolders.WorkPackage) Implements Presenters.IViewFormWorkPackageEditor.UpdateView
        TextBoxWorkPackage.Text = workPackage.Name
        TextBoxEstimatedTime.Text = workPackage.EstimatedTime
        ListViewComboBoxResponsible.Text = workPackage.ResponsibleId
        LabelResponsible.Text = workPackage.ResponsibleName
        DateTimePickerDueDate.Value = workPackage.DueDate
    End Sub

    Public Sub UpdateResposible() Implements Presenters.IViewFormWorkPackageEditor.UpdateResposible
        ListViewComboBoxResponsible.Invalidate()
    End Sub

    Public Sub ClearListViewComboBoxResponsible() Implements Presenters.IViewFormWorkPackageEditor.ClearListViewComboBoxResponsible
        ListViewComboBoxResponsible.ListView.VirtualModeCache.Clear()
    End Sub

    Public Sub AddToListViewComboBoxResponsible(ByVal fourStateListViewItem As CustomControls.FourStateListViewItem) Implements Presenters.IViewFormWorkPackageEditor.AddToListViewComboBoxResponsible
        ListViewComboBoxResponsible.ListView.VirtualModeCache.Add(fourStateListViewItem)
    End Sub
#End Region

End Class