﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Windows.Forms

Public Class PresenterFormMain
    Public Const OVERVIEW As String = "Overview"
    Public Const PROJECT_EDITOR As String = "ProjectEditor"
    Public Const CATEGORIES As String = "Categories"
    Public Const PART_DESCRIPTION As String = "PartDescription"
    Public Const ROLES As String = "Role"
    Private m_View As IViewFormMain
    Private m_CurrentPanel As Panel
    Private m_ChildViews As IDictionary(Of String, IChildView)

    Public Sub New(ByVal view As IViewFormMain, ByVal childViews As IDictionary(Of String, IChildView))
        MyClass.View = view
        MyClass.ChildViews = childViews

        'add listener for changed view
        AddHandler DispatcherChangedViewEvent.ChangedView, AddressOf ChangedView

        'add listener for closed view
        AddHandler DispatcherClosedViewEvent.ClosedView, AddressOf ClosedView

        'add listener for open view
        AddHandler DispatcherOpenViewEvent.OpenView, AddressOf OpenView

        'add listener for open edit view
        AddHandler DispatcherOpenProjectEditorViewEvent.OpenProjectEditorView, AddressOf OpenProjectEditorView

        'get and set privileges
        Dim task As TaskPrivileges = New TaskPrivileges()
        TaskInformation.Instance.SetPrivileges = task.GetPrivilegesForEmployeeID(TaskInformation.Instance.UserId)

        'show default view
        ShowChildPanelAndToolStrip(MyClass.ChildViews(OVERVIEW))

        view.SetApplicationVersion(TaskInformation.Instance.ApplicationVersion)
    End Sub

    Public Sub ShowChildPanelAndToolStrip(ByVal child As IChildView)
        'RevertMergedToolStrips(View)
        MergeAndShowChildForm(View, child)
    End Sub

    Public ReadOnly Property UserId() As String
        Get
            Return TaskInformation.Instance.UserId
        End Get
    End Property

    Private Sub ClosedView(ByVal sender As Object, ByVal e As ClosedViewEvent)
        If (sender.GetType() Is GetType(PresenterFormProjectEditor)) Then
            View.CloseProjectEditor()
            ShowChildPanelAndToolStrip(ChildViews(OVERVIEW))
        ElseIf (sender.GetType() Is GetType(PresenterFormCategoryOverview)) Then
            View.CloseCategories()
            ShowChildPanelAndToolStrip(ChildViews(OVERVIEW))
        ElseIf (sender.GetType() Is GetType(PresenterFormPartDescriptionOverview)) Then
            View.ClosePartDescription()
            ShowChildPanelAndToolStrip(ChildViews(OVERVIEW))
        ElseIf (sender.GetType() Is GetType(PresenterFormRoleOverview)) Then
            View.CloseRole()
            ShowChildPanelAndToolStrip(ChildViews(OVERVIEW))
        End If
    End Sub

    Private Sub OpenView(ByVal sender As Object, ByVal e As OpenViewEvent)
        Select Case e.Name
            Case CATEGORIES
                View.CreateCategories()
                ChildViews.Item(CATEGORIES) = View.GetCategories()
                ShowChildPanelAndToolStrip(ChildViews(CATEGORIES))
            Case PART_DESCRIPTION
                View.CreatePartDescription()
                ChildViews.Item(PART_DESCRIPTION) = View.GetPartDescription()
                ShowChildPanelAndToolStrip(ChildViews(PART_DESCRIPTION))
            Case ROLES
                View.CreateRole()
                ChildViews.Item(ROLES) = View.GetRole()
                ShowChildPanelAndToolStrip(ChildViews(ROLES))
        End Select
    End Sub

    Private Sub OpenProjectEditorView(ByVal sender As Object, ByVal e As OpenEditorViewEvent(Of ProjectProposalRevision))
        Select Case e.FormName
            Case PROJECT_EDITOR
                View.CreateProjectEditor(e.SelectedData)
                ChildViews(PROJECT_EDITOR) = View.GetProjectEditor()
                ShowChildPanelAndToolStrip(ChildViews(PROJECT_EDITOR))
        End Select
    End Sub

    Private Sub MergeAndShowChildForm(ByVal parent As IViewFormMain, ByVal child As IChildView)
        'toolstrip
        'remove merged toolstrip
        PanelAndToollStripManager.RevertMergedToolStrips(View)
        View.RefreshForm()
        'get child toolstrip
        If (child.GetToolstrip() IsNot Nothing) Then
            PanelAndToollStripManager.MergeToolStrips(View, child)
        End If

        'panel
        'get child panel
        If (Not parent.GetMainPanel().Contains(child.GetMainPanel())) Then
            PanelAndToollStripManager.MergePanels(parent, child)
        End If
        child.ShowChild()
        'update parent panel
        If (CurrentPanel IsNot Nothing) Then
            CurrentPanel.Hide()
        End If
        CurrentPanel = child.GetMainPanel()
        CurrentPanel.Show()
    End Sub

    Private Sub ChangedView(ByVal sender As Object, ByVal e As ChangedViewEvent)
        Dim debug As String = Nothing
        If (TaskInformation.Instance.IsDevelopment) Then
            debug = " - DEBUG MODE"
        End If
        View.UpdateText(String.Format("{0} [{1}]{2}", TaskInformation.Instance.ApplicationName, e.Text, debug))
    End Sub

    Private Property View() As IViewFormMain
        Get
            Return m_View
        End Get
        Set(ByVal value As IViewFormMain)
            m_View = value
        End Set
    End Property

    Private Property CurrentPanel() As Panel
        Get
            Return m_CurrentPanel
        End Get
        Set(ByVal value As Panel)
            m_CurrentPanel = value
        End Set
    End Property

    Private Property ChildViews() As IDictionary(Of String, IChildView)
        Get
            Return m_ChildViews
        End Get
        Set(ByVal value As IDictionary(Of String, IChildView))
            m_ChildViews = value
        End Set
    End Property
End Class
