﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Windows.Forms
Imports CustomControls

Public Class PresenterFormOverview
    Private m_View As IViewFormOverview
    Private m_SelectedProjectNameId As Integer

    Public Sub New(ByVal view As IViewFormOverview)
        MyClass.View = view
        SelectedProjectNameId = -1
    End Sub

    Public Sub RequestAllProjectNames()
        Try
            View.ClearExtendedListViewName()
            Dim projectNames As IList(Of ProjectProposalName) = TaskProjectProposalName.Instance.GetAllProjectProposalNames()
            If (projectNames IsNot Nothing) Then
                Dim enumerator As IEnumerator(Of ProjectProposalName) = projectNames.GetEnumerator()
                While (enumerator.MoveNext())
                    Dim listViewItem As FourStateListViewItem = New FourStateListViewItem()
                    listViewItem.Text = enumerator.Current.Name
                    listViewItem.Tag = enumerator.Current.Id
                    View.AddToExtendedListViewName(listViewItem)
                End While
            End If
            View.UpdateNames()
        Catch ex As Exception
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Sub

    Public Sub RequestAllRevisionsForProject(ByVal projectProposalNameId As Integer)
        Try
            View.ClearExtendedListViewRevision()
            Dim properties As IList(Of String) = New List(Of String)()
            properties.Add("Id")
            properties.Add("RevisionInformation")

            Dim projectRevisions As IList(Of ProjectProposalRevision) = TaskProjectProposalRevision.Instance.GetAllRevisionsForProjectPrososalNameId(projectProposalNameId, properties)
            If (projectRevisions IsNot Nothing) Then
                Dim enumerator As IEnumerator(Of ProjectProposalRevision) = projectRevisions.GetEnumerator()
                While (enumerator.MoveNext())
                    Dim listViewItem As FourStateListViewItem = New FourStateListViewItem()
                    listViewItem.Text = enumerator.Current.RevisionInformation.RevisionNumber
                    listViewItem.SubItems.Add(New ListViewItem.ListViewSubItem(listViewItem, enumerator.Current.RevisionInformation.RevisionStatus.Name))
                    listViewItem.SubItems.Add(New ListViewItem.ListViewSubItem(listViewItem, enumerator.Current.RevisionInformation.DateModified.ToShortDateString()))
                    listViewItem.SubItems.Add(New ListViewItem.ListViewSubItem(listViewItem, enumerator.Current.RevisionInformation.Description))
                    listViewItem.Tag = enumerator.Current.Id
                    View.AddToExtendedListViewRevision(listViewItem)
                End While
            End If
            View.UpdateRevisions()
        Catch ex As Exception
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Sub

    Public Sub RequestIsAdmin()
        View.AdminFunction(TaskInformation.Instance.IsAdmin)
    End Sub

    Public Sub ShowProjectEditor(ByVal id As Integer)
        Try
            Dim project As ProjectProposalRevision = TaskProjectProposalRevision.Instance.GetProjectProposalRevisionById(id)
            DispatcherOpenProjectEditorViewEvent.SendOpenProjectEditorViewEvent(Me, New OpenEditorViewEvent(Of ProjectProposalRevision)(PresenterFormMain.PROJECT_EDITOR, project))
        Catch ex As Exception
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Sub

    Public Sub SendChangedViewEvent()
        DispatcherChangedViewEvent.SendChangedViewEvent(Me, New ChangedViewEvent(View.GetText(), Nothing))
    End Sub

    Public Sub SendOpenProjectEditor(ByVal project As ProjectProposalRevision)
        DispatcherOpenProjectEditorViewEvent.SendOpenProjectEditorViewEvent(Me, New OpenEditorViewEvent(Of ProjectProposalRevision)(PresenterFormMain.PROJECT_EDITOR, project))
    End Sub

    Public Sub SendOpenCategories()
        DispatcherOpenViewEvent.SendOpenViewEvent(Me, New OpenViewEvent(PresenterFormMain.CATEGORIES))
    End Sub

    Public Sub SendOpenPartDescription()
        DispatcherOpenViewEvent.SendOpenViewEvent(Me, New OpenViewEvent(PresenterFormMain.PART_DESCRIPTION))
    End Sub

    Public Sub SendOpenRoles()
        DispatcherOpenViewEvent.SendOpenViewEvent(Me, New OpenViewEvent(PresenterFormMain.ROLES))
    End Sub

    Public Sub CreateNewRevision()
        Try
            TaskRevision.Instance.CreateAndSaveNewRevision(SelectedProjectNameId)
            RequestAllRevisionsForProject(SelectedProjectNameId)
        Catch ex As Exception
            Throw New Exception(ex.Message)
        End Try
    End Sub

    Public Property SelectedProjectNameId() As Integer
        Get
            Return m_SelectedProjectNameId
        End Get
        Set(ByVal value As Integer)
            m_SelectedProjectNameId = value
        End Set
    End Property

    Private Property View() As IViewFormOverview
        Get
            Return m_View
        End Get
        Set(ByVal value As IViewFormOverview)
            m_View = value
        End Set
    End Property

End Class
