﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Windows.Forms
Imports CustomControls

Public Class PresenterFormProjectScreen1
    Private m_View As IViewProjectScreen1
    Private m_IsAuditor As Boolean

    Public Sub New(ByVal view As IViewProjectScreen1, ByVal isAuditor As Boolean)
        MyClass.View = view
        MyClass.IsAuditor = isAuditor

        'add listener for RefreshViewOnSave event
        AddHandler DispatcherRefreshViewOnSaveEvent.RefreshViewOnSave, AddressOf RefreshViewOnSave
    End Sub

    Public Sub SendChangedViewEvent()
        DispatcherChangedViewEvent.SendChangedViewEvent(Me, New ChangedViewEvent(View.GetText(), TaskFormDescription.Instance.GetDescriptionByFormName(View.GetName()).Description))
    End Sub

    Public Sub RequestSelection()
        Try
            View.UpdateView(TaskProjectProposalRevision.Instance.GetSelection())
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public Sub RequestAllCategories()
        Try
            View.ClearListViewComboBoxCategory()
            Dim categories As IList(Of Category) = TaskCategory.Instance.GetAllCategories()
            If (categories IsNot Nothing) Then
                Dim enumerator As IEnumerator(Of Category) = categories.GetEnumerator()
                While (enumerator.MoveNext())
                    Dim listViewItem As FourStateListViewItem = New FourStateListViewItem()
                    listViewItem.Text = enumerator.Current.Description
                    listViewItem.Tag = enumerator.Current.Id
                    View.AddToListViewComboBoxCategory(listViewItem)
                End While
            End If
            View.UpdateCatagory()
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public Sub RequestSubCategoryForSelectedCategory(ByVal categoryId As Integer)
        Try
            View.ClearListViewComboBoxSubCategory()
            Dim subCategories As IList(Of SubCategory) = TaskSubCategory.Instance.GetSubCategoriesByCategoryId(categoryId)
            View.AddToListViewComboBoxSubCategory(GetFourStateListViewItemsForSubCategories(subCategories))
            View.UpdateSubCatagory()
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public Sub RequestSubCategoryForSelectedCategory(ByVal categoryDescription As String)
        Try
            View.ClearListViewComboBoxSubCategory()
            Dim category As Category = TaskCategory.Instance.GetCategoryByDescription(categoryDescription)
            Dim subCategories As IList(Of SubCategory) = TaskSubCategory.Instance.GetSubCategoriesByCategoryId(category.Id)
            View.AddToListViewComboBoxSubCategory(GetFourStateListViewItemsForSubCategories(subCategories))
            View.UpdateSubCatagory()
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public Sub RequestAllRevisionStatuses()
        Try
            View.ClearListViewComboBoxRevisionStatus()
            Dim revisionStatuses As IList(Of RevisionStatus) = TaskRevisionStatus.Instance.GetAllRevisionStatuses()
            If (revisionStatuses IsNot Nothing) Then
                Dim enumerator As IEnumerator(Of RevisionStatus) = revisionStatuses.GetEnumerator()
                While (enumerator.MoveNext())
                    Dim addToCache As Boolean = True
                    If (enumerator.Current.Name.Equals(TaskRevisionStatus.REJECTED) OrElse enumerator.Current.Name.Equals(TaskRevisionStatus.RELEASED)) Then
                        addToCache = False
                    End If
                    If (addToCache) Then
                        Dim listViewItem As FourStateListViewItem = New FourStateListViewItem()
                        listViewItem.Text = enumerator.Current.Name
                        listViewItem.Tag = enumerator.Current.Id
                        View.AddToListViewComboBoxRevisionStatus(listViewItem)
                    End If
                End While
            End If
            View.UpdateRevisionStatus()
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public Sub RequestAllEmployeesForAuditor()
        Try
            View.ClearListViewComboBoxAuditor()
            View.AddToListViewComboBoxAuditor(GetFourStateListViewItemsForEmployees())
            View.UpdateAuditor()
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public Sub RequestAllEmployeesForSponsor()
        Try
            View.ClearListViewComboBoxSponsor()
            View.AddToListViewComboBoxSponsor(GetFourStateListViewItemsForEmployees())
            View.UpdateSponsor()
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public Sub RequestAllEmployeesForPusher()
        Try
            View.ClearListViewComboBoxPusher()
            View.AddToListViewComboBoxPusher(GetFourStateListViewItemsForEmployees())
            View.UpdatePusher()
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public Sub RequestAllEmployeesForCoordinator()
        Try
            View.ClearListViewComboBoxCoordinator()
            View.AddToListViewComboBoxCoordinator(GetFourStateListViewItemsForEmployees())
            View.UpdateCoordinator()
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public Sub UpdateProjectName(ByVal name As String)
        Try
            TaskProjectProposalRevision.Instance.UpdateProjectName(name)
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public Sub UpdateAuditor(ByVal employeeId As String)
        Try
            TaskProjectProposalRevision.Instance.UpdateAuditor(employeeId)
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public Sub UpdateProjectDescription(ByVal projectDescription As String)
        Try
            TaskProjectProposalRevision.Instance.UpdateDescription(projectDescription)
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public Sub UpdateCategory(ByVal category As Category)
        Try
            TaskProjectProposalRevision.Instance.UpdateCategory(category)
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public Sub UpdateSubCategory(ByVal subCategory As SubCategory)
        Try
            TaskProjectProposalRevision.Instance.UpdateSubCategory(subCategory)
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public Sub UpdateSponsor(ByVal sponsorId As String, ByVal sponsorName As String)
        Try
            TaskProjectProposalRevision.Instance.UpdateSponsor(sponsorId, sponsorName)
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public Sub UpdatePusher(ByVal pusherId As String, ByVal pusherName As String)
        Try
            TaskProjectProposalRevision.Instance.UpdatePusher(pusherId, pusherName)
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public Sub UpdateCoordinator(ByVal coordinatorId As String, ByVal coordinatorName As String)
        Try
            TaskProjectProposalRevision.Instance.UpdateCoordinator(coordinatorId, coordinatorName)
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public Sub UpdateBudget(ByVal budget As String)
        Try
            TaskProjectProposalRevision.Instance.UpdateBudget(budget)
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public Sub UpdateCurrentRevisionStatus(ByVal revisionName As String, ByVal revisionId As Integer)
        Try
            Dim currentRevision As RevisionStatus = New RevisionStatus() With {.Id = revisionId, .Name = revisionName}
            TaskProjectProposalRevision.Instance.UpdateRevisionStatus(currentRevision)
        Catch ex As Exception
            ThrowException(ex)
        End Try
    End Sub

    Public ReadOnly Property IsEditable() As Boolean
        Get
            Return TaskProjectProposalRevision.Instance.IsEditable()
        End Get
    End Property

    Private Sub ThrowException(ByVal exception As Exception)
        Throw New Exception(exception.Message)
    End Sub

    Private Sub RefreshViewOnSave(ByVal sender As Object, ByVal e As RefreshViewOnSaveEvent)
        View.UpdateView(TaskProjectProposalRevision.Instance.GetSelection())
    End Sub

    Private Function GetFourStateListViewItemsForSubCategories(ByVal subCategories As IList(Of SubCategory)) As IList(Of FourStateListViewItem)
        GetFourStateListViewItemsForSubCategories = New List(Of FourStateListViewItem)()
        If (subCategories IsNot Nothing) Then
            Dim enumerator As IEnumerator(Of SubCategory) = subCategories.GetEnumerator()
            While (enumerator.MoveNext())
                Dim listViewItem As FourStateListViewItem = New FourStateListViewItem()
                listViewItem.Text = enumerator.Current.Description
                listViewItem.Tag = enumerator.Current.Id
                GetFourStateListViewItemsForSubCategories.Add(listViewItem)
            End While
        End If
    End Function

    Private Function GetFourStateListViewItemsForEmployees() As IList(Of FourStateListViewItem)
        GetFourStateListViewItemsForEmployees = New List(Of FourStateListViewItem)()
        Dim employees As IList(Of Employee) = TaskEmployee.Instance.GetAllEmployees()
        If (employees IsNot Nothing) Then
            Dim enumerator As IEnumerator(Of Employee) = employees.GetEnumerator()
            While (enumerator.MoveNext())
                Dim listViewItem As FourStateListViewItem = New FourStateListViewItem()
                listViewItem.Text = enumerator.Current.EmployeeId
                listViewItem.SubItems.Add(New ListViewItem.ListViewSubItem(listViewItem, enumerator.Current.ToString()))
                GetFourStateListViewItemsForEmployees.Add(listViewItem)
            End While
        End If
    End Function

    Private Property View() As IViewProjectScreen1
        Get
            Return m_View
        End Get
        Set(ByVal value As IViewProjectScreen1)
            m_View = value
        End Set
    End Property

    Private Property IsAuditor() As Boolean
        Get
            Return m_IsAuditor
        End Get
        Set(ByVal value As Boolean)
            m_IsAuditor = value
        End Set
    End Property
End Class
