﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Windows.Forms

Public Class PresenterFormProjectScreen4
    Public Const TIMING_TEAM As String = "Timing_Team"
    Public Const WORK_PACKAGE_EDITOR As String = "WorkPackageEditor"
    Public Const MILESTONE_EDITOR As String = "MilestoneEditor"
    Public Const TEAM_MEMBER_EDITOR As String = "TeamMemberEditor"

    Private m_View As IViewProjectScreen4
    Private m_CurrentPanel As Panel
    Private m_ChildViews As IDictionary(Of String, IChildView)

    Public Sub New(ByVal view As IViewProjectScreen4, ByVal childViews As IDictionary(Of String, IChildView))
        MyClass.View = view
        MyClass.ChildViews = childViews

        'add listener for closed view
        AddHandler DispatcherClosedViewEvent.ClosedView, AddressOf ClosedView

        'add listener for open view
        AddHandler DispatcherOpenWorkPackageEditorViewEvent.OpenWorkPackageEditorView, AddressOf OpenWorkPackageEditorView
        AddHandler DispatcherOpenMilestoneEditorViewEvent.OpenMilestoneEditorView, AddressOf OpenMilestoneEditorView
        AddHandler DispatcherOpenTeamMemberEditorViewEvent.OpenTeamMemberEditorView, AddressOf OpenTeamMemberEditorView
    End Sub

    Public Sub SendChangedViewEvent()
        DispatcherChangedViewEvent.SendChangedViewEvent(Me, New ChangedViewEvent(View.GetText(), TaskFormDescription.Instance.GetDescriptionByFormName(View.GetName()).Description))
    End Sub

    Public Sub ShowChildPanelAndToolStrip(ByVal child As IChildView)
        MergeAndShowChildForm(View, child)
    End Sub

    Public Sub ShowDefaultScreen()
        'default screen
        ShowChildPanelAndToolStrip(ChildViews(TIMING_TEAM))
    End Sub

    Private Sub MergeAndShowChildForm(ByVal parent As IChildView, ByVal child As IChildView)
        'panel
        'get child panel
        If (Not parent.GetMainPanel().Contains(child.GetMainPanel())) Then
            PanelAndToollStripManager.MergePanels(parent, child)
        End If
        child.ShowChild()
        'update parent panel
        If (CurrentPanel IsNot Nothing) Then
            CurrentPanel.Hide()
        End If
        CurrentPanel = child.GetMainPanel()
        CurrentPanel.Show()
    End Sub

    Private Sub ClosedView(ByVal sender As Object, ByVal e As ClosedViewEvent)
        If (sender.GetType() Is GetType(PresenterFormWorkPackageEditor) OrElse _
            sender.GetType() Is GetType(PresenterFormMilestoneEditor) OrElse _
            sender.GetType() Is GetType(PresenterFormTeamMemberEditor)) Then
            ShowChildPanelAndToolStrip(ChildViews(TIMING_TEAM))
        End If
    End Sub

    Private Sub OpenWorkPackageEditorView(ByVal sender As Object, ByVal e As OpenEditorViewEvent(Of WorkPackage))
        Select Case e.FormName
            Case WORK_PACKAGE_EDITOR
                View.CreateWorkPackageEditor(e.SelectedData)
                ChildViews(WORK_PACKAGE_EDITOR) = View.GetWorkPackageEditor()
                ShowChildPanelAndToolStrip(ChildViews(WORK_PACKAGE_EDITOR))
        End Select
    End Sub

    Private Sub OpenMilestoneEditorView(ByVal sender As Object, ByVal e As OpenEditorViewEvent(Of Milestone))
        Select Case e.FormName
            Case MILESTONE_EDITOR
                View.CreateMilestoneEditor(e.Name, e.SelectedData)
                ChildViews(MILESTONE_EDITOR) = View.GetMilestoneEditor()
                ShowChildPanelAndToolStrip(ChildViews(MILESTONE_EDITOR))
        End Select
    End Sub

    Private Sub OpenTeamMemberEditorView(ByVal sender As Object, ByVal e As OpenEditorViewEvent(Of TeamMember))
        Select Case e.FormName
            Case TEAM_MEMBER_EDITOR
                View.CreateTeamMemberEditor(e.SelectedData)
                ChildViews(TEAM_MEMBER_EDITOR) = View.GetTeamMemberEditor()
                ShowChildPanelAndToolStrip(ChildViews(TEAM_MEMBER_EDITOR))
        End Select
    End Sub

    Private Property View() As IViewProjectScreen4
        Get
            Return m_View
        End Get
        Set(ByVal value As IViewProjectScreen4)
            m_View = value
        End Set
    End Property

    Private Property CurrentPanel() As Panel
        Get
            Return m_CurrentPanel
        End Get
        Set(ByVal value As Panel)
            m_CurrentPanel = value
        End Set
    End Property

    Private Property ChildViews() As IDictionary(Of String, IChildView)
        Get
            Return m_ChildViews
        End Get
        Set(ByVal value As IDictionary(Of String, IChildView))
            m_ChildViews = New Dictionary(Of String, IChildView)(value)
        End Set
    End Property
End Class
