﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Windows.Forms

Public Class PresenterFormRole
    Public Const ROLE_OVERVIEW As String = "RoleOverview"
    Public Const ROLE_EDITOR As String = "RoleEditor"

    Private m_View As IViewFormRole
    Private m_CurrentPanel As Panel
    Private m_ChildViews As IDictionary(Of String, IChildView)

    Public Sub New(ByVal view As IViewFormRole, ByVal childViews As IDictionary(Of String, IChildView))
        MyClass.View = view
        MyClass.ChildViews = childViews

        'add listener for closed view
        AddHandler DispatcherClosedViewEvent.ClosedView, AddressOf ClosedView

        'add listener for open view
        AddHandler DispatcherOpenRoleEditorViewEvent.OpenRoleEditorView, AddressOf OpenRoleEditorView

        'default veiw
        ShowChildPanelAndToolStrip(MyClass.ChildViews(ROLE_OVERVIEW))
    End Sub

    Private Sub ShowChildPanelAndToolStrip(ByVal child As IChildView)
        MergeAndShowChildForm(View, child)
    End Sub

    Private Sub MergeAndShowChildForm(ByVal parent As IChildView, ByVal child As IChildView)
        'get child toolstrip
        If (child.GetToolstrip() IsNot Nothing) Then
            PanelAndToollStripManager.MergeToolStrips(View, child)
        End If

        'panel
        'get child panel
        If (Not parent.GetMainPanel().Contains(child.GetMainPanel())) Then
            PanelAndToollStripManager.MergePanels(parent, child)
        End If
        child.ShowChild()

        'update parent panel
        If (CurrentPanel IsNot Nothing) Then
            CurrentPanel.Hide()
        End If
        CurrentPanel = child.GetMainPanel()
        CurrentPanel.Show()
    End Sub

    Private Sub ClosedView(ByVal sender As Object, ByVal e As ClosedViewEvent)
        If (sender.GetType() Is GetType(PresenterFormRoleEditor)) Then
            View.CloseRoleEditor()
            ShowChildPanelAndToolStrip(ChildViews(ROLE_OVERVIEW))
        End If
    End Sub

    Private Sub OpenRoleEditorView(ByVal sender As Object, ByVal e As OpenEditorViewEvent(Of Role))
        Select Case e.FormName
            Case ROLE_EDITOR
                View.CreateRoleEditor(e.SelectedData)
                ChildViews(ROLE_EDITOR) = View.GetRoleEditor()
                ShowChildPanelAndToolStrip(ChildViews(ROLE_EDITOR))
        End Select
    End Sub

    Private Property View() As IViewFormRole
        Get
            Return m_View
        End Get
        Set(ByVal value As IViewFormRole)
            m_View = value
        End Set
    End Property

    Private Property CurrentPanel() As Panel
        Get
            Return m_CurrentPanel
        End Get
        Set(ByVal value As Panel)
            m_CurrentPanel = value
        End Set
    End Property

    Private Property ChildViews() As IDictionary(Of String, IChildView)
        Get
            Return m_ChildViews
        End Get
        Set(ByVal value As IDictionary(Of String, IChildView))
            m_ChildViews = value
        End Set
    End Property
End Class
