﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Windows.Forms
Imports CustomControls

Public Class PresenterFormRoleOverview
    Private m_View As IViewFormRoleOverview
    'Private m_ListViewRoleItemCache As List(Of ListViewItem)
    Private m_SelectedRoleId As Integer

    Public Sub New(ByVal view As IViewFormRoleOverview)
        MyClass.View = view
        SelectedRoleId = -1

        'add listener for closed view
        AddHandler DispatcherClosedViewEvent.ClosedView, AddressOf ClosedView
    End Sub

    Public Sub RequestAllRoles()
        Try
            View.ClearExtendedListViewRole()
            Dim roles As IList(Of Role) = TaskRole.Instance.GetAllRoles()
            If (roles IsNot Nothing) Then
                Dim enumerator As IEnumerator(Of Role) = roles.GetEnumerator()
                While (enumerator.MoveNext())
                    Dim listViewItem As FourStateListViewItem = New FourStateListViewItem()
                    listViewItem.Text = enumerator.Current.Name
                    listViewItem.Tag = enumerator.Current.Id
                    View.AddToExtendedListViewRole(listViewItem)
                End While
            End If
            View.UpdateRole()
        Catch ex As Exception
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Sub

    Public Sub RemoveRole(ByVal id As Integer)
        Try
            TaskRole.Instance.Remove(id)
        Catch ex As Exception
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Sub

    Public Sub Close()
        DispatcherClosedViewEvent.SendClosedViewEvent(Me, New ClosedViewEvent())
    End Sub

    Public Sub ShowRoleEditor(ByVal name As String)
        Dim role As Role = TaskRole.Instance.GetRoleByName(name)
        DispatcherOpenRoleEditorViewEvent.SendOpenRoleEditorViewEvent(Me, New OpenEditorViewEvent(Of Role)(PresenterFormRole.ROLE_EDITOR, role))
    End Sub

    Public Sub SendChangedViewEvent()
        DispatcherChangedViewEvent.SendChangedViewEvent(Me, New ChangedViewEvent(View.GetText(), Nothing))
    End Sub

    Private Sub ClosedView(ByVal sender As Object, ByVal e As ClosedViewEvent)
        If (sender.GetType() Is GetType(PresenterFormRoleEditor)) Then
            View.UpdateRole()
        End If
    End Sub

    Public Property SelectedRoleId() As Integer
        Get
            Return m_SelectedRoleId
        End Get
        Set(ByVal value As Integer)
            m_SelectedRoleId = value
        End Set
    End Property

    Private Property View() As IViewFormRoleOverview
        Get
            Return m_View
        End Get
        Set(ByVal value As IViewFormRoleOverview)
            m_View = value
        End Set
    End Property
End Class
