﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Windows.Forms
Imports CustomControls

Public Class PresenterFormTeamMemberEditor
    Private m_View As IViewFormTeamMemberEditor
    Private m_Task As TaskTeamMember

    Public Sub New(ByVal view As IViewFormTeamMemberEditor)
        MyClass.View = view
        Task = New TaskTeamMember()
        SendSubEditorOpenedEvent(False)
    End Sub

    Public Sub Apply()
        SendFormClosing()
    End Sub

    Public Sub Cancel()
        CloseView()
    End Sub

    Public Sub SetSelection(ByVal teamMember As TeamMember)
        Task.Add(teamMember)
    End Sub

    Public Sub UpdateTeamMemberInfo(ByVal employeeId As String, ByVal employeeName As String, ByVal department As String, ByVal subDepartment As String, ByVal emailAddress As String)
        Task.UpdateEmployeeID(employeeId)
        Task.UpdateEmployeeName(employeeName)
        Task.UpdateDepartment(department)
        Task.UpdateSubDepartment(subDepartment)
        Task.UpdateEmailAddress(emailAddress)
    End Sub

    Public Sub UpdateReportTo(ByVal employeeId As String)
        Dim reportTo As TeamMember = TaskProjectProposalRevision.Instance.GetTeamMemberByEmployeeId(employeeId)
        Task.UpdateReportTo(reportTo)
    End Sub

    Public Sub UpdateRole(ByVal roleId As Integer)
        Dim role As Role = TaskRole.Instance.GetRoleById(roleId)
        Task.UpdateRole(role)
    End Sub

    Private Sub SendClosedViewEvent()
        DispatcherClosedViewEvent.SendClosedViewEvent(Me, Nothing)
    End Sub

    Public Sub SendChangedViewEvent()
        DispatcherChangedViewEvent.SendChangedViewEvent(Me, New ChangedViewEvent(View.GetText(), Nothing))
    End Sub

    Public Sub RequestAllEmployeesForTeamMember()
        Try
            View.ClearListViewComboBoxTeamMember()
            View.AddToListViewComboBoxTeamMember(GetFourStateListViewItemsForEmployees())
            View.UpdateTeamMember()
        Catch ex As Exception
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Sub

    Public Sub UpdateTeamMember()
        TaskProjectProposalRevision.Instance.AddOrUpdateTeamMember(Task.GetSelection())
        CloseView()
    End Sub

    Public Sub RequestAllTeamMembersForReportTo()
        Try
            View.ClearListViewComboBoxReportTo()
            View.AddToListViewComboBoxReportTo(GetAllTeamMembers())
            View.UpdateReportTo()
        Catch ex As Exception
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Sub

    Public Sub RequestAllRoles()
        Try
            View.ClearListViewComboBoxRole()
            Dim roles As IList(Of Role) = TaskRole.Instance.GetAllRoles()
            If (roles IsNot Nothing) Then
                Dim enumerator As IEnumerator(Of Role) = roles.GetEnumerator()
                While (enumerator.MoveNext())
                    Dim listViewItem As FourStateListViewItem = New FourStateListViewItem()
                    listViewItem.Text = enumerator.Current.Name
                    listViewItem.Tag = enumerator.Current.Id
                    View.AddToListViewComboBoxRole(listViewItem)
                End While
            End If
            View.UpdateRole()
        Catch ex As Exception
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Sub

    Private Function GetFourStateListViewItemsForEmployees() As IList(Of FourStateListViewItem)
        GetFourStateListViewItemsForEmployees = New List(Of FourStateListViewItem)()
        Dim employees As IList(Of Employee) = TaskEmployee.Instance.GetAllEmployees()
        If (employees IsNot Nothing) Then
            Dim enumerator As IEnumerator(Of Employee) = employees.GetEnumerator()
            While (enumerator.MoveNext())
                Dim listViewItem As FourStateListViewItem = New FourStateListViewItem()
                listViewItem.Text = enumerator.Current.EmployeeId
                listViewItem.SubItems.Add(New ListViewItem.ListViewSubItem(listViewItem, enumerator.Current.ToString()))
                listViewItem.SubItems.Add(New ListViewItem.ListViewSubItem(listViewItem, enumerator.Current.Department))
                listViewItem.SubItems.Add(New ListViewItem.ListViewSubItem(listViewItem, enumerator.Current.SubDepartment))
                listViewItem.SubItems.Add(New ListViewItem.ListViewSubItem(listViewItem, enumerator.Current.EmailAddress))
                GetFourStateListViewItemsForEmployees.Add(listViewItem)
            End While
        End If
    End Function

    Private Function GetAllTeamMembers() As IList(Of FourStateListViewItem)
        GetAllTeamMembers = New List(Of FourStateListViewItem)()
        Dim teamMembers As IList(Of TeamMember) = TaskProjectProposalRevision.Instance.GetAllTeamMembers()
        If (teamMembers IsNot Nothing) Then
            Dim enumerator As IEnumerator(Of TeamMember) = teamMembers.GetEnumerator()
            While (enumerator.MoveNext())
                Dim listViewItem As FourStateListViewItem = New FourStateListViewItem()
                listViewItem.Text = enumerator.Current.EmployeeName
                listViewItem.Tag = enumerator.Current.EmployeeId
                GetAllTeamMembers.Add(listViewItem)
            End While
        End If
    End Function

    Private Sub CloseView()
        Task.ClearSelection()
        SendFormClosing()
    End Sub

    Private Sub SendFormClosing()
        SendClosedViewEvent()
        SendSubEditorOpenedEvent(True)
    End Sub

    Private Sub SendSubEditorOpenedEvent(ByVal enabled As Boolean)
        DispatcherSubEditorOpenedEvent.SendSubEditorOpenedEvent(Me, New SubEditorOpenedEvent(enabled))
    End Sub

    Private Property View() As IViewFormTeamMemberEditor
        Get
            Return m_View
        End Get
        Set(ByVal value As IViewFormTeamMemberEditor)
            m_View = value
        End Set
    End Property

    Private Property Task() As TaskTeamMember
        Get
            Return m_Task
        End Get
        Set(ByVal value As TaskTeamMember)
            m_Task = value
        End Set
    End Property
End Class
