﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports System.Windows.Forms
Imports CustomControls

Public Class PresenterFormWorkPackageEditor
    Private m_View As IViewFormWorkPackageEditor
    Private m_Task As TaskWorkPackage

    Public Sub New(ByVal view As IViewFormWorkPackageEditor)
        MyClass.View = view
        Task = New TaskWorkPackage()
        SendSubEditorOpenedEvent(False)
    End Sub

    Public Sub Apply()
        SendFormClosing()
    End Sub

    Public Sub Cancel()
        CloseView()
    End Sub

    Public Sub SetSelection(ByVal workPackage As WorkPackage)
        Task.Add(workPackage)
    End Sub

    Public Sub UpdateName(ByVal name As String)
        Task.UpdateName(name)
    End Sub

    Public Sub UpdateEstimatedTime(ByVal estimatedTime As String)
        Task.UpdateEstimatedTime(estimatedTime)
    End Sub

    Public Sub UpdateResponsible(ByVal responsibleId As String, ByVal responsibleName As String)
        Task.UpdateResponsible(responsibleId, responsibleName)
    End Sub

    Public Sub UpdateDueDate(ByVal dueDate As DateTime)
        Task.UpdateDueDate(dueDate)
    End Sub

    Public Sub SendChangedViewEvent()
        DispatcherChangedViewEvent.SendChangedViewEvent(Me, New ChangedViewEvent(View.GetText(), Nothing))
    End Sub

    Public Sub RequestAllTeamMembers()
        Try
            View.ClearListViewComboBoxResponsible()
            If (TaskProjectProposalRevision.Instance.GetSelection().TeamMembers IsNot Nothing) Then
                Dim enumerator As IEnumerator(Of TeamMember) = TaskProjectProposalRevision.Instance.GetSelection().TeamMembers.GetEnumerator()
                While (enumerator.MoveNext())
                    Dim listViewItem As FourStateListViewItem = New FourStateListViewItem()
                    listViewItem.Text = enumerator.Current.EmployeeId
                    listViewItem.SubItems.Add(New ListViewItem.ListViewSubItem(listViewItem, enumerator.Current.EmployeeName))
                    View.AddToListViewComboBoxResponsible(listViewItem)
                End While
            End If
            View.UpdateResposible()
        Catch ex As Exception
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Sub

    Public Sub UpdateWorkPackage()
        TaskProjectProposalRevision.Instance.AddOrUpdateWorkPackage(Task.GetSelection())
        CloseView()
    End Sub

    Private Sub CloseView()
        Task.ClearSelection()
        SendFormClosing()
    End Sub

    Private Sub SendFormClosing()
        SendClosedViewEvent()
        SendSubEditorOpenedEvent(True)
    End Sub

    Private Sub SendClosedViewEvent()
        DispatcherClosedViewEvent.SendClosedViewEvent(Me, Nothing)
    End Sub

    Private Sub SendSubEditorOpenedEvent(ByVal enabled As Boolean)
        DispatcherSubEditorOpenedEvent.SendSubEditorOpenedEvent(Me, New SubEditorOpenedEvent(enabled))
    End Sub

    Private Property View() As IViewFormWorkPackageEditor
        Get
            Return m_View
        End Get
        Set(ByVal value As IViewFormWorkPackageEditor)
            m_View = value
        End Set
    End Property

    Private Property Task() As TaskWorkPackage
        Get
            Return m_Task
        End Get
        Set(ByVal value As TaskWorkPackage)
            m_Task = value
        End Set
    End Property
End Class
