﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business
Imports Securex.FIT.Fitclient.Business.DataHolders
Imports NHibernate
Imports NHibernate.Criterion

Public Class SqlSubCategory
    Implements ISubCategoryDao
    Private Const ID As String = "Id"
    Private Const DESCRIPTION As String = "Description"
    Private Const CATEGORY_ID As String = "Category.Id"
    Private m_GenericNhbDao As GenericNhbDao(Of SubCategory)

    Public Sub New()
        m_GenericNhbDao = New GenericNhbDao(Of SubCategory)()
    End Sub

#Region "ISubCategoryDao"
    ''' <summary>
    ''' Get all subcategories assign to categorie
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <param name="categoryId">Catagorie row identifier</param>
    ''' <returns>Returns an IList of subcategories</returns>
    ''' <remarks></remarks>
    Public Function GetCategoryByCategoryId(ByVal session As Business.ISessionDao, ByVal categoryId As Integer) As System.Collections.Generic.IList(Of Business.DataHolders.SubCategory) Implements Business.ISubCategoryDao.GetCategoryByCategoryId
        Try
            Dim orders As Order() = {New Order(SqlSubCategory.CATEGORY_ID, True)}
            Dim criterions As ICriterion() = {New LikeExpression(SqlSubCategory.CATEGORY_ID, categoryId.ToString(), MatchMode.Exact)}
            Return m_GenericNhbDao.GetForCriteria(DirectCast(session, SqlSession).Session, criterions, orders)
        Catch ex As NHibernateException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Function

    ''' <summary>
    ''' Get subcategory by description
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <param name="description">Description to search for</param>
    ''' <returns>Returns an IList of subcategories</returns>
    ''' <remarks></remarks>
    Public Function GetSubCategoryByDescription(ByVal session As Business.ISessionDao, ByVal description As String) As System.Collections.Generic.IList(Of Business.DataHolders.SubCategory) Implements Business.ISubCategoryDao.GetSubCategoryByDescription
        Try
            Dim orders As Order() = {New Order(SqlSubCategory.DESCRIPTION, True)}
            Dim criterions As ICriterion() = {New LikeExpression(SqlSubCategory.DESCRIPTION, description, MatchMode.Exact)}
            Return m_GenericNhbDao.GetForCriteria(DirectCast(session, SqlSession).Session, criterions, orders)
        Catch ex As NHibernateException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Function

    ''' <summary>
    ''' Get subcategory by row identifier
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <param name="id">Row identifier</param>
    ''' <returns>Returns an IList of subcategories</returns>
    ''' <remarks></remarks>
    Public Function GetSubCategoryById(ByVal session As Business.ISessionDao, ByVal id As Integer) As System.Collections.Generic.IList(Of Business.DataHolders.SubCategory) Implements Business.ISubCategoryDao.GetSubCategoryById
        Try
            Dim orders As Order() = {New Order(SqlSubCategory.ID, True)}
            Dim criterions As ICriterion() = {New LikeExpression(SqlSubCategory.ID, id.ToString(), MatchMode.Exact)}
            Return m_GenericNhbDao.GetForCriteria(DirectCast(session, SqlSession).Session, criterions, orders)
        Catch ex As NHibernateException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Function

    ''' <summary>
    ''' Remove a subcategorie
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <param name="subCategory">SubCategory object to be removed</param>
    ''' <remarks></remarks>
    Public Sub Remove(ByVal session As Business.ISessionDao, ByVal subCategory As SubCategory) Implements Business.ISubCategoryDao.Remove
        Try
            m_GenericNhbDao.Delete(DirectCast(session, SqlSession).Session, subCategory)
        Catch ex As NHibernateException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Sub

    ''' <summary>
    ''' Save changes
    ''' </summary>
    ''' <param name="session">Object containing the session to the database</param>
    ''' <param name="subCategory">SubCategory object to be saved or updated</param>
    ''' <remarks></remarks>
    Public Sub Save(ByVal session As Business.ISessionDao, ByVal subCategory As Business.DataHolders.SubCategory) Implements Business.ISubCategoryDao.Save
        Try
            m_GenericNhbDao.SaveOrUpdate(DirectCast(session, SqlSession).Session, subCategory)
        Catch ex As NHibernateException
            Throw New RepositoryException(ex.Message, ex.InnerException)
        End Try
    End Sub
#End Region

End Class
