﻿Option Strict On
Option Explicit On
Option Infer Off
Imports Securex.FIT.Fitclient.Business
Imports NHibernate

Public Class SqlSession
    Implements ISessionDao
    Private m_Session As ISession
    Private m_SessionFactory As ISessionFactory
    Private m_TransAction As ITransaction

    Public Sub New(ByVal sessionFactory As ISessionFactory)
        m_SessionFactory = sessionFactory
        m_Session = m_SessionFactory.OpenSession()
        m_TransAction = m_Session.BeginTransaction()
    End Sub

    Private Sub Close()
        Try
            If (m_Session IsNot Nothing AndAlso m_Session.IsOpen AndAlso m_Session.IsConnected) Then
                m_Session.Disconnect()
                m_Session.Close()
            End If
        Catch ex As Exception
            'ignore closing error
        Finally
            m_Session = Nothing
            m_TransAction = Nothing
            m_SessionFactory = Nothing
        End Try
    End Sub

    ''' <summary>
    ''' Sql session object.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Session() As ISession
        Get
            Return m_Session
        End Get
    End Property

#Region "ISessionDao"
    ''' <summary>
    ''' Perform rollback and close the session.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub CancelAndClose() Implements Business.ISessionDao.CancelAndClose
        Try
            m_TransAction.Rollback()
            Close()
        Catch ex As Exception
            Throw New RepositoryException("Unable to rollback")
        End Try
    End Sub

    ''' <summary>
    ''' Commit changes and close the session.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub CommitAndClose() Implements Business.ISessionDao.CommitAndClose
        Try
            m_TransAction.Commit()
            Close()
        Catch ex As Exception
            Throw New RepositoryException("Unable to commit")
        End Try
    End Sub
#End Region
End Class
